function safeParseCursorConfig(config) {
  try {
    // Attempt to parse as raw JSON first
    return JSON.parse(config);
  } catch {
    try {
      // Attempt to decode and then parse
      return JSON.parse(decodeURIComponent(config));
    } catch (error) {
      console.error('Error decoding cursorConfig:', error);
      return null;
    }
  }
}

function oneko(cursorConfigs) {
  const isReducedMotion =
    window.matchMedia(`(prefers-reduced-motion: reduce)`) === true ||
    window.matchMedia(`(prefers-reduced-motion: reduce)`).matches === true;

  if (isReducedMotion) return;

  const spriteSets = {
    idle: [
      [-3, -3]
    ],
    alert: [
      [-7, -3]
    ],
    scratchSelf: [
      [-5, 0],
      [-6, 0],
      [-7, 0]
    ],
    scratchWallN: [
      [0, 0],
      [0, -1]
    ],
    scratchWallS: [
      [-7, -1],
      [-6, -2]
    ],
    scratchWallE: [
      [-2, -2],
      [-2, -3]
    ],
    scratchWallW: [
      [-4, 0],
      [-4, -1]
    ],
    tired: [
      [-3, -2]
    ],
    sleeping: [
      [-2, 0],
      [-2, -1]
    ],
    N: [
      [-1, -2],
      [-1, -3]
    ],
    NE: [
      [0, -2],
      [0, -3]
    ],
    E: [
      [-3, 0],
      [-3, -1]
    ],
    SE: [
      [-5, -1],
      [-5, -2]
    ],
    S: [
      [-6, -3],
      [-7, -2]
    ],
    SW: [
      [-5, -3],
      [-6, -1]
    ],
    W: [
      [-4, -2],
      [-4, -3]
    ],
    NW: [
      [-1, 0],
      [-1, -1]
    ]
  };

  function createNeko(container, config) {
    if (!container) {
      console.error("Invalid container for neko.");
      return null; // Or throw an error if appropriate
    }
    let nekoEl = document.createElement("div");

    let nekoPosX;
    let nekoPosY;

    if (config.placement === "entire_site" || config.placement === "entire_page") {
      nekoPosX = 50; // Set initial X position to 50px
      nekoPosY = 50; // Set initial Y position to 50px
    } else {
      nekoPosX = container.offsetWidth / 2; // Center horizontally
      nekoPosY = container.offsetHeight / 2; // Center vertically
    }



    let mousePosX = nekoPosX; // Initialize mouse position
    let mousePosY = nekoPosY;

    let frameCount = 0;
    let idleTime = 0;
    let idleAnimation = null;
    let idleAnimationFrame = 0;

    const nekoSpeed = 10;

    let animationFrameId; // Store the animation frame ID

    function init() {
      nekoEl.id = "oneko";
      nekoEl.ariaHidden = true;
      nekoEl.style.width = "32px";
      nekoEl.style.height = "32px";

      if (config.placement === "entire_site" || config.placement === "entire_page") {
        nekoEl.style.position = "fixed"; // Changed to fixed for entire_site
      } else {
        nekoEl.style.position = "absolute"; // Changed to absolute
      }

      nekoEl.style.pointerEvents = "none";
      nekoEl.style.imageRendering = "pixelated";
      nekoEl.style.left = `${nekoPosX - 16}px`;
      nekoEl.style.top = `${nekoPosY - 16}px`;
      nekoEl.style.zIndex = 2147483647;

      let nekoFile = wpccData?.pluginUrl ? wpccData.pluginUrl + "/js/oneko.gif" : "/js/oneko.gif"; // Or default if wpccData is not available.
      const curScript = document.currentScript
      if (curScript && curScript.dataset.cat) {
        nekoFile = curScript.dataset.cat
      }
      nekoEl.style.backgroundImage = `url(${nekoFile})`;

      if (config.placement !== "entire_site" && config.placement !== "entire_page") {
        container.style.position = 'relative'; //Make sure that the container is positioned to allow absolute positioning for the cat
      }


      container.appendChild(nekoEl);

      if (config.placement !== "entire_site" && config.placement !== "entire_page") {
        container.addEventListener("mousemove", handleMouseMove);
        container.addEventListener("mouseout", handleMouseOut);
        container.addEventListener("mouseleave", handleMouseOut);
      } else {
        document.addEventListener("mousemove", handleMouseMoveEntireSite);
      }


      animationFrameId = window.requestAnimationFrame(onAnimationFrame);
    }


    function destroy() {
      if (config.placement !== "entire_site" && config.placement !== "entire_page") {
        container.removeEventListener("mousemove", handleMouseMove);
        container.removeEventListener("mouseout", handleMouseOut);
        container.removeEventListener("mouseleave", handleMouseOut);
      } else {
        document.removeEventListener("mousemove", handleMouseMoveEntireSite);
      }

      if (nekoEl && nekoEl.parentNode) {
        nekoEl.parentNode.removeChild(nekoEl);
      }

      window.cancelAnimationFrame(animationFrameId); // Stop the animation
    }

    function handleMouseMove(event) {
      mousePosX = event.clientX - container.getBoundingClientRect().left;
      mousePosY = event.clientY - container.getBoundingClientRect().top;
    }

    function handleMouseMoveEntireSite(event) {
      mousePosX = event.clientX;
      mousePosY = event.clientY;
    }


    function handleMouseOut() {
      if (config.placement === "specific_section") {
        // On mouseout, smoothly move the cat back to the center
        mousePosX = container.offsetWidth / 2;
        mousePosY = container.offsetHeight / 2;
      }
    }


    let lastFrameTimestamp;

    function onAnimationFrame(timestamp) {
      // Stops execution if the neko element is removed from DOM
      if (!nekoEl.isConnected) {
        return;
      }
      if (!lastFrameTimestamp) {
        lastFrameTimestamp = timestamp;
      }
      if (timestamp - lastFrameTimestamp > 100) {
        lastFrameTimestamp = timestamp
        frame()
      }
      animationFrameId = window.requestAnimationFrame(onAnimationFrame);
    }

    function setSprite(name, frame) {
      const sprite = spriteSets[name][frame % spriteSets[name].length];
      nekoEl.style.backgroundPosition = `${sprite[0] * 32}px ${sprite[1] * 32}px`;
    }

    function resetIdleAnimation() {
      idleAnimation = null;
      idleAnimationFrame = 0;
    }

    function idle() {
      idleTime += 1;

      // every ~ 20 seconds
      if (
        idleTime > 10 &&
        Math.floor(Math.random() * 200) == 0 &&
        idleAnimation == null
      ) {
        let avalibleIdleAnimations = ["sleeping", "scratchSelf"];
        if (nekoPosX < 32) {
          avalibleIdleAnimations.push("scratchWallW");
        }
        if (nekoPosY < 32) {
          avalibleIdleAnimations.push("scratchWallN");
        }
        if (nekoPosX > container.offsetWidth - 32) {
          avalibleIdleAnimations.push("scratchWallE");
        }
        if (nekoPosY > container.offsetHeight - 32) {
          avalibleIdleAnimations.push("scratchWallS");
        }
        idleAnimation =
          avalibleIdleAnimations[
          Math.floor(Math.random() * avalibleIdleAnimations.length)
          ];
      }

      switch (idleAnimation) {
        case "sleeping":
          if (idleAnimationFrame < 8) {
            setSprite("tired", 0);
            break;
          }
          setSprite("sleeping", Math.floor(idleAnimationFrame / 4));
          if (idleAnimationFrame > 192) {
            resetIdleAnimation();
          }
          break;
        case "scratchWallN":
        case "scratchWallS":
        case "scratchWallE":
        case "scratchSelf":
          setSprite(idleAnimation, idleAnimationFrame);
          if (idleAnimationFrame > 9) {
            resetIdleAnimation();
          }
          break;
        default:
          setSprite("idle", 0);
          return;
      }
      idleAnimationFrame += 1;
    }

    function frame() {
      frameCount += 1;
      const diffX = nekoPosX - mousePosX;
      const diffY = nekoPosY - mousePosY;
      const distance = Math.sqrt(diffX ** 2 + diffY ** 2);

      if (distance < nekoSpeed || distance < 48) {
        idle();
        return;
      }

      idleAnimation = null;
      idleAnimationFrame = 0;

      if (idleTime > 1) {
        setSprite("alert", 0);
        // count down after being alerted before moving
        idleTime = Math.min(idleTime, 7);
        idleTime -= 1;
        return;
      }

      let direction;
      direction = diffY / distance > 0.5 ? "N" : "";
      direction += diffY / distance < -0.5 ? "S" : "";
      direction += diffX / distance > 0.5 ? "W" : "";
      direction += diffX / distance < -0.5 ? "E" : "";
      setSprite(direction, frameCount);

      nekoPosX -= (diffX / distance) * nekoSpeed;
      nekoPosY -= (diffY / distance) * nekoSpeed;

      if (config.placement === "entire_site" || config.placement === "entire_page") {
        // Keep within window bounds for entire_site
        nekoPosX = Math.min(Math.max(16, nekoPosX), window.innerWidth - 16);
        nekoPosY = Math.min(Math.max(16, nekoPosY), window.innerHeight - 16);
      } else {
        // Keep within container bounds for specific_section
        nekoPosX = Math.min(Math.max(16, nekoPosX), container.offsetWidth - 16);
        nekoPosY = Math.min(Math.max(16, nekoPosY), container.offsetHeight - 16);
      }


      nekoEl.style.left = `${nekoPosX - 16}px`;
      nekoEl.style.top = `${nekoPosY - 16}px`;
    }

    init();

    return {
      destroy: destroy
    };
  }

  const nekos = [];

  // Iterate through the cursor configurations
  cursorConfigs.forEach(config => {
    if (config.placement === "entire_site" || config.placement === "entire_page") {
      // Create one cat for the entire page
      const neko = createNeko(document.body, config);
      if (neko) {
        nekos.push(neko);
      }
    } else if (config.placement === "specific_section") {
      // Find all elements matching the section identifier
      const containers = document.querySelectorAll(config.section_identifier);
      containers.forEach(container => {
        const neko = createNeko(container, config);
        if (neko) {
          nekos.push(neko);
        }
      });
    }
  });


  // Optional:  Destroy all nekos when the page is unloaded.
  window.addEventListener('beforeunload', function () {
    nekos.forEach(neko => neko.destroy());
  });

}

document.addEventListener('DOMContentLoaded', () => {
  const urlParams = new URLSearchParams(window.location.search);
  const cursorConfig = urlParams.get('preview_cursor');
  let cursors = [];

  if (cursorConfig) {
    const parsedConfig = safeParseCursorConfig(cursorConfig);
    // Ensure it's an array, even if it's a single object
    cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
    // Add default placement and section_identifier for preview mode
    cursors = cursors.map(cursor => ({
      cursor: {
        id: cursor.id || null,
        name: cursor.name || 'Default Cursor',
        type: cursor.type || 'html',
        hoverSelector: cursor.hoverSelector || 'a',
        cursor: cursor.cursor || {},
        hover: cursor.hover || {}
      },
      placement: 'entire_site', // Corrected this line to match the desired behavior
      section_identifier: '',
      timestamp: Date.now()
    }));
    console.log("Preview Mode:", cursors);
  } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
    cursors = wpccData.settings;
  }

  if (Array.isArray(cursors)) {
    const aronekoConfigs = []; // Array to store Aroneko configurations
    cursors.forEach((cursor) => {
      if (cursor?.cursor?.name == "Aroneko") {
        aronekoConfigs.push(cursor);
      }
    });
    // Call oneko function *after* the loop, with the extracted configurations
    if (aronekoConfigs.length > 0) {
      oneko(aronekoConfigs);  // Pass the Aroneko configurations to the oneko function
    } else {
      console.log("No Aroneko cursors found in the configuration.");
    }
  } else {
    console.warn("No valid cursor settings found.");
  }
});