function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}

document.addEventListener('DOMContentLoaded', function () {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = [];

    // Returns the mouse position
    const getMousePos = e => {
        return {
            x: e.clientX,
            y: e.clientY
        };
    };

    // Returns the window width and height
    const getWinSize = () => {
        return {
            width: window.innerWidth,
            height: window.innerHeight
        };
    };

    const isFirefox = () => navigator.userAgent.toLowerCase().indexOf('firefox') > -1;

    // Initialize mouse position object
    let mousepos = { x: 0, y: 0 };

    // Update 'mousepos' with the current mouse position
    const updateMousePos = ev => {
        mousepos = getMousePos(ev);
    };

    // Initialize window size object
    let winsize = getWinSize();

    // Recalculate window size on 'resize' event
    const resetWinsize = () => {
        winsize = getWinSize();
    }
    // Create the cursor DOM structure dynamically
    const createCursorDOM = (style = 1) => {
        const cursorEl = document.createElement('div');
        cursorEl.classList.add('cursor');

        if (style < 1 || style > 8) {
            style = 1;
        }

        switch (style) {
            case 1:
                const innerEl = document.createElement('div');
                innerEl.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl);

                const svgEl = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl.setAttribute('version', '1.1');

                const defsEl = document.createElementNS('http://www.w3.org/2000/svg', 'defs');
                const filterEl = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl.setAttribute('id', 'gooey');

                const feGaussianBlurEl = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl.setAttribute('in', 'SourceGraphic');
                feGaussianBlurEl.setAttribute('result', 'blur');
                feGaussianBlurEl.setAttribute('stdDeviation', '3.2');

                const feColorMatrixEl = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl.setAttribute('in', 'blur');
                feColorMatrixEl.setAttribute('mode', 'matrix');
                feColorMatrixEl.setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 20 -7');
                feColorMatrixEl.setAttribute('result', 'goo');

                const feCompositeEl = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                feCompositeEl.setAttribute('in', 'SourceGraphic');
                feCompositeEl.setAttribute('in2', 'goo');
                feCompositeEl.setAttribute('operator', 'atop');

                filterEl.appendChild(feGaussianBlurEl);
                filterEl.appendChild(feColorMatrixEl);
                filterEl.appendChild(feCompositeEl);
                defsEl.appendChild(filterEl);
                svgEl.appendChild(defsEl);

                cursorEl.appendChild(svgEl);
                break;
            case 2:
                // Create the inner container
                const innerEl2 = document.createElement('div');
                innerEl2.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl2);

                // Create the SVG element
                const svgEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl2.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl2.setAttribute('version', '1.1');

                // Create the <defs> element
                const defsEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                // Create the filter
                const filterEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl2.setAttribute('id', 'gooey');

                // Add the Gaussian Blur
                const feGaussianBlurEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl2.setAttribute('in', 'SourceGraphic');
                feGaussianBlurEl2.setAttribute('result', 'blur');
                feGaussianBlurEl2.setAttribute('stdDeviation', '18');
                filterEl2.appendChild(feGaussianBlurEl2);

                // Add the Color Matrix
                const feColorMatrixEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl2.setAttribute('in', 'blur');
                feColorMatrixEl2.setAttribute('mode', 'matrix');
                feColorMatrixEl2.setAttribute(
                    'values',
                    '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 20 -7'
                );
                feColorMatrixEl2.setAttribute('result', 'goo');
                filterEl2.appendChild(feColorMatrixEl2);

                // Add the Composite Filter
                const feCompositeEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                feCompositeEl2.setAttribute('in', 'SourceGraphic');
                feCompositeEl2.setAttribute('in2', 'goo');
                feCompositeEl2.setAttribute('operator', 'atop');
                filterEl2.appendChild(feCompositeEl2);

                // Append the filter to <defs>
                defsEl2.appendChild(filterEl2);

                // Append <defs> to the SVG
                svgEl2.appendChild(defsEl2);

                // Append the SVG to the cursor element
                cursorEl.appendChild(svgEl2);
                break;
            case 3:
                // Create the inner container
                const innerEl3 = document.createElement('div');
                innerEl3.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl3);

                // Create the SVG element
                const svgEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl3.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl3.setAttribute('version', '1.1');

                // Create the <defs> element
                const defsEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                // Create the filter
                const filterEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl3.setAttribute('id', 'gooey');

                // Add feTurbulence
                const feTurbulenceEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feTurbulence');
                feTurbulenceEl3.setAttribute('type', 'fractalNoise');
                feTurbulenceEl3.setAttribute('baseFrequency', '0.02');
                feTurbulenceEl3.setAttribute('numOctaves', '3');
                feTurbulenceEl3.setAttribute('result', 'turbulence');
                filterEl3.appendChild(feTurbulenceEl3);

                // Add feDisplacementMap
                const feDisplacementMapEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feDisplacementMap');
                feDisplacementMapEl3.setAttribute('in', 'SourceGraphic');
                feDisplacementMapEl3.setAttribute('in2', 'turbulence');
                feDisplacementMapEl3.setAttribute('scale', '30');
                feDisplacementMapEl3.setAttribute('xChannelSelector', 'R');
                feDisplacementMapEl3.setAttribute('yChannelSelector', 'G');
                feDisplacementMapEl3.setAttribute('result', 'displacement');
                filterEl3.appendChild(feDisplacementMapEl3);

                // Add feGaussianBlur
                const feGaussianBlurEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl3.setAttribute('in', 'displacement');
                feGaussianBlurEl3.setAttribute('stdDeviation', '10');
                feGaussianBlurEl3.setAttribute('result', 'blur');
                filterEl3.appendChild(feGaussianBlurEl3);

                // Add feColorMatrix
                const feColorMatrixEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl3.setAttribute('in', 'blur');
                feColorMatrixEl3.setAttribute('mode', 'matrix');
                feColorMatrixEl3.setAttribute(
                    'values',
                    `
                    1 0 0 0 0
                    0 1 0 0 0
                    0 0 1 0 0
                    0 0 0 20 -7`
                );
                feColorMatrixEl3.setAttribute('result', 'gooey');
                filterEl3.appendChild(feColorMatrixEl3);

                // Append the filter to <defs>
                defsEl3.appendChild(filterEl3);

                // Append <defs> to the SVG
                svgEl3.appendChild(defsEl3);

                // Append the SVG to the cursor element
                cursorEl.appendChild(svgEl3);
                break;
            case 4:
                // Create the inner container
                const innerEl4 = document.createElement('div');
                innerEl4.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl4);

                // Create the SVG element
                const svgEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl4.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl4.setAttribute('version', '1.1');

                // Create the <defs> element
                const defsEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                // Create the filter element
                const filterEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl4.setAttribute('id', 'gooey');

                // Add feGaussianBlur
                const feGaussianBlurEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl4.setAttribute('in', 'SourceGraphic');
                feGaussianBlurEl4.setAttribute('stdDeviation', '10');
                feGaussianBlurEl4.setAttribute('result', 'blur');
                filterEl4.appendChild(feGaussianBlurEl4);

                // Add feColorMatrix
                const feColorMatrixEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl4.setAttribute('in', 'blur');
                feColorMatrixEl4.setAttribute('mode', 'matrix');
                feColorMatrixEl4.setAttribute(
                    'values',
                    `
                    1 0 0 0 0
                    0 1 0 0 0
                    0 0 1 0 0
                    0 0 0 20 -7`
                );
                feColorMatrixEl4.setAttribute('result', 'gooey');
                filterEl4.appendChild(feColorMatrixEl4);

                // Add feMorphology
                const feMorphologyEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feMorphology');
                feMorphologyEl4.setAttribute('in', 'gooey');
                feMorphologyEl4.setAttribute('operator', 'dilate');
                feMorphologyEl4.setAttribute('radius', '2');
                feMorphologyEl4.setAttribute('result', 'outline');
                filterEl4.appendChild(feMorphologyEl4);

                // Add feComposite
                const feCompositeEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                feCompositeEl4.setAttribute('in', 'outline');
                feCompositeEl4.setAttribute('in2', 'gooey');
                feCompositeEl4.setAttribute('operator', 'out');
                feCompositeEl4.setAttribute('result', 'composite');
                filterEl4.appendChild(feCompositeEl4);

                // Append the filter to <defs>
                defsEl4.appendChild(filterEl4);

                // Append <defs> to the SVG
                svgEl4.appendChild(defsEl4);

                // Append the SVG to the cursor element
                cursorEl.appendChild(svgEl4);
                break;
            case 5:
                // Create the inner container
                const innerEl5 = document.createElement('div');
                innerEl5.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl5);

                // Create the SVG element
                const svgEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl5.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl5.setAttribute('version', '1.1');

                // Create the <defs> element
                const defsEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                // Create the filter element
                const filterEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl5.setAttribute('id', 'gooey');

                // Add feTurbulence
                const feTurbulenceEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feTurbulence');
                feTurbulenceEl5.setAttribute('type', 'fractalNoise');
                feTurbulenceEl5.setAttribute('baseFrequency', '0.1');
                feTurbulenceEl5.setAttribute('numOctaves', '1');
                feTurbulenceEl5.setAttribute('seed', '0');
                feTurbulenceEl5.setAttribute('result', 'turbulence');
                filterEl5.appendChild(feTurbulenceEl5);

                // Add feDisplacementMap
                const feDisplacementMapEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feDisplacementMap');
                feDisplacementMapEl5.setAttribute('in', 'SourceGraphic');
                feDisplacementMapEl5.setAttribute('in2', 'turbulence');
                feDisplacementMapEl5.setAttribute('scale', '100');
                feDisplacementMapEl5.setAttribute('xChannelSelector', 'R');
                feDisplacementMapEl5.setAttribute('yChannelSelector', 'G');
                feDisplacementMapEl5.setAttribute('result', 'displacement');
                filterEl5.appendChild(feDisplacementMapEl5);

                // Add feGaussianBlur
                const feGaussianBlurEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl5.setAttribute('in', 'displacement');
                feGaussianBlurEl5.setAttribute('stdDeviation', '5');
                feGaussianBlurEl5.setAttribute('result', 'blur');
                filterEl5.appendChild(feGaussianBlurEl5);

                // Add feColorMatrix
                const feColorMatrixEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl5.setAttribute('in', 'blur');
                feColorMatrixEl5.setAttribute('mode', 'matrix');
                feColorMatrixEl5.setAttribute(
                    'values',
                    `
                    1 0 0 0 0
                    0 1 0 0 0
                    0 0 1 0 0
                    0 0 0 15 -7`
                );
                feColorMatrixEl5.setAttribute('result', 'gooey');
                filterEl5.appendChild(feColorMatrixEl5);

                // Append the filter to <defs>
                defsEl5.appendChild(filterEl5);

                // Append <defs> to the SVG
                svgEl5.appendChild(defsEl5);

                // Append the SVG to the cursor element
                cursorEl.appendChild(svgEl5);
                break;
            case 6:
                // Create the inner container
                const innerEl6 = document.createElement('div');
                innerEl6.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl6);

                // Create the SVG element
                const svgEl6 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl6.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl6.setAttribute('version', '1.1');

                // Create the <defs> element
                const defsEl6 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                // Create the filter element
                const filterEl6 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl6.setAttribute('id', 'gooey');

                // Add the first feGaussianBlur
                const feGaussianBlurEl61 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl61.setAttribute('in', 'SourceGraphic');
                feGaussianBlurEl61.setAttribute('result', 'blur');
                feGaussianBlurEl61.setAttribute('stdDeviation', '12');
                filterEl6.appendChild(feGaussianBlurEl61);

                // Add the first feColorMatrix
                const feColorMatrixEl61 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl61.setAttribute('in', 'blur');
                feColorMatrixEl61.setAttribute('mode', 'matrix');
                feColorMatrixEl61.setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 18 -7');
                feColorMatrixEl61.setAttribute('result', 'goo');
                filterEl6.appendChild(feColorMatrixEl61);

                // Add the second feGaussianBlur
                const feGaussianBlurEl62 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl62.setAttribute('in', 'goo');
                feGaussianBlurEl62.setAttribute('stdDeviation', '9');
                feGaussianBlurEl62.setAttribute('result', 'shadow');
                filterEl6.appendChild(feGaussianBlurEl62);

                // Add the second feColorMatrix
                const feColorMatrixEl62 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl62.setAttribute('in', 'shadow');
                feColorMatrixEl62.setAttribute('mode', 'matrix');
                feColorMatrixEl62.setAttribute('values', '0 0 0 0 0  0 0 0 0 0  0 0 0 0 0  0 0 0 8 -.2');
                feColorMatrixEl62.setAttribute('result', 'shadow');
                filterEl6.appendChild(feColorMatrixEl62);

                // Add the first feComposite
                const feCompositeEl61 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                feCompositeEl61.setAttribute('in2', 'shadow');
                feCompositeEl61.setAttribute('in', 'goo');
                feCompositeEl61.setAttribute('result', 'goo');
                filterEl6.appendChild(feCompositeEl61);

                // Add the second feComposite
                const feCompositeEl62 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                feCompositeEl62.setAttribute('in2', 'goo');
                feCompositeEl62.setAttribute('in', 'SourceGraphic');
                feCompositeEl62.setAttribute('result', 'mix');
                filterEl6.appendChild(feCompositeEl62);

                // Append the filter to <defs>
                defsEl6.appendChild(filterEl6);

                // Append <defs> to the SVG
                svgEl6.appendChild(defsEl6);

                // Append the SVG to the cursor element
                cursorEl.appendChild(svgEl6);
                break;
            case 7:
                // Create the inner container
                const innerEl7 = document.createElement('div');
                innerEl7.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl7);
                break;
            case 8:
                // Create the inner container
                const innerEl8 = document.createElement('div');
                innerEl8.classList.add('cursor__inner');
                cursorEl.appendChild(innerEl8);

                // Create the SVG element
                const svgEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svgEl8.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                svgEl8.setAttribute('version', '1.1');

                // Create the <defs> element
                const defsEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                // Create the filter element
                const filterEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                filterEl8.setAttribute('id', 'gooey');

                // Add the feGaussianBlur
                const feGaussianBlurEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                feGaussianBlurEl8.setAttribute('in', 'SourceGraphic');
                feGaussianBlurEl8.setAttribute('result', 'blur');
                feGaussianBlurEl8.setAttribute('stdDeviation', '3.2');
                filterEl8.appendChild(feGaussianBlurEl8);

                // Add the feColorMatrix
                const feColorMatrixEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                feColorMatrixEl8.setAttribute('in', 'blur');
                feColorMatrixEl8.setAttribute('mode', 'matrix');
                feColorMatrixEl8.setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 20 -7');
                feColorMatrixEl8.setAttribute('result', 'goo');
                filterEl8.appendChild(feColorMatrixEl8);

                // Add the feComposite
                const feCompositeEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                feCompositeEl8.setAttribute('in', 'SourceGraphic');
                feCompositeEl8.setAttribute('in2', 'goo');
                feCompositeEl8.setAttribute('operator', 'atop');
                filterEl8.appendChild(feCompositeEl8);

                // Append the filter to <defs>
                defsEl8.appendChild(filterEl8);

                // Append <defs> to the SVG
                svgEl8.appendChild(defsEl8);

                // Append the SVG to the cursor element
                cursorEl.appendChild(svgEl8);
                break;
            default:
                console.error(`Unknown cursor style: ${style}`);
                break;
        }

        return cursorEl;
    };

    const applyCSSVariables = (el, cssConfig) => {
        if (!cssConfig) return;

        Object.entries(cssConfig).forEach(([key, value]) => {
            // Convert camelCase to kebab-case for CSS variables
            const cssVar = `--${key.replace(/([A-Z])/g, '-$1').toLowerCase()}`;
            el.style.setProperty(cssVar, value);
        });
    };


    // Create and initialize the Goo cursor
    class GooCursor {
        DOM = {
            el: null,
            inner: null,
            cells: null,
            container: null // Section container
        };
        cellSize;
        rows;
        columns;
        settings = {
            ttl: 0.2
        };
        cachedCell = null;

        constructor(DOM_el, container) {
            this.DOM.el = DOM_el;
            this.DOM.inner = this.DOM.el.querySelector('.cursor__inner');
            this.DOM.container = container;


            if (!isFirefox()) {
                this.DOM.inner.style.filter = 'url(#gooey)';
            }

            this.settings.ttl = this.DOM.el.getAttribute('data-ttl') || this.settings.ttl;

            this.layout();
            this.initEvents();
        }

        initEvents() {
            window.addEventListener('resize', () => this.layout());

            const handleMove = (ev) => {
                if (this.DOM.container) {
                    const containerRect = this.DOM.container.getBoundingClientRect();
                    // Check if the mouse is within the section
                    if (
                        ev.clientX >= containerRect.left &&
                        ev.clientX <= containerRect.right &&
                        ev.clientY >= containerRect.top &&
                        ev.clientY <= containerRect.bottom
                    ) {
                        updateMousePos(ev); // Update mousepos only if inside the container
                        const cell = this.getCellAtCursor();

                        if (cell === null || this.cachedCell === cell) return;
                        this.cachedCell = cell;
                        cell.style.opacity = '1';
                        setTimeout(() => {
                            cell.style.opacity = '0';
                        }, this.settings.ttl * 1000);
                        this.DOM.el.style.display = 'block';
                    }
                    else {
                        this.DOM.el.style.display = 'none';
                    }

                }
                else {
                    updateMousePos(ev); // Update mousepos only if inside the container
                    const cell = this.getCellAtCursor();

                    if (cell === null || this.cachedCell === cell) return;
                    this.cachedCell = cell;
                    cell.style.opacity = '1';
                    setTimeout(() => {
                        cell.style.opacity = '0';
                    }, this.settings.ttl * 1000);
                    this.DOM.el.style.display = 'block';
                }
            };

            window.addEventListener('mousemove', handleMove);
            window.addEventListener('pointermove', handleMove, { passive: true });
        }

        layout() {
            resetWinsize();
            this.columns = getComputedStyle(this.DOM.el).getPropertyValue('--columns');
            this.cellSize = winsize.width / this.columns;
            this.rows = Math.ceil(winsize.height / this.cellSize);
            this.cellsTotal = this.rows * this.columns;

            let innerStr = '';
            this.DOM.inner.innerHTML = '';

            const customColorsAttr = this.DOM.el.getAttribute('data-custom-colors');
            let customColorsArr;
            let customColorsTotal = 0;

            if (customColorsAttr) {
                customColorsArr = this.DOM.el.getAttribute('data-custom-colors').split(',');
                customColorsTotal = customColorsArr ? customColorsArr.length : 0;
            }

            for (let i = 0; i < this.cellsTotal; ++i) {
                innerStr += customColorsTotal === 0
                    ? '<div class="cursor__inner-box"></div>'
                    : `<div style="transform: scale(${Math.random() * (2 - 0.5) + 0.5}); 
                        background:${customColorsArr[Math.round(Math.random() * (customColorsTotal - 1))]}" 
                    class="cursor__inner-box"></div>`;
            }
            this.DOM.inner.innerHTML = innerStr;
            this.DOM.cells = this.DOM.inner.children;
        }

        getCellAtCursor() {
            const columnIndex = Math.floor(mousepos.x / this.cellSize);
            const rowIndex = Math.floor(mousepos.y / this.cellSize);
            const cellIndex = rowIndex * this.columns + columnIndex;

            if (cellIndex >= this.cellsTotal || cellIndex < 0) {
                console.error('Cell index out of bounds');
                return null;
            }

            return this.DOM.cells[cellIndex];
        }
    }



    const gooCursorEffect = (demetrobyteConfigs) => {
        // Listen for 'mousemove' and 'pointermove' events and update 'mousepos' accordingly
        window.addEventListener('mousemove', updateMousePos);
        window.addEventListener('pointermove', updateMousePos, { passive: true });
        window.addEventListener('resize', () => {
            resetWinsize();
        });
        demetrobyteConfigs.forEach(config => {
            let cursorEl;
            let container = null;

            const { placement, section_identifier, cursor } = config;
            const { config: { style = 1, ...cssConfig } } = cursor.cursor;

            cursorEl = createCursorDOM(Number(style));
            if (cssConfig) {
                applyCSSVariables(cursorEl, cssConfig);
            }

            if (placement === 'specific_section') {
                container = document.querySelector(section_identifier);
                if (!container) {
                    console.warn(`Section with identifier "${section_identifier}" not found.`);
                    document.body.appendChild(cursorEl); // Append to body as fallback
                } else {
                    container.appendChild(cursorEl); // Append to specified section
                }
            } else {
                document.body.appendChild(cursorEl); // Default to entire site
            }
            let goo;
            // Initialize cursor
            if (placement === 'specific_section' && container) {
                goo = new GooCursor(cursorEl, container);
            } else {
                goo = new GooCursor(cursorEl, null);
            }
        });
    }

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor', 
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
    }

    if (Array.isArray(cursors)) {
        const demetrobyteConfigs = [];
        cursors.forEach((cursor) => {
            if (cursor?.cursor?.name == "Demetrobyte") {
                demetrobyteConfigs.push(cursor);
            }
        });
        if (demetrobyteConfigs.length > 0) {
            gooCursorEffect(demetrobyteConfigs); // Pass the Demetrobyte configurations 
        } else {
            console.log("No Demetrobyte cursors found in the configuration.");
        }
    }

    
});