import React from '@wordpress/element';
// import React from 'react';
import clsx from 'clsx'; // Optional for better class handling (npm install clsx)
import { MdArrowOutward } from 'react-icons/md';

interface CardProps {
  icon: React.ReactNode; // Icon component
  heading: string; // Card heading
  description: string; // Card description
  linkTitle: string; // Text for the link
  linkUrl: string; // URL for the link
  className?: string; // Optional additional class names
}

const Card: React.FC<CardProps> = ({ 
  icon, 
  heading, 
  description, 
  linkTitle, 
  linkUrl, 
  className 
}) => {
  return (
    <div className={clsx("bg-neutral-200 rounded-lg p-6 flex flex-col items-start", className)}>
      <div className="flex items-center justify-center gap-2 mb-4">
        {/* Icon */}
            <div className="text-lg text-white bg-[#ff4800] rounded-full w-9 h-9 flex items-center justify-center">{icon}</div>

        {/* Heading */}
        <h3 className="text-lg">{heading}</h3>
      </div>

      {/* Description */}
      <p className="text-gray-600 mb-4">{description}</p>

      {/* Link */}
      <a
        href={linkUrl}
        className="text-blue-500 font-medium hover:underline flex gap-1 items-center"
        target="_blank"
        rel="noopener noreferrer"
      >
        {linkTitle}
        <MdArrowOutward />
      </a>
      
    </div>
  );
};

export default Card;