import React from '@wordpress/element';
// import React from 'react';
import { useState, useRef, useEffect } from '@wordpress/element';
import { Icon, close } from '@wordpress/icons';
import CursorItem from '../components/CursorItem';
import { BsThreeDotsVertical } from 'react-icons/bs';
import { MdClose } from "react-icons/md";
import axios from 'axios';






type BlendMode = 'normal' | 'multiply' | 'screen' | 'overlay' | 'darken' | 'lighten' | 'color-dodge' | 'color-burn' | 'hard-light' | 'soft-light' | 'difference' | 'exclusion' | 'hue' | 'saturation' | 'color' | 'luminosity';
type BorderStyle = 'solid' | 'dashed' | 'dotted' | 'double' | 'groove' | 'ridge' | 'inset' | 'outset' | 'none' | 'hidden';
type TransitionTimingFunction = 'ease' | 'linear' | 'ease-in' | 'ease-out' | 'ease-in-out' | 'step-start' | 'step-end';
type SpinDirection = 'right' | 'left';

// Define types for shape-related properties
interface ShapeProperties {
    visible?: boolean;
    fill?: string;
    opacity?: number;
    radius?: number;
    type?: number;
    scale?: number;
    blendingMode?: BlendMode;
    spin?: boolean;
    spinDirection?: SpinDirection;
    spinDuration?: string;
    length?: number;
    color?:string
}

// Define the base CursorProperties interface with common properties
interface BaseCursorProperties {
    width?: string | number;
    height?: string | number;
    top?: string | number;
    left?: string | number;
    padding?: string | number;
    backgroundColor?: string;
    borderColor?: string;
    borderWidth?: number;
    borderStyle?: BorderStyle;
    borderRadius?: string;
    opacity?: number;
    blendingMode?: BlendMode;
    boxShadow?: string;
    transitionDuration?: string;
    transitionTimingFunction?: TransitionTimingFunction;
    zIndex?: string;
    mixBlendMode?: BlendMode;
    filter?: string;
    transition?: string;
    showDefaultCursor?: boolean;
    hoverSelector?: string;
    snap?: boolean; // Add snap property
    text?: string;
    textSize?: string;
    textWeight?: number;
    textColor?: string;
    activePoint?: { x: number; y: number };
    columns?: number;
    style?:number
    imageUrls?:string
    threshold?:number
    strokeWidth?:number
    strokeColor?:string
    strokeRadius?:number
}

// Define interface for dynamic values for canvas object
interface CanvasConfig{
    simResolution: number;
    dyeResolution: number;
    captureResolution: number;
    densityDissipation: number;
    velocityDissipation: number;
    pressure: number;
    pressureIterations: number;
    curl: number;
    splatRadius: number;
    splatForce: number;
    shading: boolean;
    colorful: boolean;
    colorUpdateSpeed: number;
    paused: boolean;
    backColor: { r: number, g: number, b: number };
    transparent: boolean;
    blendingMode: BlendMode;
    bloom: boolean;
    bloomIterations: number;
    bloomResolution: number;
    bloomIntensity: number;
    bloomThreshold: number;
    bloomSoftKnee: number;
    sunrays: boolean;
    sunraysResolution: number;
    sunraysWeight: number;
}

// Combines object with a partial BaseCursorProperties (aka, there is an "options" object)
interface optionsCursorProperties extends Partial<BaseCursorProperties>
{
    options?:Partial<BaseCursorProperties>
}

// Define properties that are assigned dynamic with dynamic properties.
interface DynamicCursorProperties extends optionsCursorProperties{
    svg?:Partial<optionsCursorProperties>
    image?:Partial<optionsCursorProperties>
    path?:Partial<optionsCursorProperties>
    centerShape?:ShapeProperties,
    middleShape?:ShapeProperties
    outerShape?:ShapeProperties
    text?:ShapeProperties
    svgShape?:Partial<optionsCursorProperties>
    config?: CanvasConfig
}

// Combines the above to one interface
export type CursorProperties =optionsCursorProperties | DynamicCursorProperties

export interface CursorData {
    id: number;
    name: string;
    type: string;
    cursor?: {
        inner?: CursorProperties;
        outer?: CursorProperties;
        options?: CursorProperties;
        config?: CursorProperties
        centerShape?: ShapeProperties
        middleShape?: ShapeProperties
        outerShape?: ShapeProperties
        text?: ShapeProperties
        image?: Partial<optionsCursorProperties>;
        svg?: Partial<optionsCursorProperties>;
        shape1?: ShapeProperties
        shape2?: ShapeProperties
        shape3?: ShapeProperties
    };
    hover?: {
        inner?: CursorProperties;
        outer?: CursorProperties;
        options?: CursorProperties;
        config?: CursorProperties
        centerShape?: ShapeProperties
        middleShape?: ShapeProperties
        outerShape?: ShapeProperties
        text?: ShapeProperties
        image?: Partial<optionsCursorProperties>;
        svg?: Partial<optionsCursorProperties>;
        shape1?: ShapeProperties
        shape2?: ShapeProperties
        shape3?: ShapeProperties
    };
    image?: string;
    placement?: string;
    section_identifier?: string | null;
}

interface FormValues {
    cursor?: {
        outer?: CursorProperties;
        inner?: CursorProperties;
    };
    hover?: {
        outer?: CursorProperties;
        inner?: CursorProperties;
    };
}

const CursorList = ({ pluginUrl, siteUrl, restUrl }) => {
    const CURSORS = [
        {
            id: 1,
            name: 'Zephricon',
            type: 'html',
            cursor: {
                outer: { width: '30px', height: '30px', borderWidth: 1, borderColor: '#000000', borderStyle: 'solid', opacity: 1, borderRadius: '50%', blendingMode: 'normal', boxShadow:'none', backgroundColor: 'transparent', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                inner: { width: '4px', height: '4px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 1, borderRadius: '50%', transitionDuration: '0', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: false}
            },
            hover: {
                outer: { width: '30px', height: '30px', borderWidth: 1, borderColor: '#000000', borderStyle: 'solid', opacity: 1, borderRadius: '50%', blendingMode: 'normal', boxShadow: 'none', backgroundColor: 'transparent', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99', snap: false, },
                inner: { width: '20px', height: '20px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 1, borderRadius: '50%', transitionDuration: '0', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 2,
            name: 'Athentri',
            type: 'html',
            cursor: {
                outer: { width: '20px', height: '20px', backgroundColor: '#ffffff', blendingMode: 'difference', opacity: 1, borderRadius: '50%',  transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true}
            },
            hover: {
                outer: { width: '50px', height: '50px', backgroundColor: '#ffffff', blendingMode: 'difference', opacity: 1, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true, hoverSelector: 'a'}
            },
        },
        {
            id: 3,
            name: 'Posquareon',
            type: 'html',
            cursor: {
                outer: { width: '30px', height: '30px', opacity: 0.2, borderRadius: '50%', backgroundColor: '#000000', blendingMode: 'normal', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                inner: { width: '4px', height: '4px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 1, borderRadius: '50%', zIndex: '99' },
                options: {showDefaultCursor: false}
            },
            hover: {
                outer: { width: '60px', height: '60px', opacity: 0.2, borderRadius: '50%', backgroundColor: '#000000', blendingMode: 'normal', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                inner: { width: '4px', height: '4px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 1, borderRadius: '50%', zIndex: '99' },
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 4,
            name: 'Apollonix',
            type: 'html',
            cursor: {
                inner: { width: '20px', height: '20px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 1, borderRadius: '50%', zIndex: '99' },
                outer: { width: '20px', height: '20px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 0.2, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true}
            },
            hover: {
                inner: { width: '5px', height: '5px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 1, borderRadius: '50%', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                outer: { width: '40px', height: '40px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 0.2, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true, hoverSelector: 'a'}
            },
        },
        {
            id: 5,
            name: 'Hadexagon',
            type: 'html',
            cursor: {
                outer: { width: '30px', height: '30px', borderColor: '#000000', borderWidth: 1, borderStyle: 'solid', blendingMode: 'normal', opacity: 1, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true}
            },
            hover: {
                outer: { width: '60px', height: '60px', borderColor: '#000000', borderWidth: 1, borderStyle: 'solid', blendingMode: 'normal', opacity: 1, borderRadius: '50%', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true, hoverSelector: 'a'}
            },
        },
        {
            id: 6,
            name: 'Hermorbis',
            type: 'html',
            cursor: {
                inner: { width: '30px', height: '30px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 0.2, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                outer: { width: '30px', height: '30px', borderColor: '#000000', borderWidth: 1, borderStyle: 'solid', blendingMode: 'normal', borderRadius: '50%', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                options: {showDefaultCursor: true}
            },
            hover: {
                inner: { width: '50px', height: '50px', backgroundColor: '#000000', blendingMode: 'normal', opacity: 0.2, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out', zIndex: '99' },
                outer: { width: '10px', height: '10px', borderColor: '#000000', borderWidth: 1, borderStyle: 'solid', blendingMode: 'normal', borderRadius: '50%', zIndex: '99' },
                options: {showDefaultCursor: true, hoverSelector: 'a'}
            },
        },
        {
            id: 7,
            name: 'Dionytris',
            type: 'svg',
            cursor: {
                centerShape: { visible: true, fill: '#ffffff', opacity: 1, radius: 5, type: 1 },
                middleShape: { visible: true, scale: 0.5, fill: 'red', opacity: 1, blendingMode: 'normal', type: 1, spin: true, spinDirection: 'right', spinDuration: '12s' },
                outerShape: { visible: true, radius: 100, fill: '#000000', opacity: 1, blendingMode: 'normal' },
                text: { visible: true, content: "Circular Text Example . Circular Text Example .", fontSize: 20, fontWeight: 900, fill: '#ffffff', opacity: 1, spin: true, spinDirection: 'left', spinDuration: '12s'  },
                svgShape: {width: 100, zIndex: '99',  transition: "all 50ms ease-out" },
                options: {showDefaultCursor: true}
            },
            hover: {
                centerShape: { visible: true, fill: '#ff0000', opacity: 1, radius: 4, type: 2 },
                middleShape: { visible: true, scale: 0.5, fill: '#0000ff', opacity: 1, blendingMode: 'normal', type: 2, spin: true, spinDirection: 'left', spinDuration: '12s' },
                outerShape: { visible: true, radius: 100, fill: '#000000', opacity: 1, blendingMode: 'normal' },
                text: { visible: true, content: "this is the hover cursor", fontSize: 20, fontWeight: 400, fill: '#00ff00', opacity: 1, spin: true, spinDirection: 'right', spinDuration: '12s'  },
                svgShape: {width: 100,  transition: "all 200ms ease-out" },
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 8,
            name: 'Aroneko',
            type: 'html',
            cursor: {},
        },
        {
            id: 9,
            name: 'Cronovoid',
            type: 'html',
            cursor: {
                inner: { width: '5px', height: '5px', backgroundColor: '#ffb703', opacity: 1, borderRadius: '50%', transitionDuration: '0s', transitionTimingFunction: 'ease-out' },
                outer: { width: 'auto', height: 'auto', top: '5px', left: '5px', padding: '3px 5px', backgroundColor: '#222222', text: 'Cursor', textSize: '14px', textWeight: 400, textColor: '#ffffff', borderColor: '#222222', borderWidth: 1, borderStyle: 'solid', borderRadius: '5px', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', zIndex: '99', opacity: 1 },
                options: {showDefaultCursor: false}
            },
            hover: {
                inner: { width: '20px', height: '20px', backgroundColor: '#000000', opacity: 0.2, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out' },
                outer: { width: 'auto', height: 'auto', top: '5px', left: '5px', padding: '3px 5px', backgroundColor: 'transparent', text: 'View', textSize: '14px', textWeight: 400, textColor: '#222222', borderColor: '#222222', borderWidth: 1, borderStyle: 'solid', borderRadius: '5px', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', zIndex: '99', opacity: 1 },
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 10,
            name: 'Hephamage',
            type: 'image',
            cursor: {
                config: {
                    imageUrls: `${pluginUrl}/public/img/1.png, ${pluginUrl}/public/img/2.png, ${pluginUrl}/public/img/3.png, ${pluginUrl}/public/img/4.png, ${pluginUrl}/public/img/5.png, ${pluginUrl}/public/img/6.png, ${pluginUrl}/public/img/7.png, ${pluginUrl}/public/img/8.png, ${pluginUrl}/public/img/9.png, ${pluginUrl}/public/img/10.png, ${pluginUrl}/public/img/11.png, ${pluginUrl}/public/img/12.png, ${pluginUrl}/public/img/13.png, ${pluginUrl}/public/img/14.png, ${pluginUrl}/public/img/15.png`,
                    threshold: 80,
                    width: '100px',
                    filter: 'none',
                    blendingMode: 'normal',
                    zIndex: '99',
                    style: 3
                }
            },
        },
        {
            id: 11,
            name: 'Artemiscle',
            type: 'html',
            cursor: {
                inner: { width: '20px', height: '20px', backgroundColor: 'blue', opacity: 1, borderRadius: '50%', boxShadow: '0 0 120px 30px', color: 'blue', blendingMode: 'normal', transitionDuration: '0s', transitionTimingFunction: 'ease-out' },
                outer: { width: '100px', height: '100px', backgroundColor: 'blue', boxShadow: '0 0 170px 100px', text: 'View', textSize: '14px', textWeight: 500, textColor: 'white', color: 'blue', blendingMode: 'normal', borderColor: 'transparent', borderWidth: 0, borderStyle: 'solid', borderRadius: '50%', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', zIndex: '99', opacity: 0 },
                options: {showDefaultCursor: false}
            },
            hover: {
                inner: { width: '20px', height: '20px', backgroundColor: '#000000', opacity: 0, borderRadius: '50%', transitionDuration: '0.2s', transitionTimingFunction: 'ease-out' },
                outer: { opacity: 1 },
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 12,
            name: 'Hestellipse',
            type: 'svg',
            cursor: {
                svg: {
                    viewBox: '0 0 1 1' 
                },
                path: {
                    fill: 'none',
                    stroke: '#ff0000',
                    strokeWidth: '4px',
                    strokeLinecap: 'round',
                    strokeLinejoin: 'round',
                }
            },
        },
        {
            id: 13,
            name: 'Demetrobyte',
            type: 'svg',
            cursor: {
                config: {
                    columns: 30,
                    borderRadius: '0px',
                    backgroundColor: 'red',
                    blendingMode: 'normal',
                    style: 1
                }
            },
        },
        {
            id: 14,
            name: 'Nyxamoke',
            type: 'canvas',
            cursor: {
                config: {
                    simResolution: 128,
                    dyeResolution: 1024,
                    captureResolution: 512,
                    densityDissipation: 3.5,
                    velocityDissipation: 1.5,
                    pressure: 0.4,
                    pressureIterations: 20,
                    curl: 15,
                    splatRadius: 0.25,
                    splatForce: 6000,
                    shading: true,
                    colorful: true,
                    colorUpdateSpeed: 10,
                    paused: false,
                    backColor: { r: 0, g: 0, b: 0 },
                    transparent: false,
                    blendingMode: 'difference',
                    bloom: false,
                    bloomIterations: 8,
                    bloomResolution: 256,
                    bloomIntensity: 0.8,
                    bloomThreshold: 0.6,
                    bloomSoftKnee: 0.7,
                    sunrays: false,
                    sunraysResolution: 196,
                    sunraysWeight: 1.0
                }
            },
        },
        {
            id: 15,
            name: 'Gaialoop',
            cursor: {
                shape1: {color: 'blue', length: 30},
                shape2: {color: 'yellow', length: 20},
                shape3: {color: 'red', length: 10},
                config: {ease: 0.7, width: 40},
                options: {showDefaultCursor: false}
            },
            hover: {
                config: {width: 10, strokeWidth: 2, strokeColor: 'red', strokeRadius: 30},
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 16,
            name: 'Aphorbit',
            type: 'image',
            cursor: {
                image: { width: '50px', imageUrl:`${pluginUrl}/public/img/cursor.png`, backgroundColor: 'transparent', opacity: 1, borderRadius: '0', mixBlendMode: 'normal', padding: 0, transitionDuration: '0.05s', transitionTimingFunction: 'ease-out', activePoint: { x:5, y: 5 } },
                options: {showDefaultCursor: false}
            },
            hover: {
                image: { width: '50px', imageUrl:`${pluginUrl}/public/img/hover.png`, backgroundColor: 'transparent', opacity: 1, borderRadius: '0', mixBlendMode: 'normal', padding: 0, transitionDuration: '0.05s', transitionTimingFunction: 'ease-out', activePoint: { x: 35, y: 5 } },
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
        {
            id: 17,
            name: 'Persephont',
            type: 'text',
            cursor: {
                inner: { width: '100px', height: '100px', backdropFilter:'blur(3px)', backgroundColor: 'rgba(0,0,0,0.1)', borderColor: 'transparent', borderWidth: 0, borderStyle: 'solid', borderRadius: '50%', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', opacity: 1 },
                outer: { width: 'auto', height: 'auto', text: 'Cursor', fontSize: '14px', fontWeight: 400, color: 'white', borderColor: 'transparent', borderWidth: 0, borderStyle: 'solid', borderRadius: '0', backgroundColor: 'transparent', opacity: 1, transitionDuration: '0.1s', transitionTimingFunction: 'ease-out'},
                options: {showDefaultCursor: false, zIndex: '99', mixBlendMode: 'normal'}
            },
            hover: {
                inner: { width: '130px', height: '130px', backdropFilter:'blur(3px)', backgroundColor: 'rgba(255,255,255,0.1)', borderColor: 'transparent', borderWidth: 0, borderStyle: 'solid', borderRadius: '50%', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out', opacity: 1 },
                outer: { width: 'auto', height: 'auto', text: 'Hover', fontSize: '14px', fontWeight: 400, color: 'black', borderColor: 'transparent', borderWidth: 0, borderStyle: 'solid', borderRadius: '0', backgroundColor: 'transparent', opacity: 1, transitionDuration: '0.1s', transitionTimingFunction: 'ease-out'},
                options: {showDefaultCursor: false, hoverSelector: 'a', mixBlendMode: 'normal'}
            },
        },
        {
            id: 18,
            name: 'Iriadient',
            type: 'svg',
            cursor: {
                centerShape: { fill: '#000000', opacity: 1, type: 1 },
                outerShape: {radius: 50, opacity: 1, mixBlendMode: 'darken', rotationSpeed: 0.1, hueSpeed: 0.001, transitionDuration: '0.1s', transitionTimingFunction: 'ease-out'},
                options: {showDefaultCursor: false}
            },
            hover: {
                centerShape: { fill: '#ffffff', opacity: 1, type: 1 },
                outerShape: {scale: 1.2, opacity: 1, mixBlendMode: 'darken', transitionDuration: '0.1s', transitionTimingFunction: 'ease-out'},
                options: {showDefaultCursor: false, hoverSelector: 'a'}
            },
        },
    ];

    const [activeMenu, setActiveMenu] = useState(null);
    const [hoveredCursor, setHoveredCursor] = useState(null);
    const [cursorPosition, setCursorPosition] = useState({ x: 0, y: 0 });
    const [cursors, setCursors] = useState(CURSORS);
    const [isCustomizeDialogOpen, setIsCustomizeDialogOpen] = useState(false);
    const [isActivateDialogOpen, setIsActivateDialogOpen] = useState(false);
    const [selectedCursor, setSelectedCursor] = useState(null);
    const [formValues, setFormValues] = useState<FormValues>({});
    const [defaultValues, setDefaultValues] = useState({});
    const [activeTab, setActiveTab] = useState('cursor');
    const [placement, setPlacement] = useState("entire_site"); // Default placement
    const [sectionIdentifier, setSectionIdentifier] = useState("");
    const containerRefs = useRef({});
    

    useEffect(() => {
        // Fetch cursor customizations
        axios.get(`${restUrl}/cursor-customization`)
            .then((response) => {
                const { customization } = response.data;
    
                if (customization) {
                    // Update formValues with fetched customizations
                    setFormValues((prevValues) => ({
                        ...prevValues,
                        ...customization,
                    }));
    
                    // Update cursors with fetched customizations
                    setCursors((prevCursors) =>
                        prevCursors.map((cursor) => ({
                            ...cursor,
                            cursor: customization[cursor.id]?.cursor || cursor.cursor,
                            hover: customization[cursor.id]?.hover || cursor.hover,
                        }))
                    );
                }
            })
            .catch((error) => {
                console.error('Error fetching customizations:', error);
            });
    }, [restUrl]);

    useEffect(() => {
        // On page load, initialize the cursor position
        const defaultContainer = containerRefs.current[cursors[0]?.id];
        if (defaultContainer) {
            const rect = defaultContainer.getBoundingClientRect();
            setCursorPosition({
                x: rect.width / 2,
                y: rect.height / 2
            });
        }
    }, [cursors]);

    useEffect(() => {
        const handleClickOutside = (event: MouseEvent) => {
            // Check if the clicked element is outside all dropdown menus
            const dropdownMenus = document.querySelectorAll('.dropdown-menu');
            const isClickInside = Array.from(dropdownMenus).some(menu =>
                menu.contains(event.target as Node)
            );
    
            if (!isClickInside) {
                setActiveMenu(null); // Close the dropdown menu
            }
        };
    
        // Add event listener when dropdown is open
        if (activeMenu !== null) {
            document.addEventListener('mousedown', handleClickOutside);
        }
    
        // Cleanup event listener
        return () => {
            document.removeEventListener('mousedown', handleClickOutside);
        };
    }, [activeMenu]);

    useEffect(() => {
        if (isCustomizeDialogOpen) {
            // Capture the initial values as defaults
            setDefaultValues(JSON.parse(JSON.stringify(formValues)));
        }
    }, [isCustomizeDialogOpen]);

    const handleMouseMove = (e, cursorId) => {
        if(cursorId !== 15) {
            const container = containerRefs.current[cursorId];
            if (container) {
                const rect = container.getBoundingClientRect();
                setCursorPosition({
                    x: e.clientX - rect.left,
                    y: e.clientY - rect.top
                });
            }
        }
    };

    const handleMouseEnter = (cursorId) => {
        if(cursorId !== 15) {
            setHoveredCursor(cursorId);
            const container = containerRefs.current[cursorId];
            if (container) {
                const rect = container.getBoundingClientRect();
                setCursorPosition({
                    x: rect.width / 2,
                    y: rect.height / 2
                });
            }
        }
    };

    const handleMouseLeave = () => {
        setHoveredCursor(null);
        setCursorPosition({ x: 0, y: 0 });
    };



    const handleMenuClick = (cursorId, e) => {
        e.stopPropagation();
        setActiveMenu(activeMenu === cursorId ? null : cursorId);
    };

    const handleInputChange = (outerKey, key, subKey, value) => {
        if (outerKey === 'options') {
            // Handle changes within the 'options' object
            setFormValues((prev) => ({
                ...prev,
                cursor: {
                    ...prev.cursor,
                    [outerKey]: {
                        ...prev.cursor[outerKey],
                        [key]: isNaN(value) ? value : Number(value),
                    },
                },
            }));
        } else if (subKey) {
            // Handle nested object values (for outer/inner properties)
            setFormValues((prev) => ({
                ...prev,
                cursor: {
                    ...prev.cursor,
                    [outerKey]: {
                        ...prev.cursor[outerKey],
                        [key]: {
                            ...prev.cursor[outerKey][key],
                            [subKey]: isNaN(value) ? value : Number(value),
                        },
                    },
                },
            }));
        } else {
            // Handle regular values (for outer/inner properties)
            setFormValues((prev) => ({
                ...prev,
                cursor: {
                    ...prev.cursor,
                    [outerKey]: {
                        ...prev.cursor[outerKey],
                        [key]: isNaN(value) ? value : Number(value),
                    },
                },
            }));
        }
    };

    const handleHoverChange = (outerKey, key, subKey, value) => {
        setFormValues((prev) => {
            let updatedHover = { ...prev.hover };

            if (outerKey === 'options') {
                updatedHover = {
                    ...updatedHover,
                    [outerKey]: {
                        ...updatedHover[outerKey],
                        [key]: value,
                    },
                };
            } else if (subKey) {
                // Handle nested object values (for outer/inner properties)
                updatedHover = {
                    ...updatedHover,
                    [outerKey]: {
                        ...updatedHover[outerKey],
                        [key]: {
                            ...updatedHover[outerKey][key],
                            [subKey]: isNaN(value) ? value : String(value),
                        },
                    },
                };
            }
             else {
                // Handle regular values (for outer/inner properties)
                updatedHover = {
                    ...updatedHover,
                    [outerKey]: {
                        ...updatedHover[outerKey],
                        [key]: isNaN(value) ? value : String(value),
                    },
                };
            }

            return {
                ...prev,
                hover: updatedHover,
            };
        });
    };

    const handleSave = () => {
        // Create a deep copy of the selected cursor as base structure
        const baseStructure = JSON.parse(JSON.stringify(selectedCursor));
    
        // Merge the form values with the base structure
        const customizationData = {
            id: selectedCursor.id,
            name: selectedCursor.name,
            type: selectedCursor.type,
            cursor: {
                ...(baseStructure.cursor?.outer && {
                    outer: { ...baseStructure.cursor.outer, ...formValues.cursor?.outer },
                }),
                ...(baseStructure.cursor?.inner && {
                    inner: { ...baseStructure.cursor.inner, ...formValues.cursor?.inner },
                }),
                ...(baseStructure.cursor?.options && { // Check if options exist
                    options: {
                        ...baseStructure.cursor.options,
                        ...formValues.cursor?.options,
                    },
                }),
                ...(baseStructure.cursor?.outer === undefined && formValues.cursor),
            },
            hover: {
                ...(baseStructure.hover?.outer && {
                    outer: { ...baseStructure.hover.outer, ...formValues.hover?.outer },
                }),
                ...(baseStructure.hover?.inner && {
                    inner: { ...baseStructure.hover.inner, ...formValues.hover?.inner },
                }),
                ...(baseStructure.hover?.options && { // Check if options exist
                    options: {
                        ...baseStructure.hover.options,
                        ...formValues.hover?.options,
                    },
                }),
                ...(baseStructure.hover?.outer === undefined && formValues.hover),
            },
        };
    
        // Save customization to the database
        axios
            .post(`${restUrl}/cursor-customization`, {
                cursor_id: selectedCursor.id,
                customization: customizationData,
            })
            .then(() => {
                // Update local state
                setCursors((prev) =>
                    prev.map((cursor) =>
                        cursor.id === selectedCursor.id ? { ...customizationData } : cursor
                    )
                );
    
                setIsCustomizeDialogOpen(false);
                setSelectedCursor(null);
            })
            .catch((error) => {
                console.error('Error saving customization:', error);
            });
    };

    const handleMenuAction = (action, cursorId) => {
        if (action === "add") {
            const selected = cursors.find((c) => c.id === cursorId);
            if (selected) {
                setSelectedCursor(selected);
                setIsActivateDialogOpen(true);
            }
        } else if (action === 'customize') {
            const selected = cursors.find((cursor) => cursor.id === cursorId);
            if (selected) {
                setSelectedCursor(selected);
                setFormValues({
                    cursor: selected.cursor || {}, // Initialize cursor
                    hover: selected.hover || {}   // Initialize hover
                }); 
                setIsCustomizeDialogOpen(true);
            }
        }
        else if (action === 'reset') {
            // Find the default state of the cursor from CURSORS
            const defaultCursor = CURSORS.find((cursor) => cursor.id === cursorId);

            if (defaultCursor) {
                // Remove the customization from the database
                axios
                    .delete(`${restUrl}/cursor-customization`, { data: { cursor_id: cursorId } })
                    .then(() => {
                        console.log(`Customization for cursor ${cursorId} removed from the database.`);

                        // Update local state to reset cursor to its default values
                        setCursors((prevCursors) =>
                            prevCursors.map((cursor) =>
                                cursor.id === cursorId
                                    ? { ...defaultCursor } // Reset to default values
                                    : cursor
                            )
                        );
                    })
                    .catch((error) => {
                        console.error('Error resetting cursor customization:', error);
                    });
            }
        }
        else if (action === 'preview') {
            const selected = cursors.find((cursor) => cursor.id === cursorId);
            if (selected) {
                // Serialize cursor configuration
                const cursorConfig = encodeURIComponent(JSON.stringify(selected));
                
                // Open a new tab with the serialized configuration
                const previewURL = `${window.location.origin}?preview_cursor=${cursorConfig}`;
                window.open(previewURL, '_blank');
            }
        }
        setActiveMenu(null);
    };

    const handleSubmitPlacement = async () => {
        if (placement === "specific_section" && !sectionIdentifier) {
            alert("Please enter a valid section identifier (e.g., #section-id or .class-name)");
            return;
        }
    
        if (!selectedCursor) {
            alert("No cursor selected.");
            return;
        }
    
        const cursorData = {
            cursor: selectedCursor, // Store the full cursor object
            placement: placement,
            section_identifier: placement === "specific_section" ? sectionIdentifier : null,
        };
    
        try {
            const response = await fetch('/wp-json/custom/v1/cursor-settings', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify(cursorData),
            });
    
            const result = await response.json();
    
            if (response.ok) {
                console.log("Cursor placement saved successfully", result);
                setIsActivateDialogOpen(false);
            } else {
                alert("Failed to save cursor placement: " + (result.message || "Unknown error"));
            }
        } catch (error) {
            console.error("Error saving cursor placement:", error);
            alert("An error occurred while saving the cursor placement.");
        }
    };

    function formatKey(key: string): string {
        return key
            .replace(/([a-z])([A-Z])/g, '$1 $2')
            .replace(/([A-Z])([A-Z][a-z])/g, '$1 $2')
            .replace(/^./, (str) => str.toUpperCase());
    }

    const renderInput = (outerKey, key, value) => {
        const numericKeys = ['opacity', 'scale', 'ease', 'rotationSpeed', 'hueSpeed', 'densityDissipation', 'velocityDissipation', 'pressure', 'splatRadius', 'bloomIntensity', 'bloomThreshold', 'bloomSoftKnee', 'sunraysWeight'];
        const isNumericInput = numericKeys.includes(key) || (typeof value === 'object' && value !== null && Object.keys(value).some(subKey => numericKeys.includes(subKey)));
    
        if (typeof value === 'object' && value !== null) {
            return (
                <div className="ml-4 flex-grow">
                    {Object.entries(value).map(([subKey, subValue]) => (
                        <div key={subKey} className="flex gap-3 items-center mb-1">
                            <label htmlFor={`cursor-${outerKey}-${key}-${subKey}`}>
                                {formatKey(subKey)}:
                            </label>
                            <input
                                id={`cursor-${outerKey}-${key}-${subKey}`}
                                type={numericKeys.includes(subKey) ? "number" : "text"}
                                {...(numericKeys.includes(subKey) ? { step: "0.1" } : {})}
                                value={subValue}
                                className="flex-grow p-1 border rounded"
                                placeholder={`${defaultValues[activeTab]?.[outerKey]?.[key]?.[subKey] || ''}`}
                                onChange={(e) =>
                                    handleInputChange(
                                        outerKey,
                                        key,
                                        subKey,
                                        numericKeys.includes(subKey) ? parseFloat(e.target.value) : e.target.value
                                    )
                                }
                            />
                        </div>
                    ))}
                </div>
            );
        }
    
        return (
            <input
                id={`cursor-${outerKey}-${key}`}
                type={numericKeys.includes(key) ? "number" : "text"}
                {...(numericKeys.includes(key) ? { step: "0.1" } : {})}
                value={value}
                className="flex-grow p-1 border rounded"
                placeholder={`${defaultValues[activeTab]?.[outerKey]?.[key] || ''}`}
                onChange={(e) =>
                    handleInputChange(
                        outerKey,
                        key,
                        null,
                        numericKeys.includes(key) ? parseFloat(e.target.value) : e.target.value
                    )
                }
            />
        );
    };

    const renderHoverInput = (outerKey, key, value) => {
        const numericKeys = ['opacity', 'scale', 'ease', 'densityDissipation', 'velocityDissipation', 'pressure', 'splatRadius', 'bloomIntensity', 'bloomThreshold', 'bloomSoftKnee', 'sunraysWeight'];
        const isNumericInput = numericKeys.includes(key) || (typeof value === 'object' && value !== null && Object.keys(value).some(subKey => numericKeys.includes(subKey)));
    
        if (typeof value === 'object' && value !== null) {
            return (
                <div className="ml-4 flex-grow">
                    {Object.entries(value).map(([subKey, subValue]) => (
                        <div key={subKey} className="flex gap-3 items-center mb-1">
                            <label htmlFor={`hover-${outerKey}-${key}-${subKey}`}>
                                {formatKey(subKey)}:
                            </label>
                            <input
                                id={`hover-${outerKey}-${key}-${subKey}`}
                                type={numericKeys.includes(subKey) ? "number" : "text"}
                                {...(numericKeys.includes(subKey) ? { step: "0.1" } : {})}
                                value={subValue}
                                className="flex-grow p-1 border rounded"
                                placeholder={`${defaultValues[activeTab]?.[outerKey]?.[key]?.[subKey] || ''}`}
                                onChange={(e) =>
                                    handleHoverChange(
                                        outerKey,
                                        key,
                                        subKey,
                                        numericKeys.includes(subKey) ? parseFloat(e.target.value) : e.target.value
                                    )
                                }
                            />
                        </div>
                    ))}
                </div>
            );
        }
    
        return (
            <input
                id={`hover-${outerKey}-${key}`}
                type={numericKeys.includes(key) ? "number" : "text"}
                {...(numericKeys.includes(key) ? { step: "0.1" } : {})}
                value={value}
                className="flex-grow p-1 border rounded"
                placeholder={`${defaultValues[activeTab]?.[outerKey]?.[key] || ''}`}
                onChange={(e) =>
                    handleHoverChange(
                        outerKey,
                        key,
                        null,
                        numericKeys.includes(key) ? parseFloat(e.target.value) : e.target.value
                    )
                }
            />
        );
    };

    return (
        <div className="bg-white rounded-lg p-8 mt-4">
            <h2 className="text-2xl text-neutral-700 mb-6">Your Cursors</h2>
            <div className="grid grid-cols-5 gap-4">
                {cursors.map((cursor) => (
                    <div 
                        key={cursor.id} 
                        className={`relative group`}
                    >
                        <div 
                            ref={el => containerRefs.current[cursor.id] = el}
                            className={`aspect-square bg-gray-100 rounded-lg mb-2 flex items-center justify-center relative overflow-hidden`}
                            onMouseMove={(e) => handleMouseMove(e, cursor.id)}
                            onMouseEnter={() => handleMouseEnter(cursor.id)}
                            onMouseLeave={handleMouseLeave}
                        >
                            <CursorItem 
                                pluginUrl={pluginUrl}
                                cursor={cursor}
                                isHovering={hoveredCursor === cursor.id}
                                position={hoveredCursor === cursor.id ? cursorPosition : { 
                                    x: containerRefs.current[cursor.id]?.offsetWidth / 2 || 0,
                                    y: containerRefs.current[cursor.id]?.offsetHeight / 2 || 0
                                }}
                            />

                            
                        </div>
                        <button 
                            className="absolute bottom-2 right-2 opacity-0 group-hover:opacity-100 transition-opacity duration-200"
                            onClick={(e) => handleMenuClick(cursor.id, e)}
                        >
                            <BsThreeDotsVertical className="text-gray-600 hover:text-gray-900" />
                        </button>

                        {activeMenu === cursor.id && (
                            <div className="absolute ottom-10 right-2 bg-white shadow-lg rounded-md py-1 z-[100] min-w-[160px]">
                                <button 
                                    className="w-full dropdown-menu text-left px-4 py-2 hover:bg-gray-100 text-sm"
                                    onClick={() => handleMenuAction('add', cursor.id)}
                                >
                                    Add to the site
                                </button>
                                <button 
                                    className="w-full dropdown-menu text-left px-4 py-2 hover:bg-gray-100 text-sm"
                                    onClick={() => handleMenuAction('preview', cursor.id)}
                                >
                                    Preview
                                </button>
                                <button 
                                    className="w-full dropdown-menu text-left px-4 py-2 hover:bg-gray-100 text-sm"
                                    onClick={() => handleMenuAction('customize', cursor.id)}
                                >
                                    Customize
                                </button>
                                <button 
                                    className="w-full dropdown-menu text-left px-4 py-2 hover:bg-gray-100 text-sm"
                                    onClick={() => handleMenuAction('reset', cursor.id)}
                                >
                                    Reset
                                </button>
                            </div>
                        )}
                        
                        <p className="text-sm text-center text-gray-700">{cursor.name}</p>
                    </div>
                ))}
            </div>

            {isCustomizeDialogOpen && (
                <div className="modal z-[100] fixed inset-0 flex justify-center items-center bg-black bg-opacity-50">
                    <div className="modal-content bg-white rounded-lg shadow-lg max-h-[90vh] min-w-[500px] flex flex-col">
                        {/* Header */}
                        <div className="modal-header p-4 flex flex-col">
                            <div className="flex justify-between items-center">
                                <h3 className="text-lg">Customize {selectedCursor.name}</h3>
                                <button
                                    className="text-gray-500 hover:text-gray-800"
                                    onClick={() => setIsCustomizeDialogOpen(false)}
                                >
                                    <MdClose className="text-lg" />
                                </button>
                            </div>
                            {/* Tabs */}
                            <div className="tabs mt-4 border-b flex">
                                <button
                                    className={`px-4 py-2 ${
                                        activeTab === 'cursor' ? 'border-b-2 border-blue-500 font-bold' : ''
                                    }`}
                                    onClick={() => setActiveTab('cursor')}
                                >
                                    Cursor
                                </button>
                                <button
                                    className={`px-4 py-2 ${
                                        activeTab === 'hover' ? 'border-b-2 border-blue-500 font-bold' : ''
                                    }`}
                                    onClick={() => setActiveTab('hover')}
                                >
                                    Hover
                                </button>
                            </div>
                        </div>

                        {/* Body (Scrollable Content) */}
                        <div className="modal-body flex-grow overflow-y-auto p-4">
                            {activeTab === 'cursor' &&
                                Object.entries(formValues.cursor).map(([outerKey, properties]) => (
                                    <div key={`${activeTab}-${outerKey}`}>
                                        <h4 className="font-bold mb-1 mt-3">{formatKey(outerKey)}</h4>
                                        <div className="bg-neutral-100 p-4 rounded-lg">
                                            {Object.entries(properties).map(([key, value]) => (
                                                <div key={key} className="flex gap-3 items-center mb-1">
                                                    <label htmlFor={`cursor-${outerKey}-${key}`}>
                                                        {formatKey(key)}:
                                                    </label>
                                                    {renderInput(outerKey, key, value)}
                                                </div>
                                            ))}
                                        </div>
                                    </div>
                                ))}

                            {activeTab === 'hover' &&
                                Object.entries(formValues.hover).map(([outerKey, properties]) => (
                                    <div key={`${activeTab}-${outerKey}`}>
                                        <h4 className="font-bold mb-1 mt-3">{formatKey(outerKey)}</h4>
                                        <div className="bg-neutral-100 p-4 rounded-lg">
                                            {Object.entries(properties).map(([key, value]) => (
                                                <div key={key} className="flex gap-3 items-center mb-1">
                                                    <label htmlFor={`hover-${outerKey}-${key}`}>
                                                        {formatKey(key)}:
                                                    </label>
                                                    {renderHoverInput(outerKey, key, value)}
                                                </div>
                                            ))}
                                        </div>
                                    </div>
                                ))}
                        </div>

                        {/* Footer */}
                        <div className="modal-footer p-4 flex justify-end gap-2">
                            <button
                                className="bg-blue-500 text-white px-4 py-2 rounded hover:bg-blue-600"
                                onClick={handleSave}
                            >
                                Save
                            </button>
                            <button
                                className="bg-gray-200 px-4 py-2 rounded hover:bg-gray-300"
                                onClick={() => setIsCustomizeDialogOpen(false)}
                            >
                                Cancel
                            </button>
                        </div>
                    </div>
                </div>
            )}

            {isActivateDialogOpen && (
                <div className="fixed z-[100] inset-0 flex items-center justify-center bg-black bg-opacity-50">
                    <div className="bg-white p-6 rounded-lg shadow-lg w-[400px]">
                        <h2 className="text-lg font-semibold">Select Cursor Placement</h2>
                        <select
                            value={placement}
                            onChange={(e) => setPlacement(e.target.value)}
                            className="w-full mt-3 p-2 border rounded-md"
                        >
                            <option value="entire_site">Entire Site</option>
                            <option value="specific_section">Specific Section</option>
                        </select>

                        {placement === "specific_section" && (
                            <input
                                type="text"
                                placeholder="Enter section ID or class (e.g., #my-section)"
                                value={sectionIdentifier}
                                onChange={(e) => setSectionIdentifier(e.target.value)}
                                className="w-full mt-3 p-2 border rounded-md"
                            />
                        )}

                        <div className="flex justify-end mt-4">
                            <button
                                className="px-4 py-2 bg-gray-300 rounded-md mr-2"
                                onClick={() => setIsActivateDialogOpen(false)}
                            >
                                Cancel
                            </button>
                            <button
                                className="px-4 py-2 bg-blue-600 text-white rounded-md"
                                onClick={handleSubmitPlacement}
                            >
                                Save
                            </button>
                        </div>
                    </div>
                </div>
            )}
        </div>
    );
};

export default CursorList;