// import React from '@wordpress/element';
import * as React from 'react';
import logo from '../../assets/img/logo.svg';

interface NavigationLink {
    id: number;
    title: string;
    path: string;
    wordpressAdminUrl?: string; // Optional WordPress admin URL
}

interface HeaderProps {
    currentPage: string;
    pluginName: string;
    pluginVersion: string; // Corrected to string
}

const Header: React.FC<HeaderProps> = ({ currentPage, pluginName, pluginVersion }) => {
    const navigationLinks: NavigationLink[] = [
        { id: 1, title: 'Dashboard', path: 'dashboard' },
        { id: 2, title: 'Settings', path: 'settings', wordpressAdminUrl: 'admin.php?page=wp_custom_cursors_settings' }, // Add the correct URL here!
        { id: 3, title: 'Reports', path: 'reports' },
    ];

    return (
        <header className="flex justify-between items-center py-4 px-6 border-b border-gray-200 mb-6">
            <div className="flex items-center gap-4">
                <img
                    src={logo}
                    alt="Plugin Logo"
                    className="w-10 h-10 object-contain"
                />
                <div className="flex items-baseline gap-3">
                    <h1 className="text-xl font-semibold m-0">{pluginName}</h1>
                    <span className="text-sm text-gray-600">{pluginVersion}</span>
                </div>
            </div>

            <nav className="flex gap-5">
                {navigationLinks.map(link => (
                    <a
                        key={link.id}
                        href={link.wordpressAdminUrl ? link.wordpressAdminUrl : `#${link.path}`} // Use WordPress URL if it exists, otherwise use the anchor link
                        className={`
                            px-3 py-2 rounded-md transition-colors duration-300
                            ${currentPage === link.path
                                ? 'bg-blue-600 text-white'
                                : 'text-gray-700 hover:bg-gray-100'
                            }
                        `}
                    >
                        {link.title}
                    </a>
                ))}
            </nav>
        </header>
    );
};

export default Header;