// import React from '@wordpress/element';
import React from 'react';
import { __ } from '@wordpress/i18n';
import domReady from '@wordpress/dom-ready';
import { createRoot } from '@wordpress/element';
import Header from './components/Header';
import { useState, useEffect, createElement } from '@wordpress/element';
import { CgFileDocument } from "react-icons/cg";
import { BiDonateHeart } from "react-icons/bi";
import { AiOutlineStar } from 'react-icons/ai';

import AdBanner from './components/AdBanner';
import EmptyState from './components/EmptyState';
import Card from './components/Card';
import CursorList from './components/CursorList';
import SavedCursorList from './components/SavedCursorList';

const App = () => {
    const [showCursorList, setShowCursorList] = useState(false);
    const [savedCursors, setSavedCursors] = useState([]);

    // Fetch saved cursors from the DB when the component mounts
    useEffect(() => {
        const fetchSavedCursors = async () => {
            try {
                const response = await fetch('/wp-json/custom/v1/cursor-settings');
                const result = await response.json();

                if (result.success && result.settings && result.settings.length > 0) {
                    setSavedCursors(result.settings);
                    setShowCursorList(false); // If there are cursors, hide empty state
                } else {
                    setSavedCursors([]); // No cursors in DB
                    setShowCursorList(true); // Show empty state
                }
            } catch (error) {
                console.error("Error fetching cursor settings:", error);
            }
        };

        fetchSavedCursors();
    }, []);

    const handleAddCursorClick = () => {
        setShowCursorList(true);
    };

    return (
        <>
            <Header currentPage='dashboard' pluginName={pluginData.name} pluginVersion={pluginData.version} />
            <div className="grid grid-cols-4 gap-4">
                <div className="col-span-3">
                    <AdBanner />
                    {/* Conditional Rendering */}
                    {showCursorList ? (
                        <CursorList cursors={savedCursors} pluginUrl={pluginData.url} siteUrl={pluginData.siteUrl} restUrl={pluginData.restUrl} />
                    ) : savedCursors.length > 0 ? (
                        <SavedCursorList onAddCursorClick={handleAddCursorClick} cursors={savedCursors} setSavedCursors={setSavedCursors} restUrl={pluginData.restUrl} />
                    ) : (
                        <EmptyState onAddCursorClick={handleAddCursorClick} />
                    )}
                </div>
                <div className="col-span-1">
                    <Card
                        icon={<CgFileDocument />}
                        heading="Documentation"
                        description="Our online documentation is available anytime to guide you through every step."
                        linkTitle="Read Docs"
                        linkUrl="https://hamidrezasepehr.com/projects/wp-custom-cursor/documentation/"
                    />
                    <Card
                        icon={<AiOutlineStar />}
                        heading="Rate This Plugin"
                        description="Your feedback helps us improve and provide the best experience."
                        linkTitle="Rate Now"
                        linkUrl="https://wordpress.org/plugins/wp-custom-cursors"
                        className="mt-4"
                    />
                    <Card
                        icon={<BiDonateHeart />}
                        heading="Donation"
                        description="Your generous donation makes a difference and enables me to provide future updates."
                        linkTitle="Donate Me"
                        linkUrl="https://hamidrezasepehr.com/donate/"
                        className="mt-4"
                    />
                </div>
            </div>
        </>
    );
};

domReady(() => {
    const root = createRoot(
        document.getElementById('app')
    );
    
    root.render(<App />);
});