// Define specific types for known string values
type BlendMode = 'normal' | 'multiply' | 'screen' | 'overlay' | 'darken' | 'lighten' | 'color-dodge' | 'color-burn' | 'hard-light' | 'soft-light' | 'difference' | 'exclusion' | 'hue' | 'saturation' | 'color' | 'luminosity';
type SpinDirection = 'right' | 'left';

interface DionytrisCenterShape {
    visible: boolean;
    fill: string;
    opacity: number;
    radius: number;
    type: number;
}

interface DionytrisMiddleShape {
    visible: boolean;
    scale: number;
    fill: string;
    opacity: number;
    blendingMode: BlendMode;
    type: number;
    spin: boolean;
    spinDirection: SpinDirection;
    spinDuration: string;
}

interface DionytrisOuterShape {
    visible: boolean;
    radius: number;
    fill: string;
    opacity: number;
    blendingMode: BlendMode;
}

interface DionytrisText {
    visible: boolean;
    content: string;
    fontSize: number;
    fontWeight: number;
    fill: string;
    opacity: number;
    spin: boolean;
    spinDirection: SpinDirection;
    spinDuration: string;
}

interface DionytrisSvgShape {
    width: number;
    zIndex: string;
    transition: string;
}

interface DionytrisOptions {
    showDefaultCursor: boolean;
    hoverSelector?: string;
}

export interface DionytrisCursorData {
    id: number;
    name: string;
    type: 'svg';
    cursor: {
        centerShape: DionytrisCenterShape;
        middleShape: DionytrisMiddleShape;
        outerShape: DionytrisOuterShape;
        text: DionytrisText;
        svgShape: DionytrisSvgShape;
        options: DionytrisOptions;
    };
    hover: {
        centerShape: DionytrisCenterShape;
        middleShape: DionytrisMiddleShape;
        outerShape: DionytrisOuterShape;
        text: DionytrisText;
        svgShape: DionytrisSvgShape;
        options: DionytrisOptions;
    };
}