// import React from 'react';
import * as React from 'react';
import { CursorData } from "./CursorList" //relative path
import { DionytrisCursorData } from '../types/dionytrisType';


interface CursorItemProps {
    cursor: CursorData;
    isHovering: boolean;
    position: { x: number; y: number };
    pluginUrl: string;
}
import { useRef, useEffect } from '@wordpress/element';

const CursorItem: React.FC<CursorItemProps> = ({ cursor, isHovering, position, pluginUrl }) => {
    const positions = useRef<{ [key: string]: { x: number; y: number } }>(
        Object.keys(cursor?.cursor || {}).reduce((acc: { [key: string]: { x: number; y: number } }, _, index) => {
            acc[String(index)] = { x: 0, y: 0 }; // Initialize positions for each item. String as key as index is string in Object.keys
            return acc;
        }, {})
    );

    const wrapperRef = useRef(null);
    const animationIds = useRef([]);

    const gaialoopRef = useRef<HTMLDivElement | null>(null);
    const demetrobyteRef = useRef<HTMLDivElement | null>(null);
    const hestellipseeRef = useRef<HTMLDivElement | null>(null);
    const hephamageRef = useRef<HTMLDivElement | null>(null);
    const nyxamokeRef = useRef<HTMLDivElement | null>(null);
    const dionytrisRef = useRef<HTMLDivElement | null>(null);
    const aronekoRef = useRef<HTMLDivElement | null>(null);
    const aphorbitRef = useRef<HTMLDivElement | null>(null);
    const persephontRef = useRef<HTMLDivElement | null>(null);
    const iriadientRef = useRef<HTMLDivElement | null>(null);

    useEffect(() => {
        // Dionytris
        if (cursor.id === 7 && dionytrisRef.current) {
            let targetDiv = dionytrisRef.current;

            // Create container for cursor SVG
            let cursorSvg = targetDiv.querySelector('.cursor-svg');
            
            // Remove any existing cursor SVGs before creating a new one
            const cleanup = () => {
                const existingSvgs = targetDiv.querySelectorAll('svg');
                existingSvgs.forEach(svg => svg.remove());
                
                // Remove any existing style tags created for animations
                const styleTag = document.querySelector('style[data-cursor-animation]');
                if (styleTag) {
                    styleTag.remove();
                }

                // Remove event listeners
                if (cursorSvg) {
                    targetDiv.removeEventListener('mousemove', handleMouseMove);
                    targetDiv.removeEventListener('mouseout', centerCursor);
                }
            };

            // Clean up existing cursors before creating new one
            cleanup();

            const middleShapesArray = {
                1: "M93.68 5.155a10 10 0 0 1 12.64 0l7.933 6.47a10.002 10.002 0 0 0 8.93 1.904l9.856-2.664a10 10 0 0 1 11.533 5.14l4.657 9.21a10 10 0 0 0 7.359 5.364l10.129 1.605a10 10 0 0 1 8.422 9.364l.533 10.39a10 10 0 0 0 4.527 7.865l8.666 5.649a10 10 0 0 1 3.878 11.951l-3.728 9.743a10 10 0 0 0 .943 9.007l5.66 8.746a10 10 0 0 1-1.312 12.49l-7.33 7.359a9.998 9.998 0 0 0-2.792 8.621l1.631 10.299a10.003 10.003 0 0 1-6.284 10.896l-9.61 3.7a10.001 10.001 0 0 0-6.072 6.765l-2.661 10.018a10 10 0 0 1-10.199 7.419l-10.209-.546a10 10 0 0 0-8.328 3.719l-6.438 8.008a10 10 0 0 1-12.361 2.63l-9.086-4.665a10.004 10.004 0 0 0-9.134 0l-9.087 4.665a9.999 9.999 0 0 1-12.36-2.63l-6.438-8.008a10 10 0 0 0-8.328-3.719l-10.209.546a10 10 0 0 1-10.2-7.419l-2.66-10.018a10 10 0 0 0-6.072-6.765l-9.61-3.7a10 10 0 0 1-6.284-10.896l1.631-10.299a10 10 0 0 0-2.792-8.621l-7.33-7.359a10 10 0 0 1-1.312-12.49l5.66-8.746a10 10 0 0 0 .943-9.007l-3.728-9.743a10 10 0 0 1 3.878-11.951l8.666-5.649a10 10 0 0 0 4.527-7.865l.533-10.39a10 10 0 0 1 8.422-9.364l10.129-1.605a10 10 0 0 0 7.359-5.364l4.657-9.21a10 10 0 0 1 11.533-5.14l9.856 2.664a10 10 0 0 0 8.93-1.904l7.933-6.47Z",
                2: "M89.9318 6.12863C94.1569 -2.04288 105.843 -2.04288 110.068 6.12863L136.973 58.1644C138.051 60.2496 139.75 61.9486 141.836 63.0268L193.871 89.9318C202.043 94.1569 202.043 105.843 193.871 110.068L141.836 136.973C139.75 138.051 138.051 139.75 136.973 141.836L110.068 193.871C105.843 202.043 94.1569 202.043 89.9318 193.871L63.0268 141.836C61.9486 139.75 60.2496 138.051 58.1644 136.973L6.12863 110.068C-2.04288 105.843 -2.04288 94.1569 6.12863 89.9318L58.1644 63.0268C60.2496 61.9486 61.9486 60.2496 63.0268 58.1644L89.9318 6.12863Z", 
                3: "M146.371 34.5888C147.629 31.5563 148.272 28.3046 148.265 25.0218C148.267 20.0831 146.806 15.2545 144.066 11.1455C141.326 7.03648 137.431 3.83135 132.871 1.9347C128.311 0.038053 123.291 -0.465076 118.445 0.488832C113.6 1.44274 109.145 3.81092 105.644 7.29439L100.004 12.5471L94.4021 7.31163C89.7108 2.62604 83.3503 -0.00403591 76.7198 4.64873e-06C70.0894 0.00404521 63.7321 2.64187 59.0465 7.33318C54.3609 12.0245 51.7308 18.385 51.7348 25.0155C51.7389 31.6459 54.3767 38.0032 59.068 42.6888L97.5477 81.7374C97.8685 82.0631 98.2508 82.3217 98.6725 82.4983C99.0941 82.6748 99.5467 82.7658 100.004 82.7658C100.461 82.7658 100.914 82.6748 101.335 82.4983C101.757 82.3217 102.139 82.0631 102.46 81.7374L140.94 42.6888C143.268 40.3744 145.114 37.6213 146.371 34.5888ZM53.6286 165.411C52.3712 168.444 51.7276 171.695 51.7349 174.978C51.7327 179.917 53.1938 184.746 55.9336 188.855C58.6735 192.964 62.5693 196.169 67.1293 198.065C71.6892 199.962 76.7089 200.465 81.5546 199.511C86.4003 198.557 90.8547 196.189 94.3556 192.706L99.9961 187.453L105.598 192.688C110.289 197.374 116.65 200.004 123.28 200C129.911 199.996 136.268 197.358 140.954 192.667C145.639 187.976 148.269 181.615 148.265 174.985C148.261 168.354 145.623 161.997 140.932 157.311L102.452 118.263C102.132 117.937 101.749 117.678 101.327 117.502C100.906 117.325 100.453 117.234 99.9961 117.234C99.539 117.234 99.0864 117.325 98.6647 117.502C98.2431 117.678 97.8607 117.937 97.54 118.263L59.0603 157.311C56.7321 159.626 54.8859 162.379 53.6286 165.411ZM174.978 148.266C171.695 148.273 168.444 147.629 165.411 146.372C162.379 145.115 159.626 143.268 157.311 140.94L118.263 102.461C117.937 102.14 117.678 101.757 117.502 101.336C117.325 100.914 117.234 100.462 117.234 100.004C117.234 99.5473 117.325 99.0947 117.502 98.6731C117.678 98.2514 117.937 97.869 118.263 97.5483L157.311 59.0686C161.997 54.3773 168.354 51.7394 174.985 51.7354C181.615 51.7314 187.976 54.3614 192.667 59.047C197.358 63.7326 199.996 70.0899 200 76.7204C200.004 83.3509 197.374 89.7114 192.688 94.4027L187.453 100.004L192.706 105.645C196.189 109.146 198.557 113.6 199.511 118.446C200.465 123.292 199.962 128.311 198.065 132.871C196.169 137.431 192.964 141.327 188.855 144.067C184.746 146.807 179.917 148.268 174.978 148.266ZM34.5888 53.628C31.5563 52.3706 28.3046 51.727 25.0218 51.7343C20.0831 51.7321 15.2544 53.1932 11.1455 55.933C7.03647 58.6729 3.83134 62.5687 1.9347 67.1287C0.0380524 71.6887 -0.465076 76.7083 0.488831 81.554C1.44274 86.3997 3.81091 90.8542 7.29439 94.355L12.5471 99.9956L7.31163 105.597C2.62603 110.289 -0.00403599 116.649 4.64892e-06 123.28C0.00404529 129.91 2.64186 136.267 7.33317 140.953C12.0245 145.639 18.385 148.269 25.0155 148.265C31.6459 148.261 38.0032 145.623 42.6888 140.931L81.7373 102.452C82.063 102.131 82.3217 101.749 82.4982 101.327C82.6748 100.905 82.7657 100.453 82.7657 99.9955C82.7657 99.5384 82.6748 99.0858 82.4982 98.6642C82.3217 98.2425 82.063 97.8602 81.7373 97.5394L42.6888 59.0597C40.3744 56.7315 37.6213 54.8853 34.5888 53.628Z", 
                4: "M100.106 0L115 54.1591L158.884 19.0983L139.1 71.6687L195.211 69.0983L148.306 100L195.211 130.902L139.1 128.331L158.884 180.902L115 145.841L100.106 200L85.211 145.841L41.3271 180.902L61.111 128.331L5 130.902L51.9057 100L5 69.0983L61.111 71.6687L41.3271 19.0983L85.211 54.1591L100.106 0Z", 
                5: "M97.8118 1.84638C98.2359 -0.615472 101.764 -0.615455 102.188 1.8464L106.863 28.9805C107.236 31.1438 110.179 31.5163 111.077 29.5138L122.339 4.39701C123.361 2.11817 126.778 2.99725 126.578 5.48745L124.371 32.9338C124.195 35.1221 126.953 36.2162 128.32 34.5003L145.463 12.9789C147.018 11.0263 150.11 12.7293 149.298 15.0913L140.348 41.1255C139.634 43.2011 142.034 44.9481 143.784 43.6267L165.73 27.053C167.721 25.5493 170.293 27.9691 168.92 30.0546L153.789 53.0407C152.583 54.8733 154.474 57.1634 156.497 56.3195L181.867 45.7349C184.169 44.7746 186.06 47.7592 184.212 49.4371L163.851 67.9308C162.228 69.4053 163.491 72.0946 165.66 71.7812L192.86 67.8507C195.328 67.4941 196.419 70.8561 194.213 72.021L169.901 84.8603C167.963 85.8839 168.519 88.8034 170.697 89.0403L198.019 92.0109C200.498 92.2805 200.719 95.8084 198.294 96.3871L171.559 102.765C169.427 103.274 169.241 106.24 171.292 107.012L197.018 116.697C199.352 117.576 198.691 121.048 196.198 121.005L168.72 120.521C166.529 120.482 165.613 123.309 167.408 124.568L189.922 140.359C191.964 141.792 190.462 144.99 188.058 144.326L161.564 137.011C159.451 136.428 157.862 138.937 159.288 140.604L177.175 161.509C178.798 163.406 176.549 166.129 174.385 164.887L150.539 151.2C148.637 150.109 146.475 152.144 147.443 154.113L159.579 178.818C160.68 181.06 157.826 183.137 156.039 181.396L136.338 162.197C134.768 160.666 132.168 162.098 132.617 164.247L138.24 191.199C138.75 193.645 135.469 194.946 134.171 192.814L119.855 169.309C118.713 167.435 115.84 168.174 115.741 170.367L114.497 197.875C114.385 200.37 110.884 200.813 110.156 198.424L102.124 172.091C101.483 169.992 98.5168 169.992 97.8764 172.091L89.8444 198.424C89.1156 200.813 85.6154 200.37 85.5025 197.875L84.259 170.367C84.1599 168.174 81.2866 167.435 80.1453 169.309L65.8294 192.814C64.5305 194.946 61.2502 193.645 61.7604 191.199L67.3835 164.247C67.8318 162.098 65.2323 160.666 63.6617 162.197L43.9615 181.396C42.1741 183.137 39.3198 181.06 40.421 178.818L52.5574 154.113C53.525 152.144 51.3625 150.109 49.4613 151.2L25.6147 164.887C23.4511 166.129 21.2022 163.406 22.8251 161.509L40.7123 140.604C42.1384 138.937 40.5488 136.428 38.4365 137.011L11.9418 144.326C9.53794 144.99 8.03576 141.792 10.0784 140.359L32.5924 124.568C34.3874 123.309 33.4707 120.482 31.28 120.521L3.80194 121.005C1.30888 121.048 0.647761 117.576 2.98188 116.697L28.708 107.012C30.7591 106.24 30.5728 103.274 28.4413 102.765L1.7065 96.3871C-0.719138 95.8084 -0.497588 92.2805 1.98131 92.0109L29.3032 89.0403C31.4815 88.8034 32.0373 85.8839 30.099 84.8603L5.78724 72.021C3.58145 70.856 4.67171 67.4941 7.13963 67.8507L34.3405 71.7812C36.5091 72.0946 37.7722 69.4053 36.1488 67.9308L15.7877 49.4371C13.9404 47.7592 15.8308 44.7746 18.1327 45.7349L43.5034 56.3195C45.5262 57.1634 47.417 54.8733 46.2107 53.0407L31.0796 30.0546C29.7067 27.9691 32.2786 25.5493 34.2698 27.053L56.2162 43.6267C57.966 44.9481 60.3658 43.2011 59.6523 41.1255L50.7019 15.0913C49.8899 12.7293 52.9816 11.0263 54.537 12.9789L71.6801 34.5003C73.0469 36.2162 75.805 35.122 75.6291 32.9338L73.4219 5.48743C73.2216 2.99724 76.6389 2.11816 77.6607 4.39699L88.9235 29.5138C89.8214 31.5163 92.7645 31.1438 93.1372 28.9805L97.8118 1.84638Z", 
                6: "M71.5579 16.3347C84.3365 -5.4449 115.825 -5.44489 128.603 16.3347L129.067 17.1257C134.963 27.1733 145.709 33.378 157.358 33.4596L158.276 33.466C183.527 33.6428 199.271 60.9123 186.798 82.8687L186.345 83.6661C180.591 93.7953 180.591 106.205 186.345 116.334L186.798 117.131C199.271 139.088 183.527 166.357 158.276 166.534L157.358 166.54C145.709 166.622 134.963 172.827 129.067 182.874L128.603 183.665C115.825 205.445 84.3365 205.445 71.5579 183.665L71.0938 182.874C65.1986 172.827 54.4517 166.622 42.8027 166.54L41.8856 166.534C16.6346 166.357 0.890585 139.088 13.3629 117.131L13.8159 116.334C19.5698 106.205 19.5698 93.7953 13.8159 83.6661L13.3629 82.8687C0.890585 60.9123 16.6346 33.6428 41.8856 33.466L42.8027 33.4596C54.4518 33.378 65.1986 27.1733 71.0938 17.1257L71.5579 16.3347Z", 
                7: "M107.143 0H92.8571V82.7556L34.3401 24.2385L24.2386 34.3401L82.7556 92.8571H0V107.143H82.7555L24.2386 165.66L34.3401 175.761L92.8571 117.244V200H107.143V117.244L165.66 175.761L175.761 165.66L117.244 107.143H200V92.8571H117.244L175.761 34.34L165.66 24.2385L107.143 82.7555V0Z", 
                8: "M107.143 0H92.8571V63.2531L69.1621 4.60582L55.9166 9.95735L80.2255 70.1239L34.3401 24.2385L24.2386 34.3401L68.2177 78.3191L11.2241 53.4181L5.50459 66.5089L65.8105 92.8571H0V107.143H65.8104L5.50461 133.491L11.2241 146.582L68.2176 121.681L24.2386 165.66L34.3401 175.761L80.2255 129.876L55.9166 190.043L69.1621 195.394L92.8571 136.747V200H107.143V136.747L130.838 195.394L144.083 190.043L119.775 129.876L165.66 175.761L175.761 165.66L131.782 121.681L188.776 146.582L194.495 133.491L134.19 107.143H200V92.8571H134.189L194.495 66.5089L188.776 53.4181L131.782 78.3191L175.761 34.34L165.66 24.2385L119.775 70.1238L144.083 9.95735L130.838 4.60582L107.143 63.2531V0Z", 
                9: "M100.254 200C97.0998 200 94.4337 197.716 93.6699 194.656C91.2352 184.903 86.5744 174.531 79.6875 163.542C71.5278 150.347 59.8958 138.108 44.7917 126.823C31.6549 116.894 18.5181 110.123 5.38138 106.511C2.27005 105.656 0 102.897 0 99.6702V99.6702C0 96.5066 2.18273 93.7795 5.22473 92.9109C18.1045 89.2335 30.5122 83.2631 42.4479 75C56.1632 65.4514 67.6215 53.993 76.8229 40.625C84.9629 28.7165 90.5507 16.9388 93.5863 5.29207C94.3815 2.24095 97.0676 0 100.221 0V0C103.409 0 106.114 2.29058 106.89 5.3833C108.642 12.3654 111.38 19.512 115.104 26.8229C119.792 35.8507 125.781 44.5312 133.073 52.8646C140.538 61.0243 148.872 68.4028 158.073 75C170.097 83.5231 182.32 89.5111 194.744 92.964C197.792 93.8109 200 96.5244 200 99.6874V99.6874C200 102.898 197.725 105.634 194.629 106.483C186.754 108.642 178.648 112.124 170.313 116.927C160.243 122.83 150.868 129.861 142.187 138.021C133.507 146.007 126.389 154.427 120.833 163.281C113.933 174.293 109.267 184.745 106.837 194.639C106.083 197.708 103.413 200 100.254 200V200Z", 
                10: "M120 0H80V51.7157L43.4315 15.1472L15.1472 43.4314L51.7158 80H0V120H51.7157L15.1472 156.568L43.4315 184.853L80 148.284V200H120V148.284L156.569 184.853L184.853 156.569L148.284 120H200V80H148.284L184.853 43.4314L156.569 15.1471L120 51.7157V0Z",
                11: "M100 200C93.4028 200 87.3264 198.351 81.7708 195.052C76.3889 191.927 72.0486 187.674 68.75 182.292C65.625 176.736 64.0625 170.66 64.0625 164.062C64.0625 155.729 65.9722 148.785 69.7917 143.229C73.6111 137.674 79.5139 131.163 87.5 123.698C93.4028 118.316 96.3542 113.194 96.3542 108.333V103.646H91.6667C86.2847 103.646 78.9062 109.028 69.5312 119.792C60.3299 130.556 49.1319 135.937 35.9375 135.937C29.3403 135.937 23.2639 134.375 17.7083 131.25C12.3264 127.951 7.98611 123.611 4.6875 118.229C1.5625 112.674 0 106.597 0 100C0 93.4028 1.5625 87.4132 4.6875 82.0312C7.98611 76.4757 12.3264 72.1354 17.7083 69.0104C23.2639 65.7118 29.3403 64.0625 35.9375 64.0625C48.9583 64.0625 60.0694 69.3576 69.2708 79.9479C78.4722 90.5382 85.9375 95.8333 91.6667 95.8333H96.3542V91.6667C96.3542 86.8055 93.4028 81.684 87.5 76.3021L81.5104 70.8333C77.1701 66.8403 73.1771 62.066 69.5312 56.5104C65.8854 50.7812 64.0625 43.9236 64.0625 35.9375C64.0625 29.3403 65.625 23.3507 68.75 17.9688C72.0486 12.4132 76.3889 8.07292 81.7708 4.94791C87.3264 1.6493 93.4028 0 100 0C106.597 0 112.587 1.6493 117.969 4.94791C123.524 8.24652 127.865 12.5868 130.99 17.9688C134.288 23.3507 135.937 29.3403 135.937 35.9375C135.937 48.9583 130.642 60.0694 120.052 69.2708C109.462 78.4722 104.167 85.9375 104.167 91.6667V95.8333H108.333C114.236 95.8333 121.701 90.5382 130.729 79.9479C139.583 69.3576 150.694 64.0625 164.063 64.0625C170.66 64.0625 176.649 65.7118 182.031 69.0104C187.587 72.1354 191.927 76.3889 195.052 81.7708C198.351 87.1528 200 93.2292 200 100C200 106.597 198.351 112.674 195.052 118.229C191.927 123.611 187.587 127.951 182.031 131.25C176.649 134.375 170.66 135.937 164.063 135.937C155.903 135.937 148.872 133.941 142.969 129.948C137.24 125.955 130.816 120.139 123.698 112.5C118.316 106.597 113.194 103.646 108.333 103.646H104.167V108.333C104.167 114.757 109.462 122.222 120.052 130.729C130.642 139.236 135.937 150.347 135.937 164.062C135.937 170.66 134.288 176.736 130.99 182.292C127.865 187.674 123.611 191.927 118.229 195.052C112.847 198.351 106.771 200 100 200Z",
                12: "M120 14.2857C120 6.39593 113.604 0 105.714 0H94.2857C86.3959 0 80 6.39594 80 14.2857V17.2269C80 29.9541 64.6123 36.3279 55.6128 27.3284L53.533 25.2487C47.9541 19.6698 38.9089 19.6698 33.3299 25.2487L25.2487 33.3299C19.6698 38.9088 19.6698 47.954 25.2487 53.533L27.3285 55.6128C36.328 64.6123 29.9542 80 17.227 80H14.2857C6.39593 80 0 86.3959 0 94.2857V105.714C0 113.604 6.39593 120 14.2857 120H17.2269C29.9542 120 36.328 135.388 27.3285 144.387L25.2487 146.467C19.6698 152.046 19.6698 161.091 25.2487 166.67L33.3299 174.751C38.9089 180.33 47.9541 180.33 53.533 174.751L55.6128 172.671C64.6122 163.672 80 170.046 80 182.773V185.714C80 193.604 86.3959 200 94.2857 200H105.714C113.604 200 120 193.604 120 185.714V182.773C120 170.046 135.388 163.672 144.387 172.671L146.467 174.751C152.046 180.33 161.091 180.33 166.67 174.751L174.751 166.67C180.33 161.091 180.33 152.046 174.751 146.467L172.672 144.387C163.672 135.388 170.046 120 182.773 120H185.714C193.604 120 200 113.604 200 105.714V94.2857C200 86.3959 193.604 80 185.714 80H182.773C170.046 80 163.672 64.6123 172.671 55.6128L174.751 53.5329C180.33 47.954 180.33 38.9088 174.751 33.3299L166.67 25.2487C161.091 19.6697 152.046 19.6697 146.467 25.2487L144.387 27.3284C135.388 36.3279 120 29.9541 120 17.2269V14.2857Z",
                13: "M128.603 16.3347C115.825 -5.44489 84.3365 -5.4449 71.5579 16.3347L71.0938 17.1257C65.1986 27.1733 54.4518 33.378 42.8027 33.4596L41.8856 33.466C16.6346 33.6428 0.890585 60.9123 13.3629 82.8687L13.8159 83.6661C19.5698 93.7953 19.5698 106.205 13.8159 116.334L13.3629 117.131C0.890586 139.088 16.6346 166.357 41.8856 166.534L42.8027 166.54C54.4517 166.622 65.1986 172.827 71.0938 182.874L71.5579 183.665C84.3365 205.445 115.825 205.445 128.603 183.665L129.067 182.874C134.963 172.827 145.709 166.622 157.358 166.54L158.276 166.534C183.527 166.357 199.271 139.088 186.798 117.131L186.345 116.334C180.591 106.205 180.591 93.7953 186.345 83.6661L186.798 82.8687C199.271 60.9123 183.527 33.6428 158.276 33.466L157.358 33.4596C145.709 33.378 134.963 27.1733 129.067 17.1257L128.603 16.3347ZM100.081 149.604C127.476 149.604 149.685 127.396 149.685 100C149.685 72.6042 127.476 50.3955 100.081 50.3955C72.6848 50.3955 50.4761 72.6042 50.4761 100C50.4761 127.396 72.6848 149.604 100.081 149.604Z",
                14: "M110 0C115.523 0 120 4.47715 120 10V27.5736C120 36.4826 130.771 40.9443 137.071 34.6446L149.497 22.2182C153.403 18.313 159.734 18.313 163.64 22.2182L177.782 36.3604C181.687 40.2656 181.687 46.5973 177.782 50.5025L165.355 62.9289C159.056 69.2286 163.517 80 172.426 80H190C195.523 80 200 84.4771 200 90V110C200 115.523 195.523 120 190 120H172.426C163.517 120 159.056 130.771 165.355 137.071L177.782 149.497C181.687 153.403 181.687 159.734 177.782 163.64L163.64 177.782C159.734 181.687 153.403 181.687 149.497 177.782L137.071 165.355C130.771 159.056 120 163.517 120 172.426V190C120 195.523 115.523 200 110 200H90C84.4771 200 80 195.523 80 190V172.426C80 163.517 69.2286 159.056 62.9289 165.355L50.5025 177.782C46.5973 181.687 40.2656 181.687 36.3604 177.782L22.2183 163.64C18.313 159.734 18.313 153.403 22.2183 149.497L34.6447 137.071C40.9443 130.771 36.4827 120 27.5736 120H10C4.47716 120 0 115.523 0 110V90C0 84.4771 4.47715 80 10 80H27.5736C36.4826 80 40.9443 69.2286 34.6447 62.9289L22.2183 50.5025C18.313 46.5972 18.313 40.2656 22.2183 36.3604L36.3604 22.2182C40.2656 18.313 46.5973 18.313 50.5025 22.2182L62.9289 34.6446C69.2286 40.9443 80 36.4826 80 27.5736V10C80 4.47715 84.4771 0 90 0H110ZM100 150C127.614 150 150 127.614 150 100C150 72.3858 127.614 50 100 50C72.3858 50 50 72.3858 50 100C50 127.614 72.3858 150 100 150Z",
                15: "M101.604 0.859655C100.827 -0.435397 98.9501 -0.435396 98.1734 0.859656L79.3113 32.3082C78.7978 33.1643 77.7318 33.5106 76.8132 33.1199L43.0684 18.7644C41.6788 18.1732 40.1604 19.2764 40.2932 20.7806L43.5184 57.3099C43.6062 58.3043 42.9474 59.2111 41.9745 59.4349L6.23647 67.6557C4.76478 67.9943 4.18481 69.7792 5.17645 70.9182L29.2571 98.5752C29.9126 99.3281 29.9126 100.449 29.2571 101.202L5.17645 128.859C4.18481 129.998 4.76478 131.783 6.23648 132.121L41.9745 140.342C42.9474 140.566 43.6062 141.473 43.5184 142.467L40.2932 178.996C40.1604 180.501 41.6788 181.604 43.0684 181.013L76.8132 166.657C77.7318 166.266 78.7978 166.613 79.3113 167.469L98.1734 198.917C98.9501 200.213 100.827 200.212 101.604 198.917L120.466 167.469C120.979 166.613 122.045 166.266 122.964 166.657L156.709 181.013C158.098 181.604 159.617 180.501 159.484 178.996L156.259 142.467C156.171 141.473 156.83 140.566 157.803 140.342L193.541 132.121C195.012 131.783 195.592 129.998 194.601 128.859L170.52 101.202C169.865 100.449 169.865 99.3281 170.52 98.5752L194.601 70.9182C195.592 69.7792 195.012 67.9943 193.541 67.6557L157.803 59.4349C156.83 59.2111 156.171 58.3043 156.259 57.3099L159.484 20.7806C159.617 19.2764 158.098 18.1732 156.709 18.7644L122.964 33.1199C122.045 33.5106 120.979 33.1643 120.466 32.3082L101.604 0.859655ZM100 150C127.614 150 150 127.614 150 100C150 72.3857 127.614 50 100 50C72.3858 50 50 72.3857 50 100C50 127.614 72.3858 150 100 150Z",
                16: "M103.143 0C105.352 0 107.143 1.79086 107.143 4V42.6749C107.143 47.0714 113.205 48.2497 114.852 44.1733L129.339 8.31456C130.167 6.26628 132.498 5.27669 134.547 6.10425L140.375 8.45892C142.423 9.28648 143.413 11.6178 142.585 13.6661L127.14 51.8948C125.507 55.935 130.595 59.3029 133.677 56.2217L162.831 27.0669C164.394 25.5048 166.926 25.5048 168.488 27.0669L172.933 31.5116C174.495 33.0737 174.495 35.6063 172.933 37.1684L146.752 63.3494C143.627 66.4748 147.132 71.6128 151.182 69.8432L185.11 55.0196C187.135 54.1351 189.493 55.0592 190.377 57.0835L192.894 62.8435C193.778 64.8679 192.854 67.2259 190.83 68.1104L151.734 85.1917C147.762 86.927 149.001 92.8571 153.335 92.8571H196C198.209 92.8571 200 94.648 200 96.8571V103.143C200 105.352 198.209 107.143 196 107.143H153.336C149.001 107.143 147.763 113.073 151.734 114.808L190.83 131.89C192.854 132.774 193.778 135.132 192.894 137.156L190.377 142.916C189.493 144.941 187.135 145.865 185.11 144.98L151.182 130.157C147.132 128.387 143.627 133.525 146.752 136.651L172.933 162.831C174.495 164.394 174.495 166.926 172.933 168.488L168.488 172.933C166.926 174.495 164.394 174.495 162.831 172.933L133.677 143.778C130.595 140.697 125.507 144.065 127.14 148.105L142.585 186.334C143.413 188.382 142.423 190.713 140.375 191.541L134.547 193.896C132.498 194.723 130.167 193.734 129.339 191.685L114.852 155.827C113.205 151.75 107.143 152.929 107.143 157.325V196C107.143 198.209 105.352 200 103.143 200H96.8571C94.648 200 92.8571 198.209 92.8571 196V157.325C92.8571 152.929 86.7953 151.75 85.1484 155.827L70.6605 191.685C69.8329 193.734 67.5016 194.723 65.4533 193.896L59.6253 191.541C57.577 190.713 56.5875 188.382 57.415 186.334L72.8605 148.105C74.4929 144.065 69.4046 140.697 66.3233 143.778L37.1685 172.933C35.6064 174.495 33.0737 174.495 31.5117 172.933L27.067 168.488C25.5049 166.926 25.5049 164.394 27.067 162.831L53.2479 136.65C56.3733 133.525 52.8683 128.387 48.818 130.157L14.8895 144.98C12.8652 145.865 10.5071 144.941 9.62265 142.916L7.10607 137.156C6.22161 135.132 7.14568 132.774 9.17004 131.89L48.2657 114.808C52.2374 113.073 50.9985 107.143 46.6642 107.143H4C1.79086 107.143 0 105.352 0 103.143V96.8571C0 94.648 1.79086 92.8571 4 92.8571H46.6644C50.9986 92.8571 52.2376 86.927 48.2659 85.1917L9.17001 68.1104C7.14565 67.2259 6.22158 64.8679 7.10604 62.8435L9.62262 57.0835C10.5071 55.0592 12.8651 54.1351 14.8895 55.0196L48.818 69.8432C52.8683 71.6129 56.3733 66.4748 53.2479 63.3494L27.067 37.1685C25.5049 35.6064 25.5049 33.0737 27.067 31.5116L31.5116 27.067C33.0737 25.5049 35.6064 25.5049 37.1685 27.067L66.3233 56.2218C69.4045 59.303 74.4928 55.9351 72.8605 51.8949L57.415 13.6661C56.5875 11.6178 57.5771 9.28648 59.6253 8.45892L65.4534 6.10425C67.5016 5.27669 69.833 6.26628 70.6605 8.31456L85.1484 44.1733C86.7953 48.2497 92.8571 47.0714 92.8571 42.6749V4C92.8571 1.79086 94.648 0 96.8571 0H103.143ZM100 142.857C123.669 142.857 142.857 123.669 142.857 100C142.857 76.3307 123.669 57.1429 100 57.1429C76.3307 57.1429 57.1429 76.3307 57.1429 100C57.1429 123.669 76.3307 142.857 100 142.857Z",
                17: "M107.143 4C107.143 1.79086 105.352 0 103.143 0H96.8571C94.648 0 92.8571 1.79086 92.8571 4V42.6749C92.8571 47.0714 86.7954 48.2497 85.1484 44.1733L70.6605 8.31456C69.833 6.26628 67.5016 5.27669 65.4534 6.10425L59.6253 8.45892C57.5771 9.28648 56.5875 11.6178 57.415 13.6661L72.8605 51.8949C74.4928 55.9351 69.4045 59.303 66.3233 56.2218L37.1685 27.067C35.6064 25.5049 33.0737 25.5049 31.5116 27.067L27.067 31.5116C25.5049 33.0737 25.5049 35.6064 27.067 37.1685L53.2479 63.3494C56.3733 66.4748 52.8683 71.6129 48.818 69.8432L14.8895 55.0196C12.8651 54.1351 10.5071 55.0592 9.62262 57.0835L7.10604 62.8435C6.22158 64.8679 7.14565 67.2259 9.17001 68.1104L48.2659 85.1917C52.2376 86.927 50.9987 92.8571 46.6644 92.8571H4C1.79086 92.8571 0 94.648 0 96.8571V103.143C0 105.352 1.79086 107.143 4 107.143H46.6642C50.9985 107.143 52.2374 113.073 48.2657 114.808L9.17006 131.89C7.1457 132.774 6.22163 135.132 7.10609 137.156L9.62267 142.916C10.5071 144.941 12.8652 145.865 14.8895 144.98L48.818 130.157C52.8683 128.387 56.3733 133.525 53.2479 136.651L27.067 162.831C25.5049 164.394 25.5049 166.926 27.067 168.488L31.5117 172.933C33.0737 174.495 35.6064 174.495 37.1685 172.933L66.3233 143.778C69.4046 140.697 74.4929 144.065 72.8605 148.105L57.415 186.334C56.5875 188.382 57.577 190.713 59.6253 191.541L65.4533 193.896C67.5016 194.723 69.8329 193.734 70.6605 191.685L85.1484 155.827C86.7954 151.75 92.8571 152.929 92.8571 157.325V196C92.8571 198.209 94.648 200 96.8571 200H103.143C105.352 200 107.143 198.209 107.143 196V157.325C107.143 152.929 113.205 151.75 114.852 155.827L129.339 191.685C130.167 193.734 132.498 194.723 134.547 193.896L140.375 191.541C142.423 190.713 143.413 188.382 142.585 186.334L127.14 148.105C125.507 144.065 130.595 140.697 133.677 143.778L162.831 172.933C164.394 174.495 166.926 174.495 168.488 172.933L172.933 168.488C174.495 166.926 174.495 164.394 172.933 162.831L146.752 136.651C143.627 133.525 147.132 128.387 151.182 130.157L185.11 144.98C187.135 145.865 189.493 144.941 190.377 142.916L192.894 137.156C193.778 135.132 192.854 132.774 190.83 131.89L151.734 114.808C147.763 113.073 149.001 107.143 153.336 107.143H196C198.209 107.143 200 105.352 200 103.143V96.8571C200 94.648 198.209 92.8571 196 92.8571H153.336C149.001 92.8571 147.762 86.927 151.734 85.1917L190.83 68.1104C192.854 67.2259 193.778 64.8678 192.894 62.8435L190.377 57.0835C189.493 55.0592 187.135 54.1351 185.11 55.0196L151.182 69.8432C147.132 71.6128 143.627 66.4748 146.752 63.3494L172.933 37.1684C174.495 35.6063 174.495 33.0737 172.933 31.5116L168.488 27.0669C166.926 25.5048 164.394 25.5048 162.831 27.0669L133.677 56.2217C130.595 59.3029 125.507 55.935 127.14 51.8948L142.585 13.6661C143.413 11.6178 142.423 9.28648 140.375 8.45892L134.547 6.10425C132.498 5.27669 130.167 6.26628 129.339 8.31456L114.852 44.1733C113.205 48.2497 107.143 47.0714 107.143 42.6749V4Z",
                18: "M100 200C155.228 200 200 155.228 200 100C200 44.7715 155.228 0 100 0C44.7715 0 0 44.7715 0 100C0 155.228 44.7715 200 100 200ZM100 143.75C124.162 143.75 143.75 124.162 143.75 100C143.75 75.8375 124.162 56.25 100 56.25C75.8375 56.25 56.25 75.8375 56.25 100C56.25 124.162 75.8375 143.75 100 143.75Z",
                19: "M112.548 87.4518C121.709 83.5318 135.093 81.0606 150 81.0606C177.614 81.0606 200 89.5401 200 100C200 110.46 177.614 118.939 150 118.939C135.093 118.939 121.709 116.468 112.548 112.548C116.468 121.709 118.939 135.093 118.939 150C118.939 177.614 110.46 200 100 200C89.5401 200 81.0606 177.614 81.0606 150C81.0606 135.093 83.5318 121.709 87.4518 112.548C78.2912 116.468 64.9075 118.939 50 118.939C22.3858 118.939 0 110.46 0 100C0 89.5401 22.3858 81.0606 50 81.0606C64.9075 81.0606 78.2912 83.5318 87.4518 87.4518C83.5318 78.2912 81.0606 64.9075 81.0606 50C81.0606 22.3858 89.5401 0 100 0C110.46 0 118.939 22.3858 118.939 50C118.939 64.9075 116.468 78.2912 112.548 87.4518Z",
                20: "M100 100C100 100 112.5 66.5265 112.5 42.8571C112.5 19.1878 106.904 0 100 0C93.0964 0 87.5 19.1878 87.5 42.8571C87.5 66.5265 100 100 100 100ZM100 100C100 100 114.831 132.508 131.567 149.245C148.304 165.982 165.829 175.592 170.711 170.711C175.592 165.829 165.982 148.304 149.245 131.567C132.508 114.831 100 100 100 100ZM100 100C100 100 133.474 87.5 157.143 87.5C180.812 87.5 200 93.0964 200 100C200 106.904 180.812 112.5 157.143 112.5C133.474 112.5 100 100 100 100ZM100 100C100 100 67.4918 114.831 50.755 131.567C34.0183 148.304 24.4077 165.829 29.2893 170.711C34.1709 175.592 51.696 165.982 68.4327 149.245C85.1695 132.508 100 100 100 100ZM100 100L100 100C100.028 100.074 112.5 133.5 112.5 157.143C112.5 180.812 106.904 200 100 200C93.0964 200 87.5 180.812 87.5 157.143C87.5 133.474 100 100 100 100ZM100 100C100 100 66.5265 87.5 42.8571 87.5C19.1878 87.5 0 93.0964 0 100C0 106.904 19.1878 112.5 42.8571 112.5C66.5265 112.5 100 100 100 100ZM100 99.9999C100 99.9999 132.508 85.1694 149.245 68.4327C165.982 51.6959 175.592 34.1708 170.711 29.2893C165.829 24.4077 148.304 34.0183 131.567 50.755C114.831 67.4918 100 99.9999 100 99.9999ZM68.4327 50.755C85.1695 67.4918 100 100 100 100C100 100 67.4918 85.1695 50.7551 68.4327C34.0183 51.696 24.4078 34.1709 29.2893 29.2893C34.1709 24.4078 51.696 34.0183 68.4327 50.755Z",
                21: "M200 98.5234C196.477 99.4861 192.769 100 188.941 100H144.233C121.117 100 102.378 118.739 102.378 141.854V191.319C102.378 194.295 102.067 197.199 101.477 200C100.514 196.477 100 192.769 100 188.941V144.233C100 121.117 81.2612 102.378 58.1456 102.378H8.68122C5.70469 102.378 2.80073 102.067 0 101.477C3.52289 100.514 7.23105 100 11.0593 100H55.7675C78.8831 100 97.622 81.2611 97.622 58.1456L97.622 8.68118C97.622 5.70466 97.9327 2.80071 98.5234 0C99.4861 3.5229 100 7.23107 100 11.0593V55.7675C100 78.8831 118.739 97.622 141.855 97.622H191.319C194.295 97.622 197.199 97.9327 200 98.5234Z",
                22: "M188.941 100C192.769 100 196.477 99.4861 200 98.5234C197.199 97.9327 194.295 97.622 191.319 97.622H141.855C133.685 97.622 126.061 95.2811 119.619 91.2334C121.027 82.9896 124.913 75.0866 131.277 68.7229L162.891 37.1094C165.598 34.4024 167.856 31.4169 169.667 28.2452C167.268 29.8079 164.995 31.6416 162.891 33.7463L127.914 68.7229C122.137 74.4999 115.091 78.2351 107.674 79.9285C102.84 73.1033 100 64.7671 100 55.7675V11.0593C100 7.23107 99.4861 3.5229 98.5234 0C97.9327 2.80071 97.622 5.70466 97.622 8.68118L97.622 58.1456C97.622 66.3155 95.2811 73.9388 91.2335 80.3812C82.9897 78.9727 75.0866 75.0865 68.7229 68.7228L37.1094 37.1094C34.4024 34.4024 31.4169 32.1437 28.2452 30.3334C29.8079 32.7315 31.6416 35.0047 33.7463 37.1094L68.7229 72.086C74.4999 77.8631 78.2352 84.9088 79.9285 92.3263C73.1033 97.1596 64.7671 100 55.7675 100H11.0593C7.23105 100 3.52289 100.514 0 101.477C2.80073 102.067 5.70469 102.378 8.68122 102.378H58.1456C66.3156 102.378 73.9388 104.719 80.3812 108.767C78.9727 117.01 75.0866 124.913 68.7229 131.277L37.1094 162.891C34.4025 165.598 32.1438 168.583 30.3335 171.755C32.7316 170.192 35.0047 168.358 37.1094 166.254L72.086 131.277C77.863 125.5 84.9088 121.765 92.3263 120.071C97.1596 126.897 100 135.233 100 144.233V188.941C100 192.769 100.514 196.477 101.477 200C102.067 197.199 102.378 194.295 102.378 191.319V141.854C102.378 133.684 104.719 126.061 108.767 119.619C117.01 121.027 124.913 124.913 131.277 131.277L162.891 162.891C165.598 165.598 168.583 167.856 171.755 169.667C170.192 167.268 168.358 164.995 166.254 162.891L131.277 127.914C125.5 122.137 121.765 115.091 120.072 107.674C126.897 102.84 135.233 100 144.233 100H188.941Z",
                23: "M135.714 0H64.2857V64.2857H3.12224e-06L0 135.714H64.2857V200H135.714V135.714H200V64.2857H135.714V0Z",
                24: "M106.973 87.6003C103.915 93.0385 96.0852 93.0385 93.027 87.6003L50.4686 11.9213C47.4696 6.58851 51.3234 7.97602e-06 57.4416 5.67316e-06L142.558 0C148.677 -5.34872e-07 152.53 6.58849 149.531 11.9213L106.973 87.6003ZM87.6003 106.973C93.0385 103.915 93.0385 96.0851 87.6003 93.0269L11.9213 50.4685C6.58848 47.4696 -1.12708e-05 51.3233 -1.15382e-05 57.4415L-1.52588e-05 142.558C-1.55262e-05 148.677 6.58849 152.53 11.9213 149.531L87.6003 106.973ZM106.973 112.4C103.915 106.961 96.0852 106.962 93.027 112.4L50.4686 188.079C47.4697 193.412 51.3234 200 57.4416 200H142.558C148.677 200 152.53 193.411 149.531 188.079L106.973 112.4ZM112.4 93.027C106.961 96.0853 106.961 103.915 112.4 106.973L188.079 149.531C193.412 152.53 200 148.677 200 142.558V57.4417C200 51.3235 193.411 47.4697 188.079 50.4687L112.4 93.027Z",
                25: "M156.064 143.936L112.127 100L156.064 56.0636L200 100L156.064 143.936ZM43.9364 143.936L0 100L43.9364 56.0636L87.8728 100L43.9364 143.936ZM100 200L56.0636 156.064L100 112.127L143.936 156.064L100 200ZM100 87.8728L56.0636 43.9364L100 0L143.936 43.9364L100 87.8728Z",
                26: "M165.963 134.037C160.496 139.504 151.631 139.504 146.164 134.037L122.027 109.899C116.559 104.432 116.559 95.5678 122.027 90.1005L146.164 65.9631C151.631 60.4957 160.496 60.4957 165.963 65.9631L190.101 90.1004C195.568 95.5678 195.568 104.432 190.101 109.899L165.963 134.037ZM53.8359 134.037C48.3686 139.504 39.5042 139.504 34.0369 134.037L9.89952 109.899C4.43218 104.432 4.43217 95.5678 9.89951 90.1005L34.0369 65.9631C39.5042 60.4957 48.3686 60.4957 53.8359 65.9631L77.9733 90.1004C83.4406 95.5678 83.4406 104.432 77.9733 109.899L53.8359 134.037ZM109.9 190.1C104.432 195.568 95.5679 195.568 90.1005 190.1L65.9631 165.963C60.4958 160.496 60.4958 151.631 65.9631 146.164L90.1005 122.027C95.5679 116.559 104.432 116.559 109.9 122.027L134.037 146.164C139.504 151.631 139.504 160.496 134.037 165.963L109.9 190.1ZM109.9 77.9732C104.432 83.4405 95.5679 83.4406 90.1005 77.9732L65.9631 53.8358C60.4958 48.3685 60.4958 39.5042 65.9631 34.0368L90.1005 9.89946C95.5679 4.43212 104.432 4.43211 109.9 9.89945L134.037 34.0368C139.504 39.5042 139.504 48.3685 134.037 53.8358L109.9 77.9732Z",
                27: "M124.899 4.91713L105.8 85.9981L149.627 15.16C150.985 12.9661 154.033 12.6119 155.857 14.4361L185.564 44.1427C187.388 45.9669 187.034 49.0153 184.84 50.3727L114.002 94.2004L195.083 75.101C197.594 74.5095 200 76.4147 200 78.9945V121.006C200 123.586 197.594 125.491 195.083 124.899L114.002 105.8L184.84 149.628C187.034 150.985 187.388 154.033 185.564 155.858L155.857 185.564C154.033 187.388 150.985 187.034 149.627 184.84L105.8 114.002L124.899 195.083C125.491 197.594 123.585 200 121.006 200H78.9943C76.4145 200 74.5094 197.594 75.1009 195.083L94.2003 114.002L50.3726 184.84C49.0153 187.034 45.9668 187.388 44.1426 185.564L14.4361 155.857C12.6119 154.033 12.9661 150.985 15.16 149.627L85.9975 105.8L4.91714 124.899C2.40606 125.491 0 123.586 0 121.006V78.9944C1.94914e-06 76.4146 2.40605 74.5095 4.91714 75.101L85.998 94.2003L15.16 50.3728C12.9661 49.0154 12.6119 45.9669 14.4361 44.1427L44.1426 14.4362C45.9668 12.612 49.0153 12.9662 50.3726 15.1601L94.2003 85.9983L75.1009 4.91714C74.5094 2.40606 76.4145 3.89828e-06 78.9943 3.67275e-06L121.006 0C123.586 0 125.491 2.40605 124.899 4.91713ZM100 111.429C106.312 111.429 111.429 106.312 111.429 100C111.429 93.6882 106.312 88.5714 100 88.5714C93.6882 88.5714 88.5714 93.6882 88.5714 100C88.5714 106.312 93.6882 111.429 100 111.429Z",
                28: "M200 99.5882C200 99.5882 155.228 150.176 100 150.176C44.7715 150.176 0 99.5882 0 99.5882C0 99.5882 44.7715 49 100 49C155.228 49 200 99.5882 200 99.5882Z",
                29: "M99.9994 153.877C141.147 153.877 176.851 127.54 194.627 111.673C201.789 105.28 201.789 94.597 194.627 88.204C176.851 72.3371 141.147 46 99.9994 46C58.8516 46 23.1479 72.3371 5.37163 88.2041C-1.79055 94.597 -1.79054 105.28 5.37164 111.673C23.1479 127.54 58.8516 153.877 99.9994 153.877ZM99.9994 137.57C120.783 137.57 137.631 120.722 137.631 99.9383C137.631 79.1551 120.783 62.3069 99.9994 62.3069C79.2161 62.3069 62.368 79.1551 62.368 99.9383C62.368 120.722 79.2161 137.57 99.9994 137.57Z",
                30: "M58.658 147.786C57.3447 144.947 55.0568 142.659 52.2173 141.345C30.1168 131.123 12.6336 116.382 3.3014 107.542C-1.10048 103.372 -1.10046 96.6313 3.30141 92.4614C12.6336 83.621 30.1168 68.8801 52.2172 58.6581C55.0568 57.3447 57.3447 55.0568 58.658 52.2173C68.8801 30.1169 83.621 12.6336 92.4614 3.3014C96.6313 -1.10047 103.372 -1.10046 107.542 3.30141C116.382 12.6336 131.123 30.1169 141.345 52.2173C142.659 55.0568 144.947 57.3447 147.786 58.658C169.887 68.8801 187.37 83.621 196.702 92.4614C201.104 96.6313 201.104 103.372 196.702 107.542C187.37 116.382 169.887 131.123 147.786 141.345C144.947 142.659 142.659 144.947 141.345 147.786C131.123 169.887 116.382 187.37 107.542 196.702C103.372 201.104 96.6313 201.104 92.4614 196.702C83.621 187.37 68.8801 169.887 58.658 147.786ZM100.002 136.761C120.303 136.761 136.761 120.303 136.761 100.002C136.761 79.7002 120.303 63.2426 100.002 63.2426C79.7003 63.2426 63.2428 79.7002 63.2428 100.002C63.2428 120.303 79.7003 136.761 100.002 136.761Z",
                31: "M29.2893 29.2893C19.2658 39.3128 15.6458 53.315 18.4294 66.2123C7.34132 73.3638 0 85.8246 0 100C-1.74156e-06 114.175 7.34132 126.636 18.4294 133.788C15.6458 146.685 19.2658 160.687 29.2893 170.711C39.3129 180.734 53.315 184.354 66.2123 181.571C73.3639 192.659 85.8246 200 100 200C114.175 200 126.636 192.659 133.788 181.571C146.685 184.354 160.687 180.734 170.711 170.711C180.734 160.687 184.354 146.685 181.571 133.788C192.659 126.636 200 114.175 200 100C200 85.8246 192.659 73.3638 181.571 66.2123C184.354 53.315 180.734 39.3129 170.711 29.2893C160.687 19.2658 146.685 15.6458 133.788 18.4294C126.636 7.34133 114.175 0 100 0C85.8246 0 73.3638 7.34131 66.2123 18.4293C53.315 15.6458 39.3129 19.2658 29.2893 29.2893Z",
                32: "M127.233 110.308C145.856 117.358 170.923 116.073 186.883 107.25L200 100L186.883 92.7498C170.923 83.9274 145.857 82.6428 127.233 89.6916L113.267 94.9781C113.18 94.6967 113.075 94.4251 112.972 94.1516L126.546 88.0321C144.7 79.8479 161.515 61.2149 166.563 43.6907L170.711 29.2888L156.309 33.4367C138.785 38.4842 120.152 55.3 111.967 73.4535L106.057 86.5642C105.784 86.4316 105.511 86.3 105.229 86.1851L110.308 72.7665C117.357 54.1433 116.072 29.0767 107.25 13.1163L100 0L92.7498 13.1167C83.9274 29.0772 82.6428 54.1433 89.6916 72.767L94.7707 86.1856C94.4884 86.3005 94.2154 86.4321 93.9428 86.5646L88.0321 73.4539C79.8479 55.3005 61.2149 38.4846 43.6907 33.4372L29.2888 29.2893L33.4367 43.6912C38.4842 61.2149 55.3 79.8484 73.4535 88.0326L87.0274 94.1521C86.9247 94.4256 86.8191 94.6972 86.733 94.9786L72.7665 89.6921C54.1433 82.6428 29.0767 83.9279 13.1163 92.7502L0 100L13.1167 107.25C29.0772 116.073 54.1433 117.358 72.767 110.308L87.3861 104.775C87.5014 105.027 87.6172 105.28 87.747 105.524L73.454 111.968C55.3005 120.152 38.4847 138.785 33.4372 156.309L29.2893 170.711L43.6912 166.563C61.2149 161.516 79.8484 144.7 88.0326 126.547L94.6656 111.834C94.9107 111.936 95.1651 112.02 95.4172 112.109L89.6921 127.234C82.6428 145.857 83.9279 170.924 92.7502 186.884L100 200.001L107.251 186.884C116.073 170.924 117.358 145.858 110.309 127.234L104.583 112.108C104.835 112.02 105.09 111.936 105.335 111.833L111.968 126.546C120.152 144.7 138.785 161.515 156.309 166.563L170.711 170.711L166.563 156.309C161.516 138.785 144.7 120.152 126.547 111.967L112.253 105.524C112.383 105.28 112.499 105.027 112.614 104.774L127.233 110.308Z",
                33: "M146.171 100C161.187 97.0302 175.962 89.6912 184.977 80.047L195.212 69.0981L180.496 66.2558C167.534 63.7521 151.267 66.5 137.373 72.9237C147.776 61.6944 155.416 47.0726 157.04 33.9716L158.885 19.0981L145.309 25.4484C133.351 31.0419 121.806 42.826 114.34 56.1898C116.156 40.9907 113.743 24.6707 107.356 13.1167L100.106 0L92.8553 13.1167C86.4688 24.6707 84.0554 40.9907 85.8712 56.1898C78.406 42.826 66.8609 31.0414 54.9028 25.4484L41.327 19.0981L43.1712 33.9716C44.7958 47.0726 52.4358 61.6944 62.8386 72.9237C48.9446 66.5 32.6772 63.7526 19.7153 66.2558L5 69.0981L15.2344 80.047C24.2493 89.6912 39.0242 97.0298 54.0409 100C39.0247 102.97 24.2493 110.308 15.2344 119.953L5 130.902L19.7153 133.744C32.6772 136.248 48.9442 133.5 62.8386 127.076C52.4358 138.306 44.7958 152.927 43.1712 166.028L41.327 180.901L54.9023 174.552C66.8605 168.958 78.4056 157.174 85.8707 143.81C84.0549 159.009 86.4684 175.329 92.8549 186.883L100.105 200L107.355 186.883C113.742 175.329 116.155 159.009 114.34 143.81C121.805 157.174 133.35 168.959 145.308 174.552L158.883 180.901L157.039 166.028C155.414 152.927 147.774 138.306 137.372 127.076C151.266 133.5 167.533 136.247 180.495 133.744L195.21 130.902L184.976 119.953C175.962 110.309 161.187 102.97 146.171 100Z",
                34: "M184.977 80.047C175.962 89.6912 161.187 97.0302 146.171 100C161.187 102.97 175.962 110.309 184.976 119.953L195.21 130.902L180.495 133.744C167.533 136.247 151.266 133.5 137.372 127.076C147.774 138.306 155.414 152.927 157.039 166.028L158.883 180.901L145.308 174.552C133.35 168.959 121.805 157.174 114.34 143.81C116.155 159.009 113.742 175.329 107.355 186.883L100.105 200L92.8549 186.883C86.4684 175.329 84.0549 159.009 85.8707 143.81C78.4056 157.174 66.8605 168.958 54.9023 174.552L41.327 180.901L43.1712 166.028C44.7958 152.927 52.4358 138.306 62.8386 127.076C48.9442 133.5 32.6772 136.248 19.7153 133.744L5 130.902L15.2344 119.953C24.2493 110.308 39.0247 102.97 54.0409 100C39.0242 97.0298 24.2493 89.6912 15.2344 80.047L5 69.0981L19.7153 66.2558C32.6772 63.7526 48.9447 66.5 62.8386 72.9237C52.4358 61.6944 44.7958 47.0726 43.1712 33.9716L41.327 19.0981L54.9028 25.4484C66.8609 31.0414 78.4061 42.826 85.8712 56.1898C84.0554 40.9907 86.4688 24.6707 92.8553 13.1167L100.106 0L107.356 13.1167C113.743 24.6707 116.156 40.9907 114.34 56.1898C121.806 42.826 133.351 31.0419 145.309 25.4484L158.885 19.0981L157.04 33.9716C155.416 47.0726 147.776 61.6944 137.373 72.9237C151.267 66.5 167.534 63.7521 180.496 66.2558L195.212 69.0981L184.977 80.047ZM81.2543 118.696C91.5325 128.974 108.197 128.974 118.475 118.696C128.753 108.418 128.753 91.7533 118.475 81.4751C108.197 71.1969 91.5325 71.1969 81.2543 81.4751C70.9761 91.7533 70.9761 108.418 81.2543 118.696Z",
                35: "M100 200C97.1048 105.262 94.738 102.91 0 100C94.738 97.1048 97.0903 94.738 100 0C102.895 94.738 105.262 97.0903 200 100C105.262 102.91 102.91 105.233 100 200Z",
                36: "M100 0C103.395 53.7596 146.24 96.6052 200 100C146.24 103.395 103.395 146.24 100 200C96.6052 146.24 53.7596 103.395 0 100C53.7596 96.6052 96.6052 53.7596 100 0Z",
                37: "M100 0L101.384 87.0669C101.44 90.5691 105.651 92.3134 108.167 89.8763L170.711 29.2893L110.124 91.8335C107.687 94.3493 109.431 98.5604 112.933 98.6161L200 100L112.933 101.384C109.431 101.44 107.687 105.651 110.124 108.167L170.711 170.711L108.167 110.124C105.651 107.687 101.44 109.431 101.384 112.933L100 200L98.6161 112.933C98.5604 109.431 94.3493 107.687 91.8335 110.124L29.2893 170.711L89.8763 108.167C92.3134 105.651 90.5691 101.44 87.0669 101.384L0 100L87.0669 98.6161C90.5691 98.5604 92.3134 94.3493 89.8763 91.8335L29.2893 29.2893L91.8335 89.8763C94.3493 92.3134 98.5604 90.5691 98.6161 87.0669L100 0Z",
                38: "M100 0L103.215 92.2394L170.711 29.2893L107.761 96.7855L200 100L107.761 103.215L170.711 170.711L103.215 107.761L100 200L96.7855 107.761L29.2893 170.711L92.2394 103.215L0 100L92.2394 96.7855L29.2893 29.2893L96.7855 92.2394L100 0Z",
                39: "M100 0L101.459 91.7276L134.202 6.03074L104.2 92.7254L164.279 23.3956L106.435 94.6006L186.603 50L107.893 97.127L198.481 82.6352L108.4 100L198.481 117.365L107.893 102.873L186.603 150L106.435 105.399L164.279 176.604L104.2 107.275L134.202 193.969L101.459 108.272L100 200L98.5414 108.272L65.798 193.969L95.8 107.275L35.7212 176.604L93.5652 105.399L13.3975 150L92.1066 102.873L1.51923 117.365L91.6 100L1.51923 82.6352L92.1066 97.127L13.3975 50L93.5652 94.6006L35.7212 23.3956L95.8 92.7254L65.798 6.03074L98.5414 91.7276L100 0Z",
                40: "M100 0L102.665 74.6397L120.791 2.18524L107.88 75.7481L140.674 8.64545L112.75 77.9164L158.779 19.0983L117.063 81.0498L174.314 33.0869L120.63 85.0115L186.603 50L123.295 89.6282L195.106 69.0983L124.943 94.6982L199.452 89.5471L125.5 100L199.452 110.453L124.943 105.302L195.106 130.902L123.295 110.372L186.603 150L120.63 114.989L174.314 166.913L117.063 118.95L158.779 180.902L112.75 122.084L140.674 191.355L107.88 124.252L120.791 197.815L102.665 125.36L100 200L97.3345 125.36L79.2088 197.815L92.1201 124.252L59.3263 191.355L87.25 122.084L41.2215 180.902L82.9372 118.95L25.6855 166.913L79.3701 114.989L13.3975 150L76.7046 110.372L4.89435 130.902L75.0572 105.302L0.547813 110.453L74.5 100L0.547813 89.5471L75.0572 94.6982L4.89435 69.0983L76.7046 89.6282L13.3975 50L79.3701 85.0115L25.6855 33.0869L82.9372 81.0498L41.2215 19.0983L87.25 77.9164L59.3263 8.64545L92.1201 75.7481L79.2088 2.18524L97.3345 74.6397L100 0Z",
                41: "M100 0L102.039 58.4951C102.498 71.6824 118.257 78.21 127.907 69.2101L170.711 29.2893L130.79 72.0931C121.79 81.7429 128.318 97.5019 141.505 97.9615L200 100L141.505 102.039C128.318 102.498 121.79 118.257 130.79 127.907L170.711 170.711L127.907 130.79C118.257 121.79 102.498 128.318 102.039 141.505L100 200L97.9615 141.505C97.5019 128.318 81.7429 121.79 72.0931 130.79L29.2893 170.711L69.2101 127.907C78.21 118.257 71.6824 102.498 58.4951 102.039L0 100L58.4951 97.9615C71.6824 97.5019 78.21 81.7429 69.2101 72.0931L29.2893 29.2893L72.0931 69.2101C81.7429 78.21 97.5019 71.6824 97.9615 58.4951L100 0Z",
                42: "M100 0C100.014 36.893 144.613 55.3667 170.711 29.2893C144.633 55.3866 163.107 99.9859 200 100C163.107 100.014 144.633 144.613 170.711 170.711C144.613 144.633 100.014 163.107 100 200C99.9859 163.107 55.3866 144.633 29.2893 170.711C55.3667 144.613 36.893 100.014 0 100C36.893 99.9859 55.3667 55.3866 29.2893 29.2893C55.3866 55.3667 99.9859 36.893 100 0Z",
                43: "M100 173.738C24.3644 236.944 -36.9438 175.636 26.2621 100C-36.9438 24.3644 24.3644 -36.9438 100 26.2621C175.621 -36.9438 236.944 24.3644 173.738 100C236.944 175.578 175.621 236.944 100 173.738Z",
                44: "M100.019 138.694C14.5644 254.761 -54.7168 185.48 61.3282 99.9966C-54.77 14.5263 14.5112 -54.7279 100.019 61.3122C185.473 -54.7678 254.754 14.5263 138.709 99.9966C254.701 185.48 185.42 254.761 100.019 138.694Z",
                45: "M117.981 130.784C171.923 223.072 28.1506 223.072 82.0191 130.784C-0.153363 223.072 -23.0737 200.161 69.221 117.985C-23.0737 171.93 -23.0737 28.1517 69.221 82.0224C-23.0737 -0.153401 -0.153363 -23.0746 82.0191 69.2237C28.0766 -23.0746 171.849 -23.0746 117.981 69.2237C200.153 -23.0746 223.074 -0.153401 130.779 82.0224C223.074 28.0777 223.074 171.856 130.779 117.985C223.074 200.161 200.122 223.072 117.981 130.784Z",
                46: "M127.14 200C99.9942 200 99.9943 167.423 72.8487 167.423C41.6048 167.423 0 158.386 0 127.133C0 99.9885 32.5678 99.9885 32.5678 72.8445C32.5678 41.6139 41.6048 0 72.8602 0C100.006 0 100.006 32.5774 127.151 32.5774C158.384 32.5774 200 41.6139 200 72.8675C200 100.012 167.421 100.012 167.421 127.156C167.409 158.444 158.384 200 127.14 200Z",
                47: "M123.344 200C100 200 100 143.969 76.6558 143.969C49.7872 143.969 0 150.205 0 123.338C0 99.9951 56.0242 99.995 56.0242 76.652C56.0242 49.7946 49.7872 0 76.6558 0C100 0 100 56.0313 123.344 56.0313C150.213 56.0313 200 49.7946 200 76.652C200 99.995 143.966 99.9951 143.966 123.338C143.966 150.205 150.213 200 123.344 200Z",
                48: "M61.3282 99.9966C-54.7168 185.48 14.5644 254.761 100.019 138.694C185.42 254.761 254.701 185.48 138.709 99.9966C254.754 14.5263 185.473 -54.7678 100.019 61.3122C14.5112 -54.7279 -54.7701 14.5263 61.3282 99.9966ZM100 125C113.807 125 125 113.807 125 100C125 86.1929 113.807 75 100 75C86.1929 75 75 86.1929 75 100C75 113.807 86.1929 125 100 125Z",
                49: "M99.9942 183.711C106.781 191.856 113.567 200 127.14 200C158.384 200 167.409 158.444 167.421 127.156C167.421 113.584 175.565 106.798 183.71 100.012C191.855 93.2255 200 86.4395 200 72.8675C200 41.6139 158.384 32.5774 127.151 32.5774C113.579 32.5774 106.792 24.4331 100.006 16.2887C93.2194 8.14435 86.433 0 72.8602 0C41.6048 0 32.5678 41.6139 32.5678 72.8445C32.5678 86.4165 24.4259 93.2025 16.2839 99.9885C8.14196 106.775 0 113.561 0 127.133C0 158.386 41.6048 167.423 72.8487 167.423C86.4215 167.423 93.2079 175.567 99.9942 183.711ZM100 137C120.435 137 137 120.435 137 100C137 79.5655 120.435 63 100 63C79.5655 63 63 79.5655 63 100C63 120.435 79.5655 137 100 137Z",
                50: "M26.2621 100C-36.9438 175.636 24.3644 236.944 100 173.738C175.621 236.944 236.944 175.578 173.738 100C236.944 24.3644 175.621 -36.9438 100 26.2621C24.3644 -36.9438 -36.9438 24.3644 26.2621 100ZM100 137C120.435 137 137 120.435 137 100C137 79.5655 120.435 63 100 63C79.5655 63 63 79.5655 63 100C63 120.435 79.5655 137 100 137Z",
                51: "M82.0191 130.784C28.1506 223.072 171.923 223.072 117.981 130.784C200.122 223.072 223.074 200.161 130.779 117.986C223.074 171.856 223.074 28.0777 130.779 82.0224C223.074 -0.153397 200.153 -23.0746 117.981 69.2237C171.849 -23.0746 28.0766 -23.0746 82.0191 69.2237C-0.153358 -23.0746 -23.0737 -0.153397 69.221 82.0224C-23.0737 28.1517 -23.0737 171.93 69.221 117.986C-23.0737 200.161 -0.153358 223.072 82.0191 130.784ZM100 125C113.807 125 125 113.807 125 100C125 86.1929 113.807 75 100 75C86.1929 75 75 86.1929 75 100C75 113.807 86.1929 125 100 125Z",
                52: "M50.7143 0H0.71429V50C0.71429 75.462 19.7466 96.4788 44.361 99.6002C19.4015 102.402 4.22025e-06 123.578 2.18557e-06 149.286L0 199.286H50C75.462 199.286 96.4788 180.253 99.6002 155.639C102.402 180.599 123.578 200 149.286 200H199.286V150C199.286 124.538 180.253 103.521 155.639 100.4C180.599 97.5984 200 76.422 200 50.7143V0.714286L150 0.714284C124.538 0.714282 103.521 19.7466 100.4 44.361C97.5984 19.4015 76.422 0 50.7143 0Z",
                53: "M100.387 91.8532C96.2415 40.435 53.2 0 0.714282 0C0.714282 52.2424 40.7753 95.1281 91.8532 99.6128C40.435 103.758 -5.33517e-06 146.8 -7.62939e-06 199.286C52.2424 199.286 95.1281 159.225 99.6128 108.147C103.758 159.565 146.8 200 199.286 200C199.286 147.758 159.225 104.872 108.147 100.387C159.565 96.2416 200 53.2 200 0.714286C147.758 0.714283 104.872 40.7753 100.387 91.8532ZM99.9975 100.002C99.9991 100.002 100.001 100.003 100.002 100.003L100.003 99.9975C100.001 99.9975 99.9992 99.9975 99.9975 99.9975C99.9975 99.9991 99.9975 100.001 99.9975 100.002Z",
                54: "M100 33.6449C92.7738 33.6449 86.9159 39.5028 86.9159 46.729H53.271C53.271 20.9213 74.1923 0 100 0C125.808 0 146.729 20.9213 146.729 46.729C146.729 72.5367 125.808 93.4579 100 93.4579V59.8131C107.226 59.8131 113.084 53.9551 113.084 46.729C113.084 39.5028 107.226 33.6449 100 33.6449ZM166.355 100C166.355 92.7738 160.497 86.9159 153.271 86.9159L153.271 53.271C179.079 53.271 200 74.1923 200 100C200 125.808 179.079 146.729 153.271 146.729C127.463 146.729 106.542 125.808 106.542 100H140.187C140.187 107.226 146.045 113.084 153.271 113.084C160.497 113.084 166.355 107.226 166.355 100ZM46.729 113.084C39.5028 113.084 33.6449 107.226 33.6449 100C33.6449 92.7738 39.5028 86.9159 46.729 86.9159C53.9551 86.9159 59.8131 92.7738 59.8131 100H93.4579C93.4579 74.1923 72.5367 53.271 46.729 53.271C20.9213 53.271 0 74.1923 0 100C0 125.808 20.9213 146.729 46.729 146.729V113.084ZM100 166.355C107.226 166.355 113.084 160.497 113.084 153.271H146.729C146.729 179.079 125.808 200 100 200C74.1923 200 53.271 179.079 53.271 153.271C53.271 127.463 74.1923 106.542 100 106.542L100 140.187C92.7738 140.187 86.9159 146.045 86.9159 153.271C86.9159 160.497 92.7738 166.355 100 166.355Z",
                55: "M200 50V4.37114e-06L100 0V49.9803C99.9893 22.3751 77.6077 4.37114e-06 50 4.37114e-06H2.18557e-06V100H50C22.3858 100 -1.20706e-06 122.386 0 150L2.18557e-06 200H100L100 150C100 177.614 122.386 200 150 200H200L200 100H150.02C177.625 99.9893 200 77.6077 200 50Z",
            };
            const centerShapesArray = {
                2: "M16.5962 1.03651L22.9428 7.38312C23.1381 7.57838 23.1381 7.89496 22.9428 8.09022C22.8679 8.16513 22.7712 8.21431 22.6665 8.23067L21.1924 8.46113L15.5356 2.80428L15.7477 1.31935C15.7868 1.04599 16.04 0.856036 16.3134 0.895088C16.4205 0.910388 16.5197 0.960011 16.5962 1.03651ZM4.59487 20.1478C8.31725 16.8163 12.5899 15.82 17.2379 14.6273L17.6843 10.6099L13.3869 6.31241L9.36936 6.7588C8.17674 11.4068 7.18038 15.6795 3.84886 19.4018L2.4541 18.0071C5.28253 14.7072 6.34319 11.0539 7.7574 4.9256L14.1214 4.21849L19.7783 9.87539L19.0711 16.2393C12.9429 17.6535 9.28947 18.7142 5.98964 21.5426L4.59487 20.1478ZM9.87872 14.118C9.09767 13.3369 9.09767 12.0706 9.87872 11.2896C10.6598 10.5085 11.9261 10.5085 12.7071 11.2896C13.4882 12.0706 13.4882 13.3369 12.7071 14.118C11.9261 14.899 10.6598 14.899 9.87872 14.118Z",
                3: "M16.0037 9.41421L7.39712 18.0208L5.98291 16.6066L14.5895 8H7.00373V6H18.0037V17H16.0037V9.41421Z", 
                4: "M5.82843 6.99955L8.36396 9.53509L6.94975 10.9493L2 5.99955L6.94975 1.0498L8.36396 2.46402L5.82843 4.99955H13C17.4183 4.99955 21 8.58127 21 12.9996C21 17.4178 17.4183 20.9996 13 20.9996H4V18.9996H13C16.3137 18.9996 19 16.3133 19 12.9996C19 9.68584 16.3137 6.99955 13 6.99955H5.82843Z", 
                5: "M11 20L1 12L11 4V9C16.5228 9 21 13.4772 21 19C21 19.2727 20.9891 19.5428 20.9677 19.81C19.5055 17.0364 16.6381 15.119 13.313 15.0053L13 15H10.9999L11 20ZM8.99986 13H10.9999L13.0341 13.0003L13.3814 13.0065C14.6657 13.0504 15.9053 13.3165 17.0568 13.7734C15.5898 12.0749 13.4204 11 11 11H9V8.16125L4.20156 12L8.99992 15.8387L8.99986 13Z",
                6: "M16.9337 8.96494C16.426 5.03562 13.0675 2 9 2 4.58172 2 1 5.58172 1 10 1 11.8924 1.65707 13.6313 2.7555 15.0011 3.56351 16.0087 4.00033 17.1252 4.00025 18.3061L4 22H13L13.001 19H15C16.1046 19 17 18.1046 17 17V14.071L18.9593 13.2317C19.3025 13.0847 19.3324 12.7367 19.1842 12.5037L16.9337 8.96494ZM3 10C3 6.68629 5.68629 4 9 4 12.0243 4 14.5665 6.25141 14.9501 9.22118L15.0072 9.66262 16.5497 12.0881 15 12.7519V17H11.0017L11.0007 20H6.00013L6.00025 18.3063C6.00036 16.6672 5.40965 15.114 4.31578 13.7499 3.46818 12.6929 3 11.3849 3 10ZM21.1535 18.1024 19.4893 16.9929C20.4436 15.5642 21 13.8471 21 12.0001 21 10.153 20.4436 8.4359 19.4893 7.00722L21.1535 5.89771C22.32 7.64386 23 9.74254 23 12.0001 23 14.2576 22.32 16.3562 21.1535 18.1024Z",
                7: "M17.0003 13V14C17.0003 16.7696 16.3364 19.445 15.0853 21.8455L14.8585 22.2663L13.1116 21.2924C14.2716 19.2115 14.9211 16.8817 14.9935 14.4559L15.0003 14V13H17.0003ZM11.0003 10H13.0003V14L12.9948 14.3787C12.9153 17.1495 11.9645 19.7731 10.3038 21.928L10.073 22.2189L8.52406 20.9536C10.0408 19.0969 10.9145 16.8017 10.9943 14.3663L11.0003 14V10ZM12.0003 6C14.7617 6 17.0003 8.23858 17.0003 11H15.0003C15.0003 9.34315 13.6571 8 12.0003 8C10.3434 8 9.00025 9.34315 9.00025 11V14C9.00025 16.2354 8.1806 18.3444 6.72928 19.9768L6.51767 20.2067L5.06955 18.8273C6.23328 17.6056 6.92099 16.0118 6.99381 14.3027L7.00025 14V11C7.00025 8.23858 9.23883 6 12.0003 6ZM12.0003 2C16.9708 2 21.0003 6.02944 21.0003 11V14C21.0003 15.6979 20.7985 17.3699 20.4035 18.9903L20.2647 19.5285L18.3349 19.0032C18.726 17.5662 18.9475 16.0808 18.9919 14.5684L19.0003 14V11C19.0003 7.13401 15.8662 4 12.0003 4C10.4279 4 8.97663 4.51841 7.80805 5.39364L6.38308 3.96769C7.92267 2.73631 9.87547 2 12.0003 2ZM4.96794 5.38282L6.39389 6.8078C5.5635 7.91652 5.0543 9.27971 5.00431 10.7593L4.99961 10.999L5.00378 13C5.00378 14.1195 4.73991 15.2026 4.24263 16.1772L4.08648 16.4663L2.34961 15.4747C2.72889 14.8103 2.95077 14.0681 2.99539 13.2924L3.00378 13L3.00361 11C3.00025 8.87522 3.73656 6.92242 4.96794 5.38282Z",
                8: "M12.5002 2C12.2241 2 12.0002 2.22386 12.0002 2.5V12H10.0002V4.5C10.0002 4.22386 9.77634 4 9.5002 4C9.22405 4 9.0002 4.22386 9.0002 4.5V14C8.64653 14 7.00024 14 7.00024 14C6.61911 12.3792 5.64236 11.4407 4.5954 11.3216C4.87926 12.0664 5.36117 13.2592 6.16634 15.0995C7.02511 17.0622 7.89128 18.5218 9.00374 19.4986C10.0783 20.442 11.4586 21 13.5002 21C16.5378 21 19.0002 18.5377 19.0002 15.5002V7C19.0002 6.72386 18.7763 6.5 18.5002 6.5C18.2241 6.5 18.0002 6.72386 18.0002 7V12H16.0002V4C16.0002 3.72386 15.7763 3.5 15.5002 3.5C15.2241 3.5 15.0002 3.72386 15.0002 4V12H13.0002V2.5C13.0002 2.22386 12.7763 2 12.5002 2ZM21.0002 15.5002C21.0002 19.6424 17.6423 23 13.5002 23C11.0417 23 9.17214 22.308 7.68416 21.0015C6.23411 19.7283 5.22528 17.9381 4.33405 15.9012C3.40393 13.7753 2.89004 12.4804 2.60991 11.7235C2.25318 10.7597 2.74616 9.41212 4.08583 9.31846C5.24076 9.23771 6.22061 9.61249 7.0002 10.2587V4.5C7.0002 3.11929 8.11949 2 9.5002 2C9.68522 2 9.86554 2.0201 10.0391 2.05823C10.2477 0.888227 11.2702 0 12.5002 0C13.5602 0 14.4661 0.659694 14.8298 1.59091C15.0431 1.53167 15.268 1.5 15.5002 1.5C16.8809 1.5 18.0002 2.61929 18.0002 4V4.55001C18.1618 4.51722 18.329 4.5 18.5002 4.5C19.8809 4.5 21.0002 5.61929 21.0002 7V15.5002Z",
                9: "M16.5 3C19.5376 3 22 5.5 22 9C22 16 14.5 20 12 21.5C9.5 20 2 16 2 9C2 5.5 4.5 3 7.5 3C9.35997 3 11 4 12 5C13 4 14.64 3 16.5 3ZM12.9339 18.6038C13.8155 18.0485 14.61 17.4955 15.3549 16.9029C18.3337 14.533 20 11.9435 20 9C20 6.64076 18.463 5 16.5 5C15.4241 5 14.2593 5.56911 13.4142 6.41421L12 7.82843L10.5858 6.41421C9.74068 5.56911 8.5759 5 7.5 5C5.55906 5 4 6.6565 4 9C4 11.9435 5.66627 14.533 8.64514 16.9029C9.39 17.4955 10.1845 18.0485 11.0661 18.6038C11.3646 18.7919 11.6611 18.9729 12 19.1752C12.3389 18.9729 12.6354 18.7919 12.9339 18.6038Z",
                10: "M10.6144 17.7956C10.277 18.5682 9.20776 18.5682 8.8704 17.7956L7.99275 15.7854C7.21171 13.9966 5.80589 12.5726 4.0523 11.7942L1.63658 10.7219C.868536 10.381.868537 9.26368 1.63658 8.92276L3.97685 7.88394C5.77553 7.08552 7.20657 5.60881 7.97427 3.75892L8.8633 1.61673C9.19319.821767 10.2916.821765 10.6215 1.61673L11.5105 3.75894C12.2782 5.60881 13.7092 7.08552 15.5079 7.88394L17.8482 8.92276C18.6162 9.26368 18.6162 10.381 17.8482 10.7219L15.4325 11.7942C13.6789 12.5726 12.2731 13.9966 11.492 15.7854L10.6144 17.7956ZM4.53956 9.82234C6.8254 10.837 8.68402 12.5048 9.74238 14.7996 10.8008 12.5048 12.6594 10.837 14.9452 9.82234 12.6321 8.79557 10.7676 7.04647 9.74239 4.71088 8.71719 7.04648 6.85267 8.79557 4.53956 9.82234ZM19.4014 22.6899 19.6482 22.1242C20.0882 21.1156 20.8807 20.3125 21.8695 19.8732L22.6299 19.5353C23.0412 19.3526 23.0412 18.7549 22.6299 18.5722L21.9121 18.2532C20.8978 17.8026 20.0911 16.9698 19.6586 15.9269L19.4052 15.3156C19.2285 14.8896 18.6395 14.8896 18.4628 15.3156L18.2094 15.9269C17.777 16.9698 16.9703 17.8026 15.956 18.2532L15.2381 18.5722C14.8269 18.7549 14.8269 19.3526 15.2381 19.5353L15.9985 19.8732C16.9874 20.3125 17.7798 21.1156 18.2198 22.1242L18.4667 22.6899C18.6473 23.104 19.2207 23.104 19.4014 22.6899ZM18.3745 19.0469 18.937 18.4883 19.4878 19.0469 18.937 19.5898 18.3745 19.0469Z",
            };
            let isMoving = false;
            const svgNS = "http://www.w3.org/2000/svg";
            const outerCircleRadius = 100; // Default radius to 100 if not provided
            const svgSize = outerCircleRadius * 2;
            let svgWidth = 200;
            // Create SVG Container
            const svg = document.createElementNS(svgNS, "svg");
            svg.classList.add('cursor-svg');
            svg.style.position = "absolute";
            svg.style.pointerEvents = "none";
            svg.style.top = '0px';
            svg.style.left = '0px';

            // Handle mouse movement with properly scoped function
            const handleMouseMove = (e: MouseEvent) => {
                if (isMoving) return;
                isMoving = true;
                const rect = targetDiv.getBoundingClientRect();
                const translateX = e.clientX - rect.left - svgWidth / 2;
                const translateY = e.clientY - rect.top - svgWidth / 2;

                requestAnimationFrame(() => {
                    svg.style.transform = `translate(${translateX}px, ${translateY}px)`;
                    isMoving = false;
                });
            };

              // Center cursor function
              const centerCursor = () => {
                const rect = targetDiv.getBoundingClientRect();
                const centerX = rect.width / 2;
                const centerY = rect.height / 2;
                const widthAttribute = svg.getAttribute("width");
                svgWidth = widthAttribute ? parseFloat(widthAttribute) : svgSize;
                const translateX = centerX - svgWidth / 2;
                const translateY = centerY - svgWidth / 2;
                 requestAnimationFrame(() => {
                  svg.style.transform = `translate(${translateX}px, ${translateY}px)`;
               });
        };

            
            
            const createCursor = (cursorData: DionytrisCursorData) =>{
                const { centerShape, middleShape, outerShape, text, svgShape } = cursorData.cursor;
            
                // Main SVG
                if(svgShape) {
                    svgWidth = svgShape.width;
                    svg.setAttribute("width", `${svgShape.width}px`);
                    svg.setAttribute("height", `${svgShape.width}px`);
                    svg.style.transition = svgShape.transition || "none";
                }
                // Set SVG attributes
                svg.setAttribute("viewBox", `0 0 ${svgSize} ${svgSize}`);
                svg.setAttribute("fill", "none");
                
                // Outer Shape
                if(outerShape && (outerShape.visible === true || String(outerShape.visible).toLowerCase() === 'true')) {
                    const outerCircle = document.createElementNS(svgNS, "circle");
                    outerCircle.setAttribute("class", "outer-circle");
                    outerCircle.setAttribute("cx", outerCircleRadius.toString()); 
                    outerCircle.setAttribute("cy", outerCircleRadius.toString()); 
                    outerCircle.setAttribute("r", outerShape.radius.toString());
                    outerCircle.setAttribute("fill", outerShape.fill || "black");
                    outerCircle.style.opacity = (outerShape.opacity !== undefined ? outerShape.opacity.toString() : "1"); 
                    outerCircle.setAttribute(
                        "style",
                        `mix-blend-mode: ${outerShape.blendingMode || "normal"};`
                    );
                    svg.appendChild(outerCircle);
                }
                
                // Middle Shape
                if (middleShape && (middleShape.visible === true || String(middleShape.visible).toLowerCase() === 'true')) {
                    const middleGroup = document.createElementNS(svgNS, "g");
                    middleGroup.setAttribute("class", "middle-group");
                    middleGroup.setAttribute("data-type", middleShape.type);
                    middleGroup.setAttribute("transform", `scale(${middleShape.scale || 0.5})`);
                    middleGroup.style.transformOrigin = "center";
            
                    const middlePath = document.createElementNS(svgNS, "path");
                    middlePath.setAttribute("fill", middleShape.fill || "red");
                    middlePath.setAttribute(
                        "style",
                        `mix-blend-mode: ${middleShape.blendingMode || "normal"};`
                    );
                    middlePath.setAttribute("opacity", middleShape.opacity !== undefined && middleShape.opacity !== null ? middleShape.opacity : "1");
            
                    if (middleShapesArray[middleShape.type]) {
                        // Set the 'd' attribute based on middleShape.type
                        const pathData = middleShapesArray[middleShape.type];
                        middlePath.setAttribute("d", pathData);
                    } 
                    middleGroup.appendChild(middlePath);
                
                    // Apply spin animation if enabled
                    if (middleShape.spin) {
                    const spinDirection = middleShape.spinDirection === "right" ? "normal" : "reverse";
                    const spinDuration = middleShape.spinDuration || "12s";
                    middlePath.style.animation = `spin ${spinDuration} linear infinite ${spinDirection}`;
                    middlePath.style.transformOrigin = "center";
                    }
                    svg.appendChild(middleGroup);
                }
                
                // Center Shape
                if (centerShape && (centerShape.visible === true || String(centerShape.visible).toLowerCase() === 'true')) {
                    if (centerShape.type === 1) {
                        // Add center circle for type 1
                        const centerCircle = document.createElementNS(svgNS, "circle");
                        centerCircle.setAttribute("cx", outerCircleRadius.toString());
                        centerCircle.setAttribute("cy", outerCircleRadius.toString());
                        centerCircle.setAttribute("r",  `${centerShape.radius || "2"}`);
                        centerCircle.setAttribute("fill", centerShape.fill || "white");
                        centerCircle.style.opacity = (centerShape.opacity !== undefined ? centerShape.opacity.toString() : "1");
            
                        // Create a new group for the shape
                        const shapeGroup = document.createElementNS(svgNS, "g");
                        shapeGroup.setAttribute("class", "center-group");
                        shapeGroup.setAttribute("data-type",  `${centerShape.type}` );
                        shapeGroup.appendChild(centerCircle);
            
                        svg.appendChild(shapeGroup);
                    } 
                    else if (centerShapesArray[centerShape.type]) {
                        // Create a new group for the shape
                        const shapeGroup = document.createElementNS(svgNS, "g");
                        shapeGroup.setAttribute("class", "center-group");
                        shapeGroup.setAttribute("data-type",  `${centerShape.type}`);
                    
                        // Create a new path element and set its 'd' attribute from the centerShapesArray
                        const shapePath = document.createElementNS(svgNS, "path");
                        const pathData = centerShapesArray[centerShape.type];
                    
                        // Set the 'd' attribute for the path from the data
                        shapePath.setAttribute("d", pathData);
                    
                        // Apply the fill color to the path
                        shapePath.setAttribute("fill", centerShape.fill || "white");
                    
                        // Append the path to the group
                        shapeGroup.appendChild(shapePath);
                    
                        // Append the shapeGroup to the SVG first to ensure it is rendered
                        svg.appendChild(shapeGroup);
                    
                        // Use requestAnimationFrame to ensure the element is rendered before calculating the bounding box
                        requestAnimationFrame(() => {
                            // Get the bounding box of the shape (now it should be correctly calculated)
                            const bbox = shapeGroup.getBBox();
                    
                            // Calculate the center of the path
                            const centerX = bbox.x + bbox.width / 2;
                            const centerY = bbox.y + bbox.height / 2;
                    
                            // Calculate the offset needed to center the shape within the outer circle
                            const offsetX = outerCircleRadius - centerX;
                            const offsetY = outerCircleRadius - centerY;
                    
                            // Apply the translation to the shape group to center it
                            shapeGroup.setAttribute('transform', `translate(${offsetX}, ${offsetY})`);
                        });
                    }
                }
                
                // Text Path and Circular Text
                if (text && (text.visible === true || String(text.visible).toLowerCase() === 'true')) {
                    // Create a group (<g>) to contain the text and path
                    const textGroup = document.createElementNS(svgNS, "g");
                    
                    // Apply spin animation to the group if enabled
                    if (text.spin) {
                        const spinDirection = text.spinDirection === "right" ? "normal" : "reverse";
                        const spinDuration = text.spinDuration || "12s";
                        textGroup.style.animation = `spin ${spinDuration} linear infinite ${spinDirection}`;
                        textGroup.style.transformOrigin = "center";
                    }
                    
                    // Text path for circular text
                    const textPath = document.createElementNS(svgNS, "path");
                    textPath.setAttribute("id", "circleTextPath");
                    textPath.setAttribute("d", "M 100 30 A 70 70 0 1 1 99.9 30");
                    textPath.setAttribute("fill", "none");
                    textGroup.appendChild(textPath);
                    
                    // Text element for the circular text
                    const textElement = document.createElementNS(svgNS, "text");
                    textElement.setAttribute("class", "circular-text");
                    textElement.style.fontSize = String(text.fontSize) || "14px";
                    textElement.style.fontWeight = String(text.fontWeight) || "normal";
                    textElement.setAttribute("fill", text.fill || "white");
                    textElement.style.opacity = String(text.opacity) || "1";
                    textElement.setAttribute("text-anchor", "middle");
                    
                    const textPathElement = document.createElementNS(svgNS, "textPath");
                    textPathElement.setAttribute("class", "circular-text-path");
                    textPathElement.setAttribute("href", "#circleTextPath");
                    textPathElement.setAttribute("startOffset", "50%");
                    textPathElement.textContent = text.content;
                    
                    textElement.appendChild(textPathElement);
                    textGroup.appendChild(textElement);
                    
                    // Append the group to the SVG
                    svg.appendChild(textGroup);
                }
                
                // Add SVG to the Document Body
                targetDiv.appendChild(svg);

                
                // Center the cursor after creation
                centerCursor();
               

                // Add event listeners
                targetDiv.addEventListener("mousemove", handleMouseMove);
                targetDiv.addEventListener("mouseout", centerCursor);

                // Add style tag with data attribute for easy cleanup
                const style = document.createElement("style");
                style.setAttribute('data-cursor-animation', 'true');
                style.textContent = `
                    @keyframes spin {
                        from { transform: rotate(0deg); }
                        to { transform: rotate(360deg); }
                    }
                `;
                document.head.appendChild(style);
            }
            createCursor(cursor);

            // Return cleanup function
            return cleanup;
        }
        // Aroneko
        if (cursor.id === 8) {
            if (aronekoRef.current) {
                const targetDiv = aronekoRef.current;

                const isReducedMotion = window.matchMedia(`(prefers-reduced-motion: reduce)`).matches;

                if (isReducedMotion) return;

                const nekoEl = document.createElement("div");

                const wrapperRect = targetDiv.getBoundingClientRect();
                
                // Initialize cat at center of wrapper
                // let nekoPosX = wrapperRect.left + wrapperRect.width / 2;
                // let nekoPosY = wrapperRect.top + wrapperRect.height / 2;
                let nekoPosX = wrapperRect.width / 2;  // Changed: removed wrapperRect.left
                let nekoPosY = wrapperRect.height / 2; 

                let mousePosX = nekoPosX;
                let mousePosY = nekoPosY;

                let frameCount = 0;
                let idleTime = 0;
                let idleAnimation = null;
                let idleAnimationFrame = 0;

                const nekoSpeed = 10;
                const spriteSets = {
                    idle: [[-3, -3]],
                    alert: [[-7, -3]],
                    scratchSelf: [
                    [-5, 0],
                    [-6, 0],
                    [-7, 0],
                    ],
                    scratchWallN: [
                    [0, 0],
                    [0, -1],
                    ],
                    scratchWallS: [
                    [-7, -1],
                    [-6, -2],
                    ],
                    scratchWallE: [
                    [-2, -2],
                    [-2, -3],
                    ],
                    scratchWallW: [
                    [-4, 0],
                    [-4, -1],
                    ],
                    tired: [[-3, -2]],
                    sleeping: [
                    [-2, 0],
                    [-2, -1],
                    ],
                    N: [
                    [-1, -2],
                    [-1, -3],
                    ],
                    NE: [
                    [0, -2],
                    [0, -3],
                    ],
                    E: [
                    [-3, 0],
                    [-3, -1],
                    ],
                    SE: [
                    [-5, -1],
                    [-5, -2],
                    ],
                    S: [
                    [-6, -3],
                    [-7, -2],
                    ],
                    SW: [
                    [-5, -3],
                    [-6, -1],
                    ],
                    W: [
                    [-4, -2],
                    [-4, -3],
                    ],
                    NW: [
                    [-1, 0],
                    [-1, -1],
                    ],
                };

                function init() {
                    nekoEl.id = "oneko";
                    nekoEl.ariaHidden = true;
                    nekoEl.style.width = "32px";
                    nekoEl.style.height = "32px";
                    nekoEl.style.position = "absolute"; // Changed from 'fixed' to 'absolute'
                    nekoEl.style.pointerEvents = "none";
                    nekoEl.style.imageRendering = "pixelated";
                    nekoEl.style.left = `${nekoPosX - 16}px`;
                    nekoEl.style.top = `${nekoPosY - 16}px`;
                    nekoEl.style.zIndex = 99;

                    let nekoFile = pluginUrl + "/public/js/oneko.gif";
                    const curScript = document.currentScript
                    if (curScript && curScript.dataset.cat) {
                    nekoFile = curScript.dataset.cat
                    }
                    nekoEl.style.backgroundImage = `url(${nekoFile})`;

                    targetDiv.style.position = 'relative'; // Ensure wrapper has relative positioning
                    targetDiv.appendChild(nekoEl);

                    targetDiv.addEventListener("mousemove", handleMouseMove);
                    targetDiv.addEventListener("mouseout", handleMouseOut);

                    window.requestAnimationFrame(onAnimationFrame);
                }

                function handleMouseMove(event) {
                    const rect = targetDiv.getBoundingClientRect();
                    mousePosX = event.clientX - rect.left;
                    mousePosY = event.clientY - rect.top;
                }
        
                function handleMouseOut() {
                    mousePosX = wrapperRect.width / 2;
                    mousePosY = wrapperRect.height / 2;
                }

                let lastFrameTimestamp;

                function onAnimationFrame(timestamp) {
                    if (!nekoEl.isConnected) {
                    return;
                    }
                    if (!lastFrameTimestamp) {
                    lastFrameTimestamp = timestamp;
                    }
                    if (timestamp - lastFrameTimestamp > 100) {
                    lastFrameTimestamp = timestamp
                    frame()
                    }
                    window.requestAnimationFrame(onAnimationFrame);
                }

                function setSprite(name, frame) {
                    const sprite = spriteSets[name][frame % spriteSets[name].length];
                    nekoEl.style.backgroundPosition = `${sprite[0] * 32}px ${sprite[1] * 32}px`;
                }

                function resetIdleAnimation() {
                    idleAnimation = null;
                    idleAnimationFrame = 0;
                }

                function idle() {
                    idleTime += 1;

                    if (idleTime > 10 && Math.floor(Math.random() * 200) == 0 && idleAnimation == null) {
                    let avalibleIdleAnimations = ["sleeping", "scratchSelf"];
                    if (nekoPosX < 32) {
                        avalibleIdleAnimations.push("scratchWallW");
                    }
                    if (nekoPosY < 32) {
                        avalibleIdleAnimations.push("scratchWallN");
                    }
                    if (nekoPosX > wrapperRect.width - 32) {
                        avalibleIdleAnimations.push("scratchWallE");
                    }
                    if (nekoPosY > wrapperRect.height - 32) {
                        avalibleIdleAnimations.push("scratchWallS");
                    }
                    idleAnimation = avalibleIdleAnimations[Math.floor(Math.random() * avalibleIdleAnimations.length)];
                    }

                    switch (idleAnimation) {
                    case "sleeping":
                        if (idleAnimationFrame < 8) {
                        setSprite("tired", 0);
                        break;
                        }
                        setSprite("sleeping", Math.floor(idleAnimationFrame / 4));
                        if (idleAnimationFrame > 192) {
                        resetIdleAnimation();
                        }
                        break;
                    case "scratchWallN":
                    case "scratchWallS":
                    case "scratchWallE":
                    case "scratchWallW":
                    case "scratchSelf":
                        setSprite(idleAnimation, idleAnimationFrame);
                        if (idleAnimationFrame > 9) {
                        resetIdleAnimation();
                        }
                        break;
                    default:
                        setSprite("idle", 0);
                        return;
                    }
                    idleAnimationFrame += 1;
                }

                function frame() {
                    frameCount += 1;
                    const diffX = nekoPosX - mousePosX;
                    const diffY = nekoPosY - mousePosY;
                    const distance = Math.sqrt(diffX ** 2 + diffY ** 2);

                    if (distance < nekoSpeed || distance < 48) {
                    idle();
                    return;
                    }

                    idleAnimation = null;
                    idleAnimationFrame = 0;

                    if (idleTime > 1) {
                    setSprite("alert", 0);
                    idleTime = Math.min(idleTime, 7);
                    idleTime -= 1;
                    return;
                    }

                    let direction;
                    direction = diffY / distance > 0.5 ? "N" : "";
                    direction += diffY / distance < -0.5 ? "S" : "";
                    direction += diffX / distance > 0.5 ? "W" : "";
                    direction += diffX / distance < -0.5 ? "E" : "";
                    setSprite(direction, frameCount);

                    nekoPosX -= (diffX / distance) * nekoSpeed;
                    nekoPosY -= (diffY / distance) * nekoSpeed;

                    // Constrain cat position to wrapper boundaries
                    nekoPosX = Math.min(Math.max(16, nekoPosX), wrapperRect.width - 16);
                    nekoPosY = Math.min(Math.max(16, nekoPosY), wrapperRect.height - 16);

                    nekoEl.style.left = `${nekoPosX - 16}px`;
                    nekoEl.style.top = `${nekoPosY - 16}px`;
                }

                init();
                // Cleanup function to prevent duplicates
                return () => {
                    if (nekoEl.isConnected) {
                    nekoEl.remove();
                    }
                    targetDiv.removeEventListener("mousemove", handleMouseMove);
                    targetDiv.removeEventListener("mouseout", handleMouseOut);
                };
            }
        }
        //Gaialoop
        if (cursor.id === 15) {
            const SVGNS = 'http://www.w3.org/2000/svg';
            const EASE = cursor.cursor.config?.ease ?? 0.7;
            let svgElements = []; // Track all created SVG elements
    
            if (gaialoopRef.current) {
                const targetDiv = gaialoopRef.current;
                const rect = targetDiv.getBoundingClientRect();
    
                const pointer = {
                    x: rect.width / 2,
                    y: rect.height / 2,
                };
    
                function updatePointer(event) {
                    // Calculate position relative to the container
                    const bounds = targetDiv.getBoundingClientRect();
                    pointer.x = event.clientX - bounds.left;
                    pointer.y = event.clientY - bounds.top;
                }

                function resetPointer() {
                    // Reset pointer to the center of the wrapper
                    pointer.x = rect.width / 2;
                    pointer.y = rect.height / 2;
                }
    
                function createLine(leader) {
                    const line = document.createElementNS(SVGNS, 'line');
                    let currentX = pointer.x;
                    let currentY = pointer.y;
                    
                    line.setAttribute('x1', '0');
                    line.setAttribute('y1', '0');
                    line.setAttribute('x2', '0');
                    line.setAttribute('y2', '0');
                    
                    function updatePosition() {
                        const leaderX = typeof leader === 'function' ? leader('x') : pointer.x;
                        const leaderY = typeof leader === 'function' ? leader('y') : pointer.y;
                        
                        currentX += (leaderX - currentX) * EASE;
                        currentY += (leaderY - currentY) * EASE;
                        
                        line.setAttribute('transform', `translate(${currentX},${currentY})`);
                        line.setAttribute('x2', leaderX - currentX);
                        line.setAttribute('y2', leaderY - currentY);
                    }
                    
                    function animate() {
                        updatePosition();
                        requestAnimationFrame(animate);
                    }
                    
                    animate();
                    
                    return {
                        element: line,
                        getProperty: (prop) => prop === 'x' ? currentX : currentY
                    };
                }
    
                function createTrail(color, length = 10) {
                    const svg = document.createElementNS(SVGNS, 'svg');
                    // Set position to absolute instead of fixed
                    svg.style.position = 'absolute';
                    svg.style.top = '0';
                    svg.style.left = '0';
                    // Use container dimensions instead of 100%
                    svg.style.width = `${rect.width}px`;
                    svg.style.height = `${rect.height}px`;
                    svg.style.pointerEvents = 'none';
                    
                    let leader = (prop) => prop === 'x' ? pointer.x : pointer.y;
                    const lines = [];
                    
                    for (let i = 0; i < length; i++) {
                        const lineObj = createLine(leader);
                        const line = lineObj.element;
                        
                        line.style.stroke = color;
                        line.style.strokeWidth = cursor.cursor.config?.width ?? '60';
                        line.style.strokeLinecap = 'round';
                        svg.appendChild(line);
                        
                        lines.push(lineObj);
                        leader = lineObj.getProperty;
                    }
    
                    if (targetDiv) {
                        // Ensure container has position relative
                        targetDiv.style.position = 'relative';
                        targetDiv.appendChild(svg);
                    }
                }
    
                if (targetDiv) {
                    targetDiv.addEventListener('pointerdown', updatePointer);
                    targetDiv.addEventListener('pointermove', updatePointer);
                    targetDiv.addEventListener('mouseout', resetPointer);
                }
                const color1 = cursor.cursor.shape1?.color ?? 'blue';
                const color2 = cursor.cursor.shape2?.color ?? 'yellow';
                const color3 = cursor.cursor.shape3?.color ?? 'red';
    
                const length1 = cursor.cursor.shape1?.length ?? 30;
                const length2 = cursor.cursor.shape2?.length ?? 20;
                const length3 = cursor.cursor.shape3?.length ?? 10;
                createTrail(color1, length1);
                createTrail(color2, length2);
                createTrail(color3, length3);
    
                // Cleanup function
                return () => {
                    if (targetDiv) {
                        const svgElements = targetDiv.querySelectorAll('svg');
                        svgElements.forEach(svg => svg.remove());
                        
                        targetDiv.removeEventListener('pointerdown', updatePointer);
                        targetDiv.removeEventListener('pointermove', updatePointer);
                        targetDiv.removeEventListener('mouseout', resetPointer);
                    }
                };
            }
        }
        //Demetrobyte
        if (cursor.id === 13) {
            if (demetrobyteRef.current) {
                const targetDiv = demetrobyteRef.current;

                const cleanup = () => {
                    const cursorEl = document.querySelector('.cursor');
                    if (cursorEl) {
                        cursorEl.remove();  
                    }
                    else return; 
                };

                // Cleanup old cursor
                cleanup();

                // Returns the mouse position
                const getMousePos = e => {
                    return { 
                        x: e.clientX, 
                        y: e.clientY 
                    };
                };

                // Returns the window width and height
                const getWinSize = () => {
                    return { 
                        width: window.innerWidth, 
                        height: window.innerHeight 
                    };
                };

                const isFirefox = () => navigator.userAgent.toLowerCase().indexOf('firefox') > -1;

                // Initialize mouse position object
                let mousepos = { x: 0, y: 0 };

                // Update 'mousepos' with the current mouse position
                const updateMousePos = ev => {
                    mousepos = getMousePos(ev);
                };

                // Listen for 'mousemove' and 'pointermove' events and update 'mousepos' accordingly
                targetDiv.addEventListener('mousemove', updateMousePos);
                targetDiv.addEventListener('pointermove', updateMousePos, { passive: true });

                // Initialize window size object
                let winsize = getWinSize();

                // Recalculate window size on 'resize' event
                window.addEventListener('resize', ev => {
                    winsize = getWinSize();
                });

                // Create the cursor DOM structure dynamically
                const createCursorDOM = (style = 1) => {
                    const cursorEl = document.createElement('div');
                    cursorEl.classList.add('cursor');

                    if (style < 1 || style > 8) {
                        style = 1;
                    }

                    switch (style) {
                        case 1:
                            const innerEl = document.createElement('div');
                            innerEl.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl);

                            const svgEl = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl.setAttribute('version', '1.1');

                            const defsEl = document.createElementNS('http://www.w3.org/2000/svg', 'defs');
                            const filterEl = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl.setAttribute('id', 'gooey');

                            const feGaussianBlurEl = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl.setAttribute('in', 'SourceGraphic');
                            feGaussianBlurEl.setAttribute('result', 'blur');
                            feGaussianBlurEl.setAttribute('stdDeviation', '3.2');

                            const feColorMatrixEl = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl.setAttribute('in', 'blur');
                            feColorMatrixEl.setAttribute('mode', 'matrix');
                            feColorMatrixEl.setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 20 -7');
                            feColorMatrixEl.setAttribute('result', 'goo');

                            const feCompositeEl = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                            feCompositeEl.setAttribute('in', 'SourceGraphic');
                            feCompositeEl.setAttribute('in2', 'goo');
                            feCompositeEl.setAttribute('operator', 'atop');

                            filterEl.appendChild(feGaussianBlurEl);
                            filterEl.appendChild(feColorMatrixEl);
                            filterEl.appendChild(feCompositeEl);
                            defsEl.appendChild(filterEl);
                            svgEl.appendChild(defsEl);

                            cursorEl.appendChild(svgEl);
                        break;
                        case 2:
                            // Create the inner container
                            const innerEl2 = document.createElement('div');
                            innerEl2.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl2);

                            // Create the SVG element
                            const svgEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl2.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl2.setAttribute('version', '1.1');

                            // Create the <defs> element
                            const defsEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                            // Create the filter
                            const filterEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl2.setAttribute('id', 'gooey');

                            // Add the Gaussian Blur
                            const feGaussianBlurEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl2.setAttribute('in', 'SourceGraphic');
                            feGaussianBlurEl2.setAttribute('result', 'blur');
                            feGaussianBlurEl2.setAttribute('stdDeviation', '18');
                            filterEl2.appendChild(feGaussianBlurEl2);

                            // Add the Color Matrix
                            const feColorMatrixEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl2.setAttribute('in', 'blur');
                            feColorMatrixEl2.setAttribute('mode', 'matrix');
                            feColorMatrixEl2.setAttribute(
                                'values',
                                '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 20 -7'
                            );
                            feColorMatrixEl2.setAttribute('result', 'goo');
                            filterEl2.appendChild(feColorMatrixEl2);

                            // Add the Composite Filter
                            const feCompositeEl2 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                            feCompositeEl2.setAttribute('in', 'SourceGraphic');
                            feCompositeEl2.setAttribute('in2', 'goo');
                            feCompositeEl2.setAttribute('operator', 'atop');
                            filterEl2.appendChild(feCompositeEl2);

                            // Append the filter to <defs>
                            defsEl2.appendChild(filterEl2);

                            // Append <defs> to the SVG
                            svgEl2.appendChild(defsEl2);

                            // Append the SVG to the cursor element
                            cursorEl.appendChild(svgEl2);
                        break;
                        case 3:
                            // Create the inner container
                            const innerEl3 = document.createElement('div');
                            innerEl3.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl3);

                            // Create the SVG element
                            const svgEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl3.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl3.setAttribute('version', '1.1');

                            // Create the <defs> element
                            const defsEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                            // Create the filter
                            const filterEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl3.setAttribute('id', 'gooey');

                            // Add feTurbulence
                            const feTurbulenceEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feTurbulence');
                            feTurbulenceEl3.setAttribute('type', 'fractalNoise');
                            feTurbulenceEl3.setAttribute('baseFrequency', '0.02');
                            feTurbulenceEl3.setAttribute('numOctaves', '3');
                            feTurbulenceEl3.setAttribute('result', 'turbulence');
                            filterEl3.appendChild(feTurbulenceEl3);

                            // Add feDisplacementMap
                            const feDisplacementMapEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feDisplacementMap');
                            feDisplacementMapEl3.setAttribute('in', 'SourceGraphic');
                            feDisplacementMapEl3.setAttribute('in2', 'turbulence');
                            feDisplacementMapEl3.setAttribute('scale', '30');
                            feDisplacementMapEl3.setAttribute('xChannelSelector', 'R');
                            feDisplacementMapEl3.setAttribute('yChannelSelector', 'G');
                            feDisplacementMapEl3.setAttribute('result', 'displacement');
                            filterEl3.appendChild(feDisplacementMapEl3);

                            // Add feGaussianBlur
                            const feGaussianBlurEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl3.setAttribute('in', 'displacement');
                            feGaussianBlurEl3.setAttribute('stdDeviation', '10');
                            feGaussianBlurEl3.setAttribute('result', 'blur');
                            filterEl3.appendChild(feGaussianBlurEl3);

                            // Add feColorMatrix
                            const feColorMatrixEl3 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl3.setAttribute('in', 'blur');
                            feColorMatrixEl3.setAttribute('mode', 'matrix');
                            feColorMatrixEl3.setAttribute(
                                'values',
                                `
                                1 0 0 0 0
                                0 1 0 0 0
                                0 0 1 0 0
                                0 0 0 20 -7`
                            );
                            feColorMatrixEl3.setAttribute('result', 'gooey');
                            filterEl3.appendChild(feColorMatrixEl3);

                            // Append the filter to <defs>
                            defsEl3.appendChild(filterEl3);

                            // Append <defs> to the SVG
                            svgEl3.appendChild(defsEl3);

                            // Append the SVG to the cursor element
                            cursorEl.appendChild(svgEl3);
                        break;
                        case 4:
                            // Create the inner container
                            const innerEl4 = document.createElement('div');
                            innerEl4.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl4);

                            // Create the SVG element
                            const svgEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl4.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl4.setAttribute('version', '1.1');

                            // Create the <defs> element
                            const defsEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                            // Create the filter element
                            const filterEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl4.setAttribute('id', 'gooey');

                            // Add feGaussianBlur
                            const feGaussianBlurEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl4.setAttribute('in', 'SourceGraphic');
                            feGaussianBlurEl4.setAttribute('stdDeviation', '10');
                            feGaussianBlurEl4.setAttribute('result', 'blur');
                            filterEl4.appendChild(feGaussianBlurEl4);

                            // Add feColorMatrix
                            const feColorMatrixEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl4.setAttribute('in', 'blur');
                            feColorMatrixEl4.setAttribute('mode', 'matrix');
                            feColorMatrixEl4.setAttribute(
                                'values',
                                `
                                1 0 0 0 0
                                0 1 0 0 0
                                0 0 1 0 0
                                0 0 0 20 -7`
                            );
                            feColorMatrixEl4.setAttribute('result', 'gooey');
                            filterEl4.appendChild(feColorMatrixEl4);

                            // Add feMorphology
                            const feMorphologyEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feMorphology');
                            feMorphologyEl4.setAttribute('in', 'gooey');
                            feMorphologyEl4.setAttribute('operator', 'dilate');
                            feMorphologyEl4.setAttribute('radius', '2');
                            feMorphologyEl4.setAttribute('result', 'outline');
                            filterEl4.appendChild(feMorphologyEl4);

                            // Add feComposite
                            const feCompositeEl4 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                            feCompositeEl4.setAttribute('in', 'outline');
                            feCompositeEl4.setAttribute('in2', 'gooey');
                            feCompositeEl4.setAttribute('operator', 'out');
                            feCompositeEl4.setAttribute('result', 'composite');
                            filterEl4.appendChild(feCompositeEl4);

                            // Append the filter to <defs>
                            defsEl4.appendChild(filterEl4);

                            // Append <defs> to the SVG
                            svgEl4.appendChild(defsEl4);

                            // Append the SVG to the cursor element
                            cursorEl.appendChild(svgEl4);
                        break;
                        case 5:
                            // Create the inner container
                            const innerEl5 = document.createElement('div');
                            innerEl5.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl5);

                            // Create the SVG element
                            const svgEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl5.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl5.setAttribute('version', '1.1');

                            // Create the <defs> element
                            const defsEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                            // Create the filter element
                            const filterEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl5.setAttribute('id', 'gooey');

                            // Add feTurbulence
                            const feTurbulenceEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feTurbulence');
                            feTurbulenceEl5.setAttribute('type', 'fractalNoise');
                            feTurbulenceEl5.setAttribute('baseFrequency', '0.1');
                            feTurbulenceEl5.setAttribute('numOctaves', '1');
                            feTurbulenceEl5.setAttribute('seed', '0');
                            feTurbulenceEl5.setAttribute('result', 'turbulence');
                            filterEl5.appendChild(feTurbulenceEl5);

                            // Add feDisplacementMap
                            const feDisplacementMapEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feDisplacementMap');
                            feDisplacementMapEl5.setAttribute('in', 'SourceGraphic');
                            feDisplacementMapEl5.setAttribute('in2', 'turbulence');
                            feDisplacementMapEl5.setAttribute('scale', '100');
                            feDisplacementMapEl5.setAttribute('xChannelSelector', 'R');
                            feDisplacementMapEl5.setAttribute('yChannelSelector', 'G');
                            feDisplacementMapEl5.setAttribute('result', 'displacement');
                            filterEl5.appendChild(feDisplacementMapEl5);

                            // Add feGaussianBlur
                            const feGaussianBlurEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl5.setAttribute('in', 'displacement');
                            feGaussianBlurEl5.setAttribute('stdDeviation', '5');
                            feGaussianBlurEl5.setAttribute('result', 'blur');
                            filterEl5.appendChild(feGaussianBlurEl5);

                            // Add feColorMatrix
                            const feColorMatrixEl5 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl5.setAttribute('in', 'blur');
                            feColorMatrixEl5.setAttribute('mode', 'matrix');
                            feColorMatrixEl5.setAttribute(
                                'values',
                                `
                                1 0 0 0 0
                                0 1 0 0 0
                                0 0 1 0 0
                                0 0 0 15 -7`
                            );
                            feColorMatrixEl5.setAttribute('result', 'gooey');
                            filterEl5.appendChild(feColorMatrixEl5);

                            // Append the filter to <defs>
                            defsEl5.appendChild(filterEl5);

                            // Append <defs> to the SVG
                            svgEl5.appendChild(defsEl5);

                            // Append the SVG to the cursor element
                            cursorEl.appendChild(svgEl5);
                        break;
                        case 6:
                            // Create the inner container
                            const innerEl6 = document.createElement('div');
                            innerEl6.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl6);

                            // Create the SVG element
                            const svgEl6 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl6.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl6.setAttribute('version', '1.1');

                            // Create the <defs> element
                            const defsEl6 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                            // Create the filter element
                            const filterEl6 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl6.setAttribute('id', 'gooey');

                            // Add the first feGaussianBlur
                            const feGaussianBlurEl61 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl61.setAttribute('in', 'SourceGraphic');
                            feGaussianBlurEl61.setAttribute('result', 'blur');
                            feGaussianBlurEl61.setAttribute('stdDeviation', '12');
                            filterEl6.appendChild(feGaussianBlurEl61);

                            // Add the first feColorMatrix
                            const feColorMatrixEl61 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl61.setAttribute('in', 'blur');
                            feColorMatrixEl61.setAttribute('mode', 'matrix');
                            feColorMatrixEl61.setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 18 -7');
                            feColorMatrixEl61.setAttribute('result', 'goo');
                            filterEl6.appendChild(feColorMatrixEl61);

                            // Add the second feGaussianBlur
                            const feGaussianBlurEl62 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl62.setAttribute('in', 'goo');
                            feGaussianBlurEl62.setAttribute('stdDeviation', '9');
                            feGaussianBlurEl62.setAttribute('result', 'shadow');
                            filterEl6.appendChild(feGaussianBlurEl62);

                            // Add the second feColorMatrix
                            const feColorMatrixEl62 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl62.setAttribute('in', 'shadow');
                            feColorMatrixEl62.setAttribute('mode', 'matrix');
                            feColorMatrixEl62.setAttribute('values', '0 0 0 0 0  0 0 0 0 0  0 0 0 0 0  0 0 0 8 -.2');
                            feColorMatrixEl62.setAttribute('result', 'shadow');
                            filterEl6.appendChild(feColorMatrixEl62);

                            // Add the first feComposite
                            const feCompositeEl61 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                            feCompositeEl61.setAttribute('in2', 'shadow');
                            feCompositeEl61.setAttribute('in', 'goo');
                            feCompositeEl61.setAttribute('result', 'goo');
                            filterEl6.appendChild(feCompositeEl61);

                            // Add the second feComposite
                            const feCompositeEl62 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                            feCompositeEl62.setAttribute('in2', 'goo');
                            feCompositeEl62.setAttribute('in', 'SourceGraphic');
                            feCompositeEl62.setAttribute('result', 'mix');
                            filterEl6.appendChild(feCompositeEl62);

                            // Append the filter to <defs>
                            defsEl6.appendChild(filterEl6);

                            // Append <defs> to the SVG
                            svgEl6.appendChild(defsEl6);

                            // Append the SVG to the cursor element
                            cursorEl.appendChild(svgEl6);
                        break;
                        case 7:
                            // Create the inner container
                            const innerEl7 = document.createElement('div');
                            innerEl7.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl7);
                        break;
                        case 8:
                            // Create the inner container
                            const innerEl8 = document.createElement('div');
                            innerEl8.classList.add('cursor__inner');
                            cursorEl.appendChild(innerEl8);

                            // Create the SVG element
                            const svgEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                            svgEl8.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
                            svgEl8.setAttribute('version', '1.1');

                            // Create the <defs> element
                            const defsEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'defs');

                            // Create the filter element
                            const filterEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'filter');
                            filterEl8.setAttribute('id', 'gooey');

                            // Add the feGaussianBlur
                            const feGaussianBlurEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'feGaussianBlur');
                            feGaussianBlurEl8.setAttribute('in', 'SourceGraphic');
                            feGaussianBlurEl8.setAttribute('result', 'blur');
                            feGaussianBlurEl8.setAttribute('stdDeviation', '3.2');
                            filterEl8.appendChild(feGaussianBlurEl8);

                            // Add the feColorMatrix
                            const feColorMatrixEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'feColorMatrix');
                            feColorMatrixEl8.setAttribute('in', 'blur');
                            feColorMatrixEl8.setAttribute('mode', 'matrix');
                            feColorMatrixEl8.setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 20 -7');
                            feColorMatrixEl8.setAttribute('result', 'goo');
                            filterEl8.appendChild(feColorMatrixEl8);

                            // Add the feComposite
                            const feCompositeEl8 = document.createElementNS('http://www.w3.org/2000/svg', 'feComposite');
                            feCompositeEl8.setAttribute('in', 'SourceGraphic');
                            feCompositeEl8.setAttribute('in2', 'goo');
                            feCompositeEl8.setAttribute('operator', 'atop');
                            filterEl8.appendChild(feCompositeEl8);

                            // Append the filter to <defs>
                            defsEl8.appendChild(filterEl8);

                            // Append <defs> to the SVG
                            svgEl8.appendChild(defsEl8);

                            // Append the SVG to the cursor element
                            cursorEl.appendChild(svgEl8);
                        break;
                        default:
                            console.error(`Unknown cursor style: ${style}`);
                        break;
                    }

                    targetDiv.appendChild(cursorEl);
                    return cursorEl;
                };

                const applyCSSVariables = (el, cssConfig) => {
                    if (!el || !cssConfig) return; // Add null check

                    Object.entries(cssConfig).forEach(([key, value]) => {
                        // Convert camelCase to kebab-case for CSS variables
                        const cssVar = `--${key.replace(/([A-Z])/g, '-$1').toLowerCase()}`;
                        el.style.setProperty(cssVar, value);
                    });
                };

                
                // Create cursor DOM
                let cursorEl;
                let cursorConfig = {
                    config: {
                        columns: cursor.cursor.config.columns ?? 30,
                        borderRadius: cursor.cursor.config.borderRadius ?? '0px',
                        backgroundColor: cursor.cursor.config.backgroundColor ?? 'blue',
                        blendingMode: cursor.cursor.config.blendingMode ?? 'normal',
                        style: Number(cursor.cursor.config.style) ?? 1
                    }
                }
                if (cursorConfig) {
                    const { config: { style = 1, ...config} } = cursorConfig;
                    cursorEl = createCursorDOM(style);
                    if (config) {
                        applyCSSVariables(cursorEl, config);
                    }
                } else {
                    cursorEl = createCursorDOM(1); // Default to style 1 if no config
                }

                // Create and initialize the Goo cursor
                class GooCursor {
                    DOM = {
                        el: null,
                        inner: null,
                        cells: null,
                    };
                    cellSize;
                    rows;
                    columns;
                    settings = {
                        ttl: 0.2
                    };

                    constructor(DOM_el) {
                        this.DOM.el = DOM_el;
    
                        // Safely get .cursor__inner
                        this.DOM.inner = this.DOM.el?.querySelector('.cursor__inner');
                        if (!this.DOM.inner) {
                            console.error('.cursor__inner element missing!');
                            return;
                        }

                        // Only apply filter if browser isn't Firefox AND SVG exists
                        const gooeyFilter = document.getElementById('gooey');
                        if (!isFirefox() && gooeyFilter) {
                            this.DOM.inner.style.filter = 'url(#gooey)';
                        }

                        this.settings.ttl = this.DOM.el.getAttribute('data-ttl') || this.settings.ttl;

                        this.layout();
                        this.initEvents();
                    }

                    initEvents() {
                        window.addEventListener('resize', () => this.layout());

                        const handleMove = () => {
                            const cell = this.getCellAtCursor();

                            if (cell === null || this.cachedCell === cell) return;
                            this.cachedCell = cell;
                            if(cell){
                                cell.style.opacity = '1';
                                setTimeout(() => {
                                    if(cell){
                                      cell.style.opacity = '0';
                                    }

                                }, this.settings.ttl * 1000);
                            }

                        };

                        window.addEventListener('mousemove', handleMove);
                        window.addEventListener('pointermove', handleMove, { passive: true });
                    }

                    layout() {
                        this.columns = parseInt(getComputedStyle(this.DOM.el).getPropertyValue('--columns'));
                        this.cellSize = winsize.width / this.columns;
                        this.rows = Math.ceil(winsize.height / this.cellSize);
                        this.cellsTotal = this.rows * this.columns;

                        let innerStr = '';
                        this.DOM.inner.innerHTML = '';

                        const customColorsAttr = this.DOM.el.getAttribute('data-custom-colors');
                        let customColorsArr;
                        let customColorsTotal = 0;

                        if (customColorsAttr) {
                            customColorsArr = this.DOM.el.getAttribute('data-custom-colors').split(',');
                            customColorsTotal = customColorsArr ? customColorsArr.length : 0;
                        }

                        for (let i = 0; i < this.cellsTotal; ++i) {
                            innerStr += customColorsTotal === 0
                                ? '<div class="cursor__inner-box"></div>'
                                : `<div style="transform: scale(${Math.random() * (2 - 0.5) + 0.5}); 
                                    background:${customColorsArr[Math.round(Math.random() * (customColorsTotal - 1))]}" 
                                class="cursor__inner-box"></div>`;
                        }
                        this.DOM.inner.innerHTML = innerStr;
                        this.DOM.cells = this.DOM.inner.children;
                    }

                    getCellAtCursor() {
                        const columnIndex = Math.min(Math.max(Math.floor(mousepos.x / this.cellSize), 0), this.columns - 1);
                        const rowIndex = Math.min(Math.max(Math.floor(mousepos.y / this.cellSize), 0), this.rows - 1);
                        const cellIndex = rowIndex * this.columns + columnIndex;
                    
                        if (cellIndex >= this.cellsTotal || cellIndex < 0) {
                            return null;
                        }
                    
                        return this.DOM.cells[cellIndex]; 
                    }
                }

                // const cursorEl = createCursorDOM();

                // Initialize cursor
                const goo = new GooCursor(cursorEl);
            }
        }
        //Hestellipse
        if (cursor.id === 12) {
            if (hestellipseeRef.current) {
                // Define elements object in wider scope
                const elements = {};
                const points = [];
                const segments = 100;
                const mouse = { x: 0, y: 0 };

                // Get container dimensions
                const container = hestellipseeRef.current;
                const containerRect = container.getBoundingClientRect();

                function createSVGElement(svgStyles, pathStyles) {
                    const svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                    
                    // Apply SVG attributes
                    Object.entries(svgStyles).forEach(([key, value]) => svg.setAttribute(key, value));
                    
                    svg.style.position = 'absolute';
                    svg.style.pointerEvents = 'none';
                    svg.style.top = '0';
                    svg.style.left = '0';
                    svg.style.width = '100%';
                    svg.style.height = '100%';
                    
                    const path = document.createElementNS('http://www.w3.org/2000/svg', 'path');
                    
                    function camelToKebab(key) {
                        return key.replace(/[A-Z]/g, (match) => `-${match.toLowerCase()}`);
                    }
                    
                    Object.entries(pathStyles).forEach(([key, value]) => {
                        const kebabKey = camelToKebab(key);
                        path.setAttribute(kebabKey, value);
                    });

                    svg.appendChild(path);
                    
                    container.style.position = 'relative';
                    container.appendChild(svg);

                    // Store elements before returning
                    elements.svg = svg;
                    elements.path = path;

                    return svg;
                }

                function initSVGAnimation() {
                    // Initialize points if empty
                    if (points.length === 0) {
                        for (let i = 0; i < segments; i++) {
                            points.push({ x: mouse.x, y: mouse.y });
                        }
                    }

                    // Define animate function
                    function animate() {
                        // Ensure elements.path exists before using it
                        if (!elements.path) return;

                        let px = mouse.x;
                        let py = mouse.y;

                        points.forEach((point, index) => {
                            point.x = px;
                            point.y = py;

                            const next = points[index + 1];
                            if (next) {
                                px = px - (point.x - next.x) * 0.6;
                                py = py - (point.y - next.y) * 0.6;
                            }
                        });

                        elements.path.setAttribute('d', `M ${points.map((p) => `${p.x} ${p.y}`).join(' L ')}`);
                        requestAnimationFrame(animate);
                    }

                    // Start animation
                    animate();
                }

                function resizeSVG() {
                    if (elements.svg) {
                        const rect = container.getBoundingClientRect();
                        elements.svg.setAttribute('viewBox', `0 0 ${rect.width} ${rect.height}`);
                    }
                }

                function handleMouseMove(event) {
                    const rect = container.getBoundingClientRect();
                    mouse.x = event.clientX - rect.left;
                    mouse.y = event.clientY - rect.top;
                }

                // Create SVG and initialize animation in sequence
                createSVGElement(cursor.cursor.svg, cursor.cursor.path);
                resizeSVG();
                initSVGAnimation();

                // Add event listeners
                window.addEventListener('resize', resizeSVG);
                container.addEventListener('pointermove', handleMouseMove);

                // Cleanup function
                return () => {
                    window.removeEventListener('resize', resizeSVG);
                    container.removeChild(elements.svg);
                    elements.svg = null;
                    elements.path = null;
                    points.length = 0;
                };
            }
        }
        //Hephamage
        if (cursor.id === 10) {
            if (hephamageRef.current) {
                const targetDiv = hephamageRef.current;
        
                // Store references to the event listeners and the content div
                let mouseMoveListener;
                let resizeListener;
                let contentDiv;
                let animationFrameId;
        
        
                const createContent = (cursor) => {
                    // Create content container
                    contentDiv = document.createElement('div');
                    contentDiv.className = 'content';
                    targetDiv.appendChild(contentDiv);
        
                    // Dynamically construct image URLs using the plugin URL
                    const defaultImageSources = [
                        `${pluginUrl}/img/1.jpg`,
                        `${pluginUrl}/img/2.jpg`,
                        `${pluginUrl}/img/3.jpg`,
                        `${pluginUrl}/img/4.jpg`,
                        `${pluginUrl}/img/5.jpg`,
                        `${pluginUrl}/img/6.jpg`,
                        `${pluginUrl}/img/7.jpg`,
                        `${pluginUrl}/img/8.jpg`,
                        `${pluginUrl}/img/9.jpg`,
                        `${pluginUrl}/img/10.jpg`,
                        `${pluginUrl}/img/11.jpg`,
                        `${pluginUrl}/img/12.jpg`,
                        `${pluginUrl}/img/13.jpg`,
                        `${pluginUrl}/img/14.jpg`,
                        `${pluginUrl}/img/15.jpg`
                    ];
        
                    // Set CSS variables for content wrapper
                    const { width, filter, blendingMode } = cursor?.cursor?.config || {};
                    if (width) contentDiv.style.setProperty('--width', width);
                    if (filter) contentDiv.style.setProperty('--filter', filter);
                    if (blendingMode) contentDiv.style.setProperty('--blending-mode', blendingMode);
        
                    const imageUrls = cursor?.cursor?.config?.imageUrls || defaultImageSources.join(',');
        
                    const imageSources = imageUrls.split(',').map(url => url.trim()); // Split into array and trim
        
                    imageSources.forEach(src => {
                        const img = document.createElement('img');
                        img.className = 'content__img';
                        img.src = src; // Use the source from the array
                        img.alt = 'Some image';
                        contentDiv.appendChild(img); // Append the image to the container
                    });
                    return contentDiv;
                };
        
                // Helper functions
                const MathUtils = {
                    lerp: (a, b, n) => (1 - n) * a + n * b,
                    distance: (x1, y1, x2, y2) => Math.hypot(x2 - x1, y2 - y1),
                    getRandomFloat: (min, max) => Math.random() * (max - min) + min,
                }
        
                let winsize;
                const calcWinsize = () => {
                    winsize = { width: window.innerWidth, height: window.innerHeight };
                };
                calcWinsize();
                resizeListener = () => calcWinsize(); // Store resizeListener for cleanup
                window.addEventListener('resize', resizeListener);
        
                const getMousePos = (ev) => {
                    let posx = 0;
                    let posy = 0;
                    if (!ev) ev = window.event;
                    if (ev.clientX || ev.clientY) {
                        posx = ev.clientX;
                        posy = ev.clientY;
                    }
                    return { x: posx, y: posy };
                }
        
        
                let mousePos, lastMousePos, cacheMousePos;
                mousePos = lastMousePos = cacheMousePos = { x: 0, y: 0 };
        
                mouseMoveListener = (ev) => mousePos = getMousePos(ev); // Store mouseMoveListener for cleanup
                targetDiv.addEventListener('mousemove', mouseMoveListener);
        
                const getMouseDistance = () => MathUtils.distance(mousePos.x, mousePos.y, lastMousePos.x, lastMousePos.y);
        
                class Image {
                    constructor(el) {
                        this.DOM = { el: el };
                        this.defaultStyle = {
                            scale: 1,
                            x: 0,
                            y: 0,
                            opacity: 0
                        };
                        this.initEvents();
        
                        // Ensure dimensions are ready
                        if (this.DOM.el.complete) {
                            // If the image is already loaded
                            this.getRect();
                        } else {
                            // Wait for the image to load
                            this.DOM.el.addEventListener('load', () => this.getRect());
                        }
                    }
        
                    initEvents() {
                        window.addEventListener('resize', () => this.resize());
                    }
        
                    resize() {
                        Object.assign(this.DOM.el.style, {
                            transform: `translate(0,0) scale(1)`,
                            opacity: 0
                        });
                        this.getRect();
                    }
        
                    getRect() {
                        this.rect = this.DOM.el.getBoundingClientRect();
                    }
        
                    isActive() {
                        return this.DOM.el.style.opacity !== '0';
                    }
                }
        
                class ImageTrail {
                    constructor() {
                        this.DOM = { content: createContent(cursor) };
                        this.images = [];
                        [...this.DOM.content.querySelectorAll('img')].forEach(img => this.images.push(new Image(img)));
                        this.imagesTotal = this.images.length;
                        this.imgPosition = 0;
                        this.zIndexVal = 1;
                        this.threshold = cursor?.cursor?.config?.threshold ?? 100;
                        this.render();
                    }
        
                    render() {
                        let distance = getMouseDistance();
        
                        cacheMousePos.x = MathUtils.lerp(cacheMousePos.x || mousePos.x, mousePos.x, 0.1);
                        cacheMousePos.y = MathUtils.lerp(cacheMousePos.y || mousePos.y, mousePos.y, 0.1);
        
                        if (distance > this.threshold) {
                            this.showNextImage();
                            ++this.zIndexVal;
                            this.imgPosition = this.imgPosition < this.imagesTotal - 1 ? this.imgPosition + 1 : 0;
                            lastMousePos = mousePos;
                        }
        
                        let isIdle = true;
                        for (let img of this.images) {
                            if (img.isActive()) {
                                isIdle = false;
                                break;
                            }
                        }
        
                        if (isIdle && this.zIndexVal !== 1) {
                            this.zIndexVal = 1;
                        }
        
                        animationFrameId = requestAnimationFrame(() => this.render());
                    }
        
                    showNextImage() {
                        const img = this.images[this.imgPosition];
                        const el = img.DOM.el;
        
                        // Calculate the initial position
                        const initialX = mousePos.x - img.rect.width / 2;
                        const initialY = mousePos.y - img.rect.height / 2;
        
                        // Ensure the image starts fresh
                        el.style.transition = 'none'; // Disable transition temporarily
                        el.style.opacity = 0; // Reset opacity
                        el.style.transform = `translate(${initialX}px, ${initialY}px)`; // Start scaled down at cursor
        
                        // Force reflow to apply the initial state
                        el.offsetHeight;
        
                        // Fade in and grow smoothly to scale(1)
                        el.style.transition = 'transform 0.9s cubic-bezier(0.19, 1, 0.22, 1)';
                        el.style.opacity = 1;
                        el.style.transform = `translate(${initialX}px, ${initialY}px) scale(1)`;
                        el.style.zIndex = this.zIndexVal;
        
                        // Update the position smoothly as the cursor moves
                        requestAnimationFrame(() => {
                            el.style.transform = `translate(${mousePos.x - img.rect.width / 2}px, ${mousePos.y - img.rect.height / 2}px) scale(1)`;
                        });
        
                        // Define timeout behaviors based on cursor style
                        const timeoutBehaviors = {
                            '1': () => {
                                setTimeout(() => {
                                    el.style.transition = 'opacity 1s ease-out, transform 1s cubic-bezier(0.23, 1, 0.32, 1)';
                                    el.style.opacity = 0;
                                    el.style.transform = `translate(${initialX}px, ${initialY}px) scale(0.2)`;
                                }, 400);
                            },
                            '2': () => {
                                setTimeout(() => {
                                    el.style.transition = 'opacity 0.8s ease-out, transform 0.8s cubic-bezier(0.86, 0, 0.07, 1)';
                                    el.style.opacity = 0;
                                    el.style.transformOrigin = 'center';
                                    el.style.transform = `translate(${initialX}px, ${initialY}px) scale(2)`;
                                }, 800);
                            },
                            '3': () => {
                                // Translate the image down off-screen after 0.4s
                                setTimeout(() => {
                                    el.style.transition = 'transform 800ms ease-in-out';
                                    el.style.transform = `translate(${initialX}px, ${
                                        mousePos.y - img.rect.height / 2 + window.innerHeight + img.rect.height / 2
                                    }px)`;
                                }, 400);
                            },
                            '4': () => {
                                // Translate the image off-screen with random rotation after 0.6s
                                setTimeout(() => {
                                    const randomX = MathUtils.getRandomFloat(
                                        -1 * (winsize.width + img.rect.width / 2),
                                        winsize.width + img.rect.width / 2
                                    );
                                    const randomY = MathUtils.getRandomFloat(
                                        -1 * (winsize.height + img.rect.height / 2),
                                        winsize.height + img.rect.height / 2
                                    );
                                    const randomRotation = MathUtils.getRandomFloat(-40, 40);
        
                                    el.style.transition = 'opacity 1s ease-out, transform 1s ease-out';
                                    el.style.opacity = 0;
                                    el.style.transform = `
                                        translate(${mousePos.x + randomX}px, ${mousePos.y + randomY}px) 
                                        rotate(${randomRotation}deg)
                                    `;
                                }, 400);
                            },
                            '5': () => {
                                // Scale to (0.7, 1.7) and translate down after 0.6s
                                setTimeout(() => {
                                    const translateYDistance = 150; // Adjust this value as needed
                                    el.style.transition = 'opacity 0.3s ease-out, transform 0.5s ease-out';
                                    el.style.opacity = 0;
                                    el.style.transform = `translate(${initialX}px, ${initialY + translateYDistance}px) scale(0.7, 1.7)`;
                                }, 600);
                            },
                            '6': () => {
                                // Animate out after delay
                                setTimeout(() => {
                                    el.style.transition = 'opacity 0.4s ease-out';
                                    el.style.opacity = '0';
                                }, 450);
                            }
                        };
        
                        // Get the cursor style from parsedConfig
                        const cursorStyle = cursor?.cursor.config.style || '1';
        
                        // Apply the appropriate behavior or a default behavior
                        (timeoutBehaviors[cursorStyle] || timeoutBehaviors['1'])();
                    }
                }
        
                // Initialize
                const imageTrail = new ImageTrail();
        
                // Cleanup function to remove event listeners and content
                return () => {
                    //Remove event listeners
                    targetDiv.removeEventListener('mousemove', mouseMoveListener);
                    window.removeEventListener('resize', resizeListener);
        
                    // Cancel animation frame
                    cancelAnimationFrame(animationFrameId);
        
        
                    // Remove the content div
                    if (contentDiv) {
                        contentDiv.remove();
                    }
                };
            }
        }
        //Nyxamoke
        if (cursor.id === 14) {
            if (nyxamokeRef.current) {
                // Create a new <canvas> element
                const canvas = document.createElement('canvas');
                canvas.width = 800;
                canvas.height = 600;
                nyxamokeRef.current.appendChild(canvas);

                // const canvas = document.getElementsByTagName('canvas')[0];

                resizeCanvas();
                // Default config values
                let defaultConfig = {
                    SIM_RESOLUTION: 128,
                    DYE_RESOLUTION: 1024,
                    CAPTURE_RESOLUTION: 512,
                    DENSITY_DISSIPATION: 3.5,
                    VELOCITY_DISSIPATION: 1.5,
                    PRESSURE: 0.4,
                    PRESSURE_ITERATIONS: 20,
                    CURL: 15,
                    SPLAT_RADIUS: 0.25,
                    SPLAT_FORCE: 6000,
                    SHADING: true,
                    COLORFUL: true,
                    COLOR_UPDATE_SPEED: 10,
                    PAUSED: false,
                    BACK_COLOR: { r: 0, g: 0, b: 0 },
                    TRANSPARENT: false,
                    BLOOM: false,
                    BLOOM_ITERATIONS: 8,
                    BLOOM_RESOLUTION: 256,
                    BLOOM_INTENSITY: 0.8,
                    BLOOM_THRESHOLD: 0.6,
                    BLOOM_SOFT_KNEE: 0.7,
                    SUNRAYS: false,
                    SUNRAYS_RESOLUTION: 196,
                    SUNRAYS_WEIGHT: 1.0,
                };

                // Initialize config by merging cursor values
                const simRes = Number(cursor?.cursor?.config?.simResolution);
                const dyeRes = Number(cursor?.cursor?.config?.dyeResolution);
                const captureRes = Number(cursor?.cursor?.config?.captureResolution);
                const densityDis = Number(cursor?.cursor?.config?.densityDissipation);
                const velocityDis = Number(cursor?.cursor?.config?.velocityDissipation);
                const pressureAmo = Number(cursor?.cursor?.config?.pressure);
                const pressureIte = Number(cursor?.cursor?.config?.pressureIterations);
                const curlAmo = Number(cursor?.cursor?.config?.curl);
                const splatRad = Number(cursor?.cursor?.config?.splatRadius);
                const splatFor = Number(cursor?.cursor?.config?.splatForce);
                const colorUpd = Number(cursor?.cursor?.config?.colorUpdateSpeed);
                const bloomIte = Number(cursor?.cursor?.config?.bloomIterations);
                const bloomRes = Number(cursor?.cursor?.config?.bloomResolution);
                const bloomInt = Number(cursor?.cursor?.config?.bloomIntensity);
                const bloomThr = Number(cursor?.cursor?.config?.bloomThreshold);
                const bloomSof = Number(cursor?.cursor?.config?.bloomSoftKnee);
                const sunraysRes = Number(cursor?.cursor?.config?.sunraysResolution);
                const sunraysWei = Number(cursor?.cursor?.config?.sunraysWeight);
                
                let config = {
                    SIM_RESOLUTION: isNaN(simRes) ? defaultConfig.SIM_RESOLUTION : simRes,
                    DYE_RESOLUTION: isNaN(dyeRes) ? defaultConfig.DYE_RESOLUTION : dyeRes,
                    CAPTURE_RESOLUTION: isNaN(captureRes) ? defaultConfig.CAPTURE_RESOLUTION : captureRes,
                    DENSITY_DISSIPATION: isNaN(densityDis) ? defaultConfig.DENSITY_DISSIPATION : densityDis,
                    VELOCITY_DISSIPATION: isNaN(velocityDis) ? defaultConfig.VELOCITY_DISSIPATION : velocityDis,
                    PRESSURE: isNaN(pressureAmo) ? defaultConfig.PRESSURE : pressureAmo,
                    PRESSURE_ITERATIONS: isNaN(pressureIte) ? defaultConfig.PRESSURE_ITERATIONS : pressureIte,
                    CURL: isNaN(curlAmo) ? defaultConfig.CURL : curlAmo,
                    SPLAT_RADIUS: isNaN(splatRad) ? defaultConfig.SPLAT_RADIUS : splatRad,
                    SPLAT_FORCE: isNaN(splatFor) ? defaultConfig.SPLAT_FORCE : splatFor,
                    SHADING: String(cursor?.cursor?.config?.shading).toLowerCase() === 'true' ?? defaultConfig.SHADING,
                    COLORFUL: String(cursor?.cursor?.config?.colorful).toLowerCase() === 'true' ?? defaultConfig.COLORFUL,
                    COLOR_UPDATE_SPEED: isNaN(colorUpd) ? defaultConfig.COLOR_UPDATE_SPEED : colorUpd,
                    PAUSED: String(cursor?.cursor?.config?.paused).toLowerCase() === 'true' ?? defaultConfig.PAUSED,
                    BACK_COLOR: cursor?.cursor?.config?.backColor ?? defaultConfig.BACK_COLOR,
                    TRANSPARENT: String(cursor?.cursor?.config?.transparent).toLowerCase() === 'true' ?? defaultConfig.TRANSPARENT,
                    BLOOM: String(cursor?.cursor?.config?.bloom).toLowerCase() === 'true' ?? defaultConfig.BLOOM,
                    BLOOM_ITERATIONS: isNaN(bloomIte) ? defaultConfig.BLOOM_ITERATIONS : bloomIte,
                    BLOOM_RESOLUTION: isNaN(bloomRes) ? defaultConfig.BLOOM_RESOLUTION : bloomRes,
                    BLOOM_INTENSITY: isNaN(bloomInt) ? defaultConfig.BLOOM_INTENSITY : bloomInt,
                    BLOOM_THRESHOLD: isNaN(bloomThr) ? defaultConfig.BLOOM_THRESHOLD : bloomThr,
                    BLOOM_SOFT_KNEE: isNaN(bloomSof) ? defaultConfig.BLOOM_SOFT_KNEE : bloomSof,
                    SUNRAYS: String(cursor?.cursor?.config?.sunrays).toLowerCase() === 'true' ?? defaultConfig.SUNRAYS,
                    SUNRAYS_RESOLUTION: isNaN(sunraysRes) ? defaultConfig.SUNRAYS_RESOLUTION : sunraysRes,
                    SUNRAYS_WEIGHT: isNaN(sunraysWei) ? defaultConfig.SUNRAYS_WEIGHT: sunraysWei,
                };

                function pointerPrototype () {
                    this.id = -1;
                    this.texcoordX = 0;
                    this.texcoordY = 0;
                    this.prevTexcoordX = 0;
                    this.prevTexcoordY = 0;
                    this.deltaX = 0;
                    this.deltaY = 0;
                    this.down = false;
                    this.moved = false;
                    this.color = [30, 0, 300];
                }

                let pointers = [];
                let splatStack = [];
                pointers.push(new pointerPrototype());

                const { gl, ext } = getWebGLContext(canvas);

                if (isMobile()) {
                    config.DYE_RESOLUTION = 512;
                }
                if (!ext.supportLinearFiltering) {
                    config.DYE_RESOLUTION = 512;
                    config.SHADING = false;
                    config.BLOOM = false;
                    config.SUNRAYS = false;
                }

                function getWebGLContext (canvas) {
                    const params = { alpha: true, depth: false, stencil: false, antialias: false, preserveDrawingBuffer: false };

                    let gl = canvas.getContext('webgl2', params);
                    const isWebGL2 = !!gl;
                    if (!isWebGL2)
                        gl = canvas.getContext('webgl', params) || canvas.getContext('experimental-webgl', params);

                    let halfFloat;
                    let supportLinearFiltering;
                    if (isWebGL2) {
                        gl.getExtension('EXT_color_buffer_float');
                        supportLinearFiltering = gl.getExtension('OES_texture_float_linear');
                    } else {
                        halfFloat = gl.getExtension('OES_texture_half_float');
                        supportLinearFiltering = gl.getExtension('OES_texture_half_float_linear');
                    }

                    gl.clearColor(0.0, 0.0, 0.0, 1.0);

                    const halfFloatTexType = isWebGL2 ? gl.HALF_FLOAT : halfFloat.HALF_FLOAT_OES;
                    let formatRGBA;
                    let formatRG;
                    let formatR;

                    if (isWebGL2)
                    {
                        formatRGBA = getSupportedFormat(gl, gl.RGBA16F, gl.RGBA, halfFloatTexType);
                        formatRG = getSupportedFormat(gl, gl.RG16F, gl.RG, halfFloatTexType);
                        formatR = getSupportedFormat(gl, gl.R16F, gl.RED, halfFloatTexType);
                    }
                    else
                    {
                        formatRGBA = getSupportedFormat(gl, gl.RGBA, gl.RGBA, halfFloatTexType);
                        formatRG = getSupportedFormat(gl, gl.RGBA, gl.RGBA, halfFloatTexType);
                        formatR = getSupportedFormat(gl, gl.RGBA, gl.RGBA, halfFloatTexType);
                    }

                    return {
                        gl,
                        ext: {
                            formatRGBA,
                            formatRG,
                            formatR,
                            halfFloatTexType,
                            supportLinearFiltering
                        }
                    };
                }

                function getSupportedFormat (gl, internalFormat, format, type)
                {
                    if (!supportRenderTextureFormat(gl, internalFormat, format, type))
                    {
                        switch (internalFormat)
                        {
                            case gl.R16F:
                                return getSupportedFormat(gl, gl.RG16F, gl.RG, type);
                            case gl.RG16F:
                                return getSupportedFormat(gl, gl.RGBA16F, gl.RGBA, type);
                            default:
                                return null;
                        }
                    }

                    return {
                        internalFormat,
                        format
                    }
                }

                function supportRenderTextureFormat (gl, internalFormat, format, type) {
                    let texture = gl.createTexture();
                    gl.bindTexture(gl.TEXTURE_2D, texture);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
                    gl.texImage2D(gl.TEXTURE_2D, 0, internalFormat, 4, 4, 0, format, type, null);

                    let fbo = gl.createFramebuffer();
                    gl.bindFramebuffer(gl.FRAMEBUFFER, fbo);
                    gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);

                    let status = gl.checkFramebufferStatus(gl.FRAMEBUFFER);
                    return status == gl.FRAMEBUFFER_COMPLETE;
                }

                function isMobile () {
                    return /Mobi|Android/i.test(navigator.userAgent);
                }

                class Material {
                    constructor (vertexShader, fragmentShaderSource) {
                        this.vertexShader = vertexShader;
                        this.fragmentShaderSource = fragmentShaderSource;
                        this.programs = [];
                        this.activeProgram = null;
                        this.uniforms = [];
                    }

                    setKeywords (keywords) {
                        let hash = 0;
                        for (let i = 0; i < keywords.length; i++)
                            hash += hashCode(keywords[i]);

                        let program = this.programs[hash];
                        if (program == null)
                        {
                            let fragmentShader = compileShader(gl.FRAGMENT_SHADER, this.fragmentShaderSource, keywords);
                            program = createProgram(this.vertexShader, fragmentShader);
                            this.programs[hash] = program;
                        }

                        if (program == this.activeProgram) return;

                        this.uniforms = getUniforms(program);
                        this.activeProgram = program;
                    }

                    bind () {
                        gl.useProgram(this.activeProgram);
                    }
                }

                class Program {
                    constructor (vertexShader, fragmentShader) {
                        this.uniforms = {};
                        this.program = createProgram(vertexShader, fragmentShader);
                        this.uniforms = getUniforms(this.program);
                    }

                    bind () {
                        gl.useProgram(this.program);
                    }
                }

                function createProgram (vertexShader, fragmentShader) {
                    let program = gl.createProgram();
                    gl.attachShader(program, vertexShader);
                    gl.attachShader(program, fragmentShader);
                    gl.linkProgram(program);

                    if (!gl.getProgramParameter(program, gl.LINK_STATUS))
                        console.trace(gl.getProgramInfoLog(program));

                    return program;
                }

                function getUniforms (program) {
                    let uniforms = [];
                    let uniformCount = gl.getProgramParameter(program, gl.ACTIVE_UNIFORMS);
                    for (let i = 0; i < uniformCount; i++) {
                        let uniformName = gl.getActiveUniform(program, i).name;
                        uniforms[uniformName] = gl.getUniformLocation(program, uniformName);
                    }
                    return uniforms;
                }

                function compileShader (type, source, keywords) {
                    source = addKeywords(source, keywords);

                    const shader = gl.createShader(type);
                    gl.shaderSource(shader, source);
                    gl.compileShader(shader);

                    if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS))
                        console.trace(gl.getShaderInfoLog(shader));

                    return shader;
                };

                function addKeywords (source, keywords) {
                    if (keywords == null) return source;
                    let keywordsString = '';
                    keywords.forEach(keyword => {
                        keywordsString += '#define ' + keyword + '\n';
                    });
                    return keywordsString + source;
                }

                const baseVertexShader = compileShader(gl.VERTEX_SHADER, `
                    precision highp float;

                    attribute vec2 aPosition;
                    varying vec2 vUv;
                    varying vec2 vL;
                    varying vec2 vR;
                    varying vec2 vT;
                    varying vec2 vB;
                    uniform vec2 texelSize;

                    void main () {
                        vUv = aPosition * 0.5 + 0.5;
                        vL = vUv - vec2(texelSize.x, 0.0);
                        vR = vUv + vec2(texelSize.x, 0.0);
                        vT = vUv + vec2(0.0, texelSize.y);
                        vB = vUv - vec2(0.0, texelSize.y);
                        gl_Position = vec4(aPosition, 0.0, 1.0);
                    }
                `);

                const blurVertexShader = compileShader(gl.VERTEX_SHADER, `
                    precision highp float;

                    attribute vec2 aPosition;
                    varying vec2 vUv;
                    varying vec2 vL;
                    varying vec2 vR;
                    uniform vec2 texelSize;

                    void main () {
                        vUv = aPosition * 0.5 + 0.5;
                        float offset = 1.33333333;
                        vL = vUv - texelSize * offset;
                        vR = vUv + texelSize * offset;
                        gl_Position = vec4(aPosition, 0.0, 1.0);
                    }
                `);

                const blurShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying vec2 vUv;
                    varying vec2 vL;
                    varying vec2 vR;
                    uniform sampler2D uTexture;

                    void main () {
                        vec4 sum = texture2D(uTexture, vUv) * 0.29411764;
                        sum += texture2D(uTexture, vL) * 0.35294117;
                        sum += texture2D(uTexture, vR) * 0.35294117;
                        gl_FragColor = sum;
                    }
                `);

                const copyShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying highp vec2 vUv;
                    uniform sampler2D uTexture;

                    void main () {
                        gl_FragColor = texture2D(uTexture, vUv);
                    }
                `);

                const clearShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying highp vec2 vUv;
                    uniform sampler2D uTexture;
                    uniform float value;

                    void main () {
                        gl_FragColor = value * texture2D(uTexture, vUv);
                    }
                `);

                const colorShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;

                    uniform vec4 color;

                    void main () {
                        gl_FragColor = color;
                    }
                `);

                const checkerboardShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    uniform sampler2D uTexture;
                    uniform float aspectRatio;

                    #define SCALE 25.0

                    void main () {
                        vec2 uv = floor(vUv * SCALE * vec2(aspectRatio, 1.0));
                        float v = mod(uv.x + uv.y, 2.0);
                        v = v * 0.1 + 0.8;
                        gl_FragColor = vec4(vec3(v), 1.0);
                    }
                `);

                const displayShaderSource = `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    varying vec2 vL;
                    varying vec2 vR;
                    varying vec2 vT;
                    varying vec2 vB;
                    uniform sampler2D uTexture;
                    uniform sampler2D uBloom;
                    uniform sampler2D uSunrays;
                    uniform sampler2D uDithering;
                    uniform vec2 ditherScale;
                    uniform vec2 texelSize;

                    vec3 linearToGamma (vec3 color) {
                        color = max(color, vec3(0));
                        return max(1.055 * pow(color, vec3(0.416666667)) - 0.055, vec3(0));
                    }

                    void main () {
                        vec3 c = texture2D(uTexture, vUv).rgb;

                    #ifdef SHADING
                        vec3 lc = texture2D(uTexture, vL).rgb;
                        vec3 rc = texture2D(uTexture, vR).rgb;
                        vec3 tc = texture2D(uTexture, vT).rgb;
                        vec3 bc = texture2D(uTexture, vB).rgb;

                        float dx = length(rc) - length(lc);
                        float dy = length(tc) - length(bc);

                        vec3 n = normalize(vec3(dx, dy, length(texelSize)));
                        vec3 l = vec3(0.0, 0.0, 1.0);

                        float diffuse = clamp(dot(n, l) + 0.7, 0.7, 1.0);
                        c *= diffuse;
                    #endif

                    #ifdef BLOOM
                        vec3 bloom = texture2D(uBloom, vUv).rgb;
                    #endif

                    #ifdef SUNRAYS
                        float sunrays = texture2D(uSunrays, vUv).r;
                        c *= sunrays;
                    #ifdef BLOOM
                        bloom *= sunrays;
                    #endif
                    #endif

                    #ifdef BLOOM
                        float noise = texture2D(uDithering, vUv * ditherScale).r;
                        noise = noise * 2.0 - 1.0;
                        bloom += noise / 255.0;
                        bloom = linearToGamma(bloom);
                        c += bloom;
                    #endif

                        float a = max(c.r, max(c.g, c.b));
                        gl_FragColor = vec4(c, a);
                    }
                `;

                const bloomPrefilterShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying vec2 vUv;
                    uniform sampler2D uTexture;
                    uniform vec3 curve;
                    uniform float threshold;

                    void main () {
                        vec3 c = texture2D(uTexture, vUv).rgb;
                        float br = max(c.r, max(c.g, c.b));
                        float rq = clamp(br - curve.x, 0.0, curve.y);
                        rq = curve.z * rq * rq;
                        c *= max(rq, br - threshold) / max(br, 0.0001);
                        gl_FragColor = vec4(c, 0.0);
                    }
                `);

                const bloomBlurShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying vec2 vL;
                    varying vec2 vR;
                    varying vec2 vT;
                    varying vec2 vB;
                    uniform sampler2D uTexture;

                    void main () {
                        vec4 sum = vec4(0.0);
                        sum += texture2D(uTexture, vL);
                        sum += texture2D(uTexture, vR);
                        sum += texture2D(uTexture, vT);
                        sum += texture2D(uTexture, vB);
                        sum *= 0.25;
                        gl_FragColor = sum;
                    }
                `);

                const bloomFinalShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying vec2 vL;
                    varying vec2 vR;
                    varying vec2 vT;
                    varying vec2 vB;
                    uniform sampler2D uTexture;
                    uniform float intensity;

                    void main () {
                        vec4 sum = vec4(0.0);
                        sum += texture2D(uTexture, vL);
                        sum += texture2D(uTexture, vR);
                        sum += texture2D(uTexture, vT);
                        sum += texture2D(uTexture, vB);
                        sum *= 0.25;
                        gl_FragColor = sum * intensity;
                    }
                `);

                const sunraysMaskShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    uniform sampler2D uTexture;

                    void main () {
                        vec4 c = texture2D(uTexture, vUv);
                        float br = max(c.r, max(c.g, c.b));
                        c.a = 1.0 - min(max(br * 20.0, 0.0), 0.8);
                        gl_FragColor = c;
                    }
                `);

                const sunraysShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    uniform sampler2D uTexture;
                    uniform float weight;

                    #define ITERATIONS 16

                    void main () {
                        float Density = 0.3;
                        float Decay = 0.95;
                        float Exposure = 0.7;

                        vec2 coord = vUv;
                        vec2 dir = vUv - 0.5;

                        dir *= 1.0 / float(ITERATIONS) * Density;
                        float illuminationDecay = 1.0;

                        float color = texture2D(uTexture, vUv).a;

                        for (int i = 0; i < ITERATIONS; i++)
                        {
                            coord -= dir;
                            float col = texture2D(uTexture, coord).a;
                            color += col * illuminationDecay * weight;
                            illuminationDecay *= Decay;
                        }

                        gl_FragColor = vec4(color * Exposure, 0.0, 0.0, 1.0);
                    }
                `);

                const splatShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    uniform sampler2D uTarget;
                    uniform float aspectRatio;
                    uniform vec3 color;
                    uniform vec2 point;
                    uniform float radius;

                    void main () {
                        vec2 p = vUv - point.xy;
                        p.x *= aspectRatio;
                        vec3 splat = exp(-dot(p, p) / radius) * color;
                        vec3 base = texture2D(uTarget, vUv).xyz;
                        gl_FragColor = vec4(base + splat, 1.0);
                    }
                `);

                const advectionShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    uniform sampler2D uVelocity;
                    uniform sampler2D uSource;
                    uniform vec2 texelSize;
                    uniform vec2 dyeTexelSize;
                    uniform float dt;
                    uniform float dissipation;

                    vec4 bilerp (sampler2D sam, vec2 uv, vec2 tsize) {
                        vec2 st = uv / tsize - 0.5;

                        vec2 iuv = floor(st);
                        vec2 fuv = fract(st);

                        vec4 a = texture2D(sam, (iuv + vec2(0.5, 0.5)) * tsize);
                        vec4 b = texture2D(sam, (iuv + vec2(1.5, 0.5)) * tsize);
                        vec4 c = texture2D(sam, (iuv + vec2(0.5, 1.5)) * tsize);
                        vec4 d = texture2D(sam, (iuv + vec2(1.5, 1.5)) * tsize);

                        return mix(mix(a, b, fuv.x), mix(c, d, fuv.x), fuv.y);
                    }

                    void main () {
                    #ifdef MANUAL_FILTERING
                        vec2 coord = vUv - dt * bilerp(uVelocity, vUv, texelSize).xy * texelSize;
                        vec4 result = bilerp(uSource, coord, dyeTexelSize);
                    #else
                        vec2 coord = vUv - dt * texture2D(uVelocity, vUv).xy * texelSize;
                        vec4 result = texture2D(uSource, coord);
                    #endif
                        float decay = 1.0 + dissipation * dt;
                        gl_FragColor = result / decay;
                    }`,
                    ext.supportLinearFiltering ? null : ['MANUAL_FILTERING']
                );

                const divergenceShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying highp vec2 vUv;
                    varying highp vec2 vL;
                    varying highp vec2 vR;
                    varying highp vec2 vT;
                    varying highp vec2 vB;
                    uniform sampler2D uVelocity;

                    void main () {
                        float L = texture2D(uVelocity, vL).x;
                        float R = texture2D(uVelocity, vR).x;
                        float T = texture2D(uVelocity, vT).y;
                        float B = texture2D(uVelocity, vB).y;

                        vec2 C = texture2D(uVelocity, vUv).xy;
                        if (vL.x < 0.0) { L = -C.x; }
                        if (vR.x > 1.0) { R = -C.x; }
                        if (vT.y > 1.0) { T = -C.y; }
                        if (vB.y < 0.0) { B = -C.y; }

                        float div = 0.5 * (R - L + T - B);
                        gl_FragColor = vec4(div, 0.0, 0.0, 1.0);
                    }
                `);

                const curlShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying highp vec2 vUv;
                    varying highp vec2 vL;
                    varying highp vec2 vR;
                    varying highp vec2 vT;
                    varying highp vec2 vB;
                    uniform sampler2D uVelocity;

                    void main () {
                        float L = texture2D(uVelocity, vL).y;
                        float R = texture2D(uVelocity, vR).y;
                        float T = texture2D(uVelocity, vT).x;
                        float B = texture2D(uVelocity, vB).x;
                        float vorticity = R - L - T + B;
                        gl_FragColor = vec4(0.5 * vorticity, 0.0, 0.0, 1.0);
                    }
                `);

                const vorticityShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision highp float;
                    precision highp sampler2D;

                    varying vec2 vUv;
                    varying vec2 vL;
                    varying vec2 vR;
                    varying vec2 vT;
                    varying vec2 vB;
                    uniform sampler2D uVelocity;
                    uniform sampler2D uCurl;
                    uniform float curl;
                    uniform float dt;

                    void main () {
                        float L = texture2D(uCurl, vL).x;
                        float R = texture2D(uCurl, vR).x;
                        float T = texture2D(uCurl, vT).x;
                        float B = texture2D(uCurl, vB).x;
                        float C = texture2D(uCurl, vUv).x;

                        vec2 force = 0.5 * vec2(abs(T) - abs(B), abs(R) - abs(L));
                        force /= length(force) + 0.0001;
                        force *= curl * C;
                        force.y *= -1.0;

                        vec2 velocity = texture2D(uVelocity, vUv).xy;
                        velocity += force * dt;
                        velocity = min(max(velocity, -1000.0), 1000.0);
                        gl_FragColor = vec4(velocity, 0.0, 1.0);
                    }
                `);

                const pressureShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying highp vec2 vUv;
                    varying highp vec2 vL;
                    varying highp vec2 vR;
                    varying highp vec2 vT;
                    varying highp vec2 vB;
                    uniform sampler2D uPressure;
                    uniform sampler2D uDivergence;

                    void main () {
                        float L = texture2D(uPressure, vL).x;
                        float R = texture2D(uPressure, vR).x;
                        float T = texture2D(uPressure, vT).x;
                        float B = texture2D(uPressure, vB).x;
                        float C = texture2D(uPressure, vUv).x;
                        float divergence = texture2D(uDivergence, vUv).x;
                        float pressure = (L + R + B + T - divergence) * 0.25;
                        gl_FragColor = vec4(pressure, 0.0, 0.0, 1.0);
                    }
                `);

                const gradientSubtractShader = compileShader(gl.FRAGMENT_SHADER, `
                    precision mediump float;
                    precision mediump sampler2D;

                    varying highp vec2 vUv;
                    varying highp vec2 vL;
                    varying highp vec2 vR;
                    varying highp vec2 vT;
                    varying highp vec2 vB;
                    uniform sampler2D uPressure;
                    uniform sampler2D uVelocity;

                    void main () {
                        float L = texture2D(uPressure, vL).x;
                        float R = texture2D(uPressure, vR).x;
                        float T = texture2D(uPressure, vT).x;
                        float B = texture2D(uPressure, vB).x;
                        vec2 velocity = texture2D(uVelocity, vUv).xy;
                        velocity.xy -= vec2(R - L, T - B);
                        gl_FragColor = vec4(velocity, 0.0, 1.0);
                    }
                `);

                const blit = (() => {
                    gl.bindBuffer(gl.ARRAY_BUFFER, gl.createBuffer());
                    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1, -1, -1, 1, 1, 1, 1, -1]), gl.STATIC_DRAW);
                    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, gl.createBuffer());
                    gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array([0, 1, 2, 0, 2, 3]), gl.STATIC_DRAW);
                    gl.vertexAttribPointer(0, 2, gl.FLOAT, false, 0, 0);
                    gl.enableVertexAttribArray(0);

                    return (target, clear = false) => {
                        if (target == null)
                        {
                            gl.viewport(0, 0, gl.drawingBufferWidth, gl.drawingBufferHeight);
                            gl.bindFramebuffer(gl.FRAMEBUFFER, null);
                        }
                        else
                        {
                            gl.viewport(0, 0, target.width, target.height);
                            gl.bindFramebuffer(gl.FRAMEBUFFER, target.fbo);
                        }
                        if (clear)
                        {
                            gl.clearColor(0.0, 0.0, 0.0, 1.0);
                            gl.clear(gl.COLOR_BUFFER_BIT);
                        }
                        // CHECK_FRAMEBUFFER_STATUS();
                        gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);
                    }
                })();

                let dye;
                let velocity;
                let divergence;
                let curl;
                let pressure;
                let bloom;
                let bloomFramebuffers = [];
                let sunrays;
                let sunraysTemp;

                let ditheringTexture = createTextureAsync('LDR_LLL1_0.png');

                const blurProgram            = new Program(blurVertexShader, blurShader);
                const copyProgram            = new Program(baseVertexShader, copyShader);
                const clearProgram           = new Program(baseVertexShader, clearShader);
                const colorProgram           = new Program(baseVertexShader, colorShader);
                const checkerboardProgram    = new Program(baseVertexShader, checkerboardShader);
                const bloomPrefilterProgram  = new Program(baseVertexShader, bloomPrefilterShader);
                const bloomBlurProgram       = new Program(baseVertexShader, bloomBlurShader);
                const bloomFinalProgram      = new Program(baseVertexShader, bloomFinalShader);
                const sunraysMaskProgram     = new Program(baseVertexShader, sunraysMaskShader);
                const sunraysProgram         = new Program(baseVertexShader, sunraysShader);
                const splatProgram           = new Program(baseVertexShader, splatShader);
                const advectionProgram       = new Program(baseVertexShader, advectionShader);
                const divergenceProgram      = new Program(baseVertexShader, divergenceShader);
                const curlProgram            = new Program(baseVertexShader, curlShader);
                const vorticityProgram       = new Program(baseVertexShader, vorticityShader);
                const pressureProgram        = new Program(baseVertexShader, pressureShader);
                const gradienSubtractProgram = new Program(baseVertexShader, gradientSubtractShader);

                const displayMaterial = new Material(baseVertexShader, displayShaderSource);

                function initFramebuffers () {
                    let simRes = getResolution(config.SIM_RESOLUTION);
                    let dyeRes = getResolution(config.DYE_RESOLUTION);

                    const texType = ext.halfFloatTexType;
                    const rgba    = ext.formatRGBA;
                    const rg      = ext.formatRG;
                    const r       = ext.formatR;
                    const filtering = ext.supportLinearFiltering ? gl.LINEAR : gl.NEAREST;

                    gl.disable(gl.BLEND);

                    if (dye == null)
                        dye = createDoubleFBO(dyeRes.width, dyeRes.height, rgba.internalFormat, rgba.format, texType, filtering);
                    else
                        dye = resizeDoubleFBO(dye, dyeRes.width, dyeRes.height, rgba.internalFormat, rgba.format, texType, filtering);

                    if (velocity == null)
                        velocity = createDoubleFBO(simRes.width, simRes.height, rg.internalFormat, rg.format, texType, filtering);
                    else
                        velocity = resizeDoubleFBO(velocity, simRes.width, simRes.height, rg.internalFormat, rg.format, texType, filtering);

                    divergence = createFBO      (simRes.width, simRes.height, r.internalFormat, r.format, texType, gl.NEAREST);
                    curl       = createFBO      (simRes.width, simRes.height, r.internalFormat, r.format, texType, gl.NEAREST);
                    pressure   = createDoubleFBO(simRes.width, simRes.height, r.internalFormat, r.format, texType, gl.NEAREST);

                    initBloomFramebuffers();
                    initSunraysFramebuffers();
                }

                function initBloomFramebuffers () {
                    let res = getResolution(config.BLOOM_RESOLUTION);

                    const texType = ext.halfFloatTexType;
                    const rgba = ext.formatRGBA;
                    const filtering = ext.supportLinearFiltering ? gl.LINEAR : gl.NEAREST;

                    bloom = createFBO(res.width, res.height, rgba.internalFormat, rgba.format, texType, filtering);

                    bloomFramebuffers.length = 0;
                    for (let i = 0; i < config.BLOOM_ITERATIONS; i++)
                    {
                        let width = res.width >> (i + 1);
                        let height = res.height >> (i + 1);

                        if (width < 2 || height < 2) break;

                        let fbo = createFBO(width, height, rgba.internalFormat, rgba.format, texType, filtering);
                        bloomFramebuffers.push(fbo);
                    }
                }

                function initSunraysFramebuffers () {
                    let res = getResolution(config.SUNRAYS_RESOLUTION);

                    const texType = ext.halfFloatTexType;
                    const r = ext.formatR;
                    const filtering = ext.supportLinearFiltering ? gl.LINEAR : gl.NEAREST;

                    sunrays     = createFBO(res.width, res.height, r.internalFormat, r.format, texType, filtering);
                    sunraysTemp = createFBO(res.width, res.height, r.internalFormat, r.format, texType, filtering);
                }

                function createFBO (w, h, internalFormat, format, type, param) {
                    gl.activeTexture(gl.TEXTURE0);
                    let texture = gl.createTexture();
                    gl.bindTexture(gl.TEXTURE_2D, texture);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, param);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, param);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
                    gl.texImage2D(gl.TEXTURE_2D, 0, internalFormat, w, h, 0, format, type, null);

                    let fbo = gl.createFramebuffer();
                    gl.bindFramebuffer(gl.FRAMEBUFFER, fbo);
                    gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
                    gl.viewport(0, 0, w, h);
                    gl.clear(gl.COLOR_BUFFER_BIT);

                    let texelSizeX = 1.0 / w;
                    let texelSizeY = 1.0 / h;

                    return {
                        texture,
                        fbo,
                        width: w,
                        height: h,
                        texelSizeX,
                        texelSizeY,
                        attach (id) {
                            gl.activeTexture(gl.TEXTURE0 + id);
                            gl.bindTexture(gl.TEXTURE_2D, texture);
                            return id;
                        }
                    };
                }

                function createDoubleFBO (w, h, internalFormat, format, type, param) {
                    let fbo1 = createFBO(w, h, internalFormat, format, type, param);
                    let fbo2 = createFBO(w, h, internalFormat, format, type, param);

                    return {
                        width: w,
                        height: h,
                        texelSizeX: fbo1.texelSizeX,
                        texelSizeY: fbo1.texelSizeY,
                        get read () {
                            return fbo1;
                        },
                        set read (value) {
                            fbo1 = value;
                        },
                        get write () {
                            return fbo2;
                        },
                        set write (value) {
                            fbo2 = value;
                        },
                        swap () {
                            let temp = fbo1;
                            fbo1 = fbo2;
                            fbo2 = temp;
                        }
                    }
                }

                function resizeFBO (target, w, h, internalFormat, format, type, param) {
                    let newFBO = createFBO(w, h, internalFormat, format, type, param);
                    copyProgram.bind();
                    gl.uniform1i(copyProgram.uniforms.uTexture, target.attach(0));
                    blit(newFBO);
                    return newFBO;
                }

                function resizeDoubleFBO (target, w, h, internalFormat, format, type, param) {
                    if (target.width == w && target.height == h)
                        return target;
                    target.read = resizeFBO(target.read, w, h, internalFormat, format, type, param);
                    target.write = createFBO(w, h, internalFormat, format, type, param);
                    target.width = w;
                    target.height = h;
                    target.texelSizeX = 1.0 / w;
                    target.texelSizeY = 1.0 / h;
                    return target;
                }

                function createTextureAsync (url) {
                    let texture = gl.createTexture();
                    gl.bindTexture(gl.TEXTURE_2D, texture);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.REPEAT);
                    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.REPEAT);
                    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGB, 1, 1, 0, gl.RGB, gl.UNSIGNED_BYTE, new Uint8Array([255, 255, 255]));

                    let obj = {
                        texture,
                        width: 1,
                        height: 1,
                        attach (id) {
                            gl.activeTexture(gl.TEXTURE0 + id);
                            gl.bindTexture(gl.TEXTURE_2D, texture);
                            return id;
                        }
                    };

                    let image = new Image();
                    image.onload = () => {
                        obj.width = image.width;
                        obj.height = image.height;
                        gl.bindTexture(gl.TEXTURE_2D, texture);
                        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGB, gl.RGB, gl.UNSIGNED_BYTE, image);
                    };
                    image.src = url;

                    return obj;
                }

                function updateKeywords () {
                    let displayKeywords = [];
                    if (config.SHADING) displayKeywords.push("SHADING");
                    if (config.BLOOM) displayKeywords.push("BLOOM");
                    if (config.SUNRAYS) displayKeywords.push("SUNRAYS");
                    displayMaterial.setKeywords(displayKeywords);
                }

                updateKeywords();
                initFramebuffers();
                multipleSplats(parseInt(Math.random() * 20) + 5);

                let lastUpdateTime = Date.now();
                let colorUpdateTimer = 0.0;
                update();

                function update () {
                    const dt = calcDeltaTime();
                    if (resizeCanvas())
                        initFramebuffers();
                    updateColors(dt);
                    applyInputs();
                    if (!config.PAUSED)
                        step(dt);
                    render(null);
                    requestAnimationFrame(update);
                }

                function calcDeltaTime () {
                    let now = Date.now();
                    let dt = (now - lastUpdateTime) / 1000;
                    dt = Math.min(dt, 0.016666);
                    lastUpdateTime = now;
                    return dt;
                }

                function resizeCanvas () {
                    let width = scaleByPixelRatio(canvas.clientWidth);
                    let height = scaleByPixelRatio(canvas.clientHeight);
                    if (canvas.width != width || canvas.height != height) {
                        canvas.width = width;
                        canvas.height = height;
                        return true;
                    }
                    return false;
                }

                function updateColors (dt) {
                    if (!config.COLORFUL) return;

                    colorUpdateTimer += dt * config.COLOR_UPDATE_SPEED;
                    if (colorUpdateTimer >= 1) {
                        colorUpdateTimer = wrap(colorUpdateTimer, 0, 1);
                        pointers.forEach(p => {
                            p.color = generateColor();
                        });
                    }
                }

                function applyInputs () {
                    if (splatStack.length > 0)
                        multipleSplats(splatStack.pop());

                    pointers.forEach(p => {
                        if (p.moved) {
                            p.moved = false;
                            splatPointer(p);
                        }
                    });
                }

                function step (dt) {
                    gl.disable(gl.BLEND);

                    curlProgram.bind();
                    gl.uniform2f(curlProgram.uniforms.texelSize, velocity.texelSizeX, velocity.texelSizeY);
                    gl.uniform1i(curlProgram.uniforms.uVelocity, velocity.read.attach(0));
                    blit(curl);

                    vorticityProgram.bind();
                    gl.uniform2f(vorticityProgram.uniforms.texelSize, velocity.texelSizeX, velocity.texelSizeY);
                    gl.uniform1i(vorticityProgram.uniforms.uVelocity, velocity.read.attach(0));
                    gl.uniform1i(vorticityProgram.uniforms.uCurl, curl.attach(1));
                    gl.uniform1f(vorticityProgram.uniforms.curl, config.CURL);
                    gl.uniform1f(vorticityProgram.uniforms.dt, dt);
                    blit(velocity.write);
                    velocity.swap();

                    divergenceProgram.bind();
                    gl.uniform2f(divergenceProgram.uniforms.texelSize, velocity.texelSizeX, velocity.texelSizeY);
                    gl.uniform1i(divergenceProgram.uniforms.uVelocity, velocity.read.attach(0));
                    blit(divergence);

                    clearProgram.bind();
                    gl.uniform1i(clearProgram.uniforms.uTexture, pressure.read.attach(0));
                    gl.uniform1f(clearProgram.uniforms.value, config.PRESSURE);
                    blit(pressure.write);
                    pressure.swap();

                    pressureProgram.bind();
                    gl.uniform2f(pressureProgram.uniforms.texelSize, velocity.texelSizeX, velocity.texelSizeY);
                    gl.uniform1i(pressureProgram.uniforms.uDivergence, divergence.attach(0));
                    for (let i = 0; i < config.PRESSURE_ITERATIONS; i++) {
                        gl.uniform1i(pressureProgram.uniforms.uPressure, pressure.read.attach(1));
                        blit(pressure.write);
                        pressure.swap();
                    }

                    gradienSubtractProgram.bind();
                    gl.uniform2f(gradienSubtractProgram.uniforms.texelSize, velocity.texelSizeX, velocity.texelSizeY);
                    gl.uniform1i(gradienSubtractProgram.uniforms.uPressure, pressure.read.attach(0));
                    gl.uniform1i(gradienSubtractProgram.uniforms.uVelocity, velocity.read.attach(1));
                    blit(velocity.write);
                    velocity.swap();

                    advectionProgram.bind();
                    gl.uniform2f(advectionProgram.uniforms.texelSize, velocity.texelSizeX, velocity.texelSizeY);
                    if (!ext.supportLinearFiltering)
                        gl.uniform2f(advectionProgram.uniforms.dyeTexelSize, velocity.texelSizeX, velocity.texelSizeY);
                    let velocityId = velocity.read.attach(0);
                    gl.uniform1i(advectionProgram.uniforms.uVelocity, velocityId);
                    gl.uniform1i(advectionProgram.uniforms.uSource, velocityId);
                    gl.uniform1f(advectionProgram.uniforms.dt, dt);
                    gl.uniform1f(advectionProgram.uniforms.dissipation, config.VELOCITY_DISSIPATION);
                    blit(velocity.write);
                    velocity.swap();

                    if (!ext.supportLinearFiltering)
                        gl.uniform2f(advectionProgram.uniforms.dyeTexelSize, dye.texelSizeX, dye.texelSizeY);
                    gl.uniform1i(advectionProgram.uniforms.uVelocity, velocity.read.attach(0));
                    gl.uniform1i(advectionProgram.uniforms.uSource, dye.read.attach(1));
                    gl.uniform1f(advectionProgram.uniforms.dissipation, config.DENSITY_DISSIPATION);
                    blit(dye.write);
                    dye.swap();
                }

                function render (target) {
                    if (config.BLOOM)
                        applyBloom(dye.read, bloom);
                    if (config.SUNRAYS) {
                        applySunrays(dye.read, dye.write, sunrays);
                        blur(sunrays, sunraysTemp, 1);
                    }

                    if (target == null || !config.TRANSPARENT) {
                        gl.blendFunc(gl.ONE, gl.ONE_MINUS_SRC_ALPHA);
                        gl.enable(gl.BLEND);
                    }
                    else {
                        gl.disable(gl.BLEND);
                    }

                    if (!config.TRANSPARENT)
                        drawColor(target, normalizeColor(config.BACK_COLOR));
                    if (target == null && config.TRANSPARENT)
                        drawCheckerboard(target);
                    drawDisplay(target);
                }

                function drawColor (target, color) {
                    colorProgram.bind();
                    gl.uniform4f(colorProgram.uniforms.color, color.r, color.g, color.b, 1);
                    blit(target);
                }

                function drawCheckerboard (target) {
                    checkerboardProgram.bind();
                    gl.uniform1f(checkerboardProgram.uniforms.aspectRatio, canvas.width / canvas.height);
                    blit(target);
                }

                function drawDisplay (target) {
                    let width = target == null ? gl.drawingBufferWidth : target.width;
                    let height = target == null ? gl.drawingBufferHeight : target.height;

                    displayMaterial.bind();
                    if (config.SHADING)
                        gl.uniform2f(displayMaterial.uniforms.texelSize, 1.0 / width, 1.0 / height);
                    gl.uniform1i(displayMaterial.uniforms.uTexture, dye.read.attach(0));
                    if (config.BLOOM) {
                        gl.uniform1i(displayMaterial.uniforms.uBloom, bloom.attach(1));
                        gl.uniform1i(displayMaterial.uniforms.uDithering, ditheringTexture.attach(2));
                        let scale = getTextureScale(ditheringTexture, width, height);
                        gl.uniform2f(displayMaterial.uniforms.ditherScale, scale.x, scale.y);
                    }
                    if (config.SUNRAYS)
                        gl.uniform1i(displayMaterial.uniforms.uSunrays, sunrays.attach(3));
                    blit(target);
                }

                function applyBloom (source, destination) {
                    if (bloomFramebuffers.length < 2)
                        return;

                    let last = destination;

                    gl.disable(gl.BLEND);
                    bloomPrefilterProgram.bind();
                    let knee = config.BLOOM_THRESHOLD * config.BLOOM_SOFT_KNEE + 0.0001;
                    let curve0 = config.BLOOM_THRESHOLD - knee;
                    let curve1 = knee * 2;
                    let curve2 = 0.25 / knee;
                    gl.uniform3f(bloomPrefilterProgram.uniforms.curve, curve0, curve1, curve2);
                    gl.uniform1f(bloomPrefilterProgram.uniforms.threshold, config.BLOOM_THRESHOLD);
                    gl.uniform1i(bloomPrefilterProgram.uniforms.uTexture, source.attach(0));
                    blit(last);

                    bloomBlurProgram.bind();
                    for (let i = 0; i < bloomFramebuffers.length; i++) {
                        let dest = bloomFramebuffers[i];
                        gl.uniform2f(bloomBlurProgram.uniforms.texelSize, last.texelSizeX, last.texelSizeY);
                        gl.uniform1i(bloomBlurProgram.uniforms.uTexture, last.attach(0));
                        blit(dest);
                        last = dest;
                    }

                    gl.blendFunc(gl.ONE, gl.ONE);
                    gl.enable(gl.BLEND);

                    for (let i = bloomFramebuffers.length - 2; i >= 0; i--) {
                        let baseTex = bloomFramebuffers[i];
                        gl.uniform2f(bloomBlurProgram.uniforms.texelSize, last.texelSizeX, last.texelSizeY);
                        gl.uniform1i(bloomBlurProgram.uniforms.uTexture, last.attach(0));
                        gl.viewport(0, 0, baseTex.width, baseTex.height);
                        blit(baseTex);
                        last = baseTex;
                    }

                    gl.disable(gl.BLEND);
                    bloomFinalProgram.bind();
                    gl.uniform2f(bloomFinalProgram.uniforms.texelSize, last.texelSizeX, last.texelSizeY);
                    gl.uniform1i(bloomFinalProgram.uniforms.uTexture, last.attach(0));
                    gl.uniform1f(bloomFinalProgram.uniforms.intensity, config.BLOOM_INTENSITY);
                    blit(destination);
                }

                function applySunrays (source, mask, destination) {
                    gl.disable(gl.BLEND);
                    sunraysMaskProgram.bind();
                    gl.uniform1i(sunraysMaskProgram.uniforms.uTexture, source.attach(0));
                    blit(mask);

                    sunraysProgram.bind();
                    gl.uniform1f(sunraysProgram.uniforms.weight, config.SUNRAYS_WEIGHT);
                    gl.uniform1i(sunraysProgram.uniforms.uTexture, mask.attach(0));
                    blit(destination);
                }

                function blur (target, temp, iterations) {
                    blurProgram.bind();
                    for (let i = 0; i < iterations; i++) {
                        gl.uniform2f(blurProgram.uniforms.texelSize, target.texelSizeX, 0.0);
                        gl.uniform1i(blurProgram.uniforms.uTexture, target.attach(0));
                        blit(temp);

                        gl.uniform2f(blurProgram.uniforms.texelSize, 0.0, target.texelSizeY);
                        gl.uniform1i(blurProgram.uniforms.uTexture, temp.attach(0));
                        blit(target);
                    }
                }

                function splatPointer (pointer) {
                    let dx = pointer.deltaX * config.SPLAT_FORCE;
                    let dy = pointer.deltaY * config.SPLAT_FORCE;
                    splat(pointer.texcoordX, pointer.texcoordY, dx, dy, pointer.color);
                }

                function multipleSplats (amount) {
                    for (let i = 0; i < amount; i++) {
                        const color = generateColor();
                        color.r *= 10.0;
                        color.g *= 10.0;
                        color.b *= 10.0;
                        const x = Math.random();
                        const y = Math.random();
                        const dx = 1000 * (Math.random() - 0.5);
                        const dy = 1000 * (Math.random() - 0.5);
                        splat(x, y, dx, dy, color);
                    }
                }

                function splat (x, y, dx, dy, color) {
                    splatProgram.bind();
                    gl.uniform1i(splatProgram.uniforms.uTarget, velocity.read.attach(0));
                    gl.uniform1f(splatProgram.uniforms.aspectRatio, canvas.width / canvas.height);
                    gl.uniform2f(splatProgram.uniforms.point, x, y);
                    gl.uniform3f(splatProgram.uniforms.color, dx, dy, 0.0);
                    gl.uniform1f(splatProgram.uniforms.radius, correctRadius(config.SPLAT_RADIUS / 100.0));
                    blit(velocity.write);
                    velocity.swap();

                    gl.uniform1i(splatProgram.uniforms.uTarget, dye.read.attach(0));
                    gl.uniform3f(splatProgram.uniforms.color, color.r, color.g, color.b);
                    blit(dye.write);
                    dye.swap();
                }

                function correctRadius (radius) {
                    let aspectRatio = canvas.width / canvas.height;
                    if (aspectRatio > 1)
                        radius *= aspectRatio;
                    return radius;
                }

                canvas.addEventListener('mousedown', e => {
                    let posX = scaleByPixelRatio(e.offsetX);
                    let posY = scaleByPixelRatio(e.offsetY);
                    let pointer = pointers.find(p => p.id == -1);
                    if (pointer == null)
                        pointer = new pointerPrototype();
                    updatePointerDownData(pointer, -1, posX, posY);
                });

                document.addEventListener('mousemove', e => {
                    let pointer = pointers[0];
                    //if (!pointer.down) return;

                    // Get canvas position and size
                    const canvasRect = canvas.getBoundingClientRect();

                    // Calculate mouse position relative to the canvas
                    let posX = scaleByPixelRatio(e.clientX - canvasRect.left);
                    let posY = scaleByPixelRatio(e.clientY - canvasRect.top);
                    updatePointerMoveData(pointer, posX, posY);
                });

                window.addEventListener('mouseup', () => {
                    updatePointerUpData(pointers[0]);
                });

                canvas.addEventListener('touchstart', e => {
                    e.preventDefault();
                    const touches = e.targetTouches;
                    while (touches.length >= pointers.length)
                        pointers.push(new pointerPrototype());
                    for (let i = 0; i < touches.length; i++) {
                        let posX = scaleByPixelRatio(touches[i].pageX);
                        let posY = scaleByPixelRatio(touches[i].pageY);
                        updatePointerDownData(pointers[i + 1], touches[i].identifier, posX, posY);
                    }
                });

                canvas.addEventListener('touchmove', e => {
                    e.preventDefault();
                    const touches = e.targetTouches;
                    for (let i = 0; i < touches.length; i++) {
                        let pointer = pointers[i + 1];
                        if (!pointer.down) continue;
                        let posX = scaleByPixelRatio(touches[i].pageX);
                        let posY = scaleByPixelRatio(touches[i].pageY);
                        updatePointerMoveData(pointer, posX, posY);
                    }
                }, false);

                window.addEventListener('touchend', e => {
                    const touches = e.changedTouches;
                    for (let i = 0; i < touches.length; i++)
                    {
                        let pointer = pointers.find(p => p.id == touches[i].identifier);
                        if (pointer == null) continue;
                        updatePointerUpData(pointer);
                    }
                });

                window.addEventListener('keydown', e => {
                    if (e.code === 'KeyP')
                        config.PAUSED = !config.PAUSED;
                    if (e.key === ' ')
                        splatStack.push(parseInt(Math.random() * 20) + 5);
                });

                function updatePointerDownData (pointer, id, posX, posY) {
                    pointer.id = id;
                    pointer.down = true;
                    pointer.moved = false;
                    pointer.texcoordX = posX / canvas.width;
                    pointer.texcoordY = 1.0 - posY / canvas.height;
                    pointer.prevTexcoordX = pointer.texcoordX;
                    pointer.prevTexcoordY = pointer.texcoordY;
                    pointer.deltaX = 0;
                    pointer.deltaY = 0;
                    pointer.color = generateColor();
                }

                function updatePointerMoveData (pointer, posX, posY) {
                    pointer.prevTexcoordX = pointer.texcoordX;
                    pointer.prevTexcoordY = pointer.texcoordY;
                    pointer.texcoordX = posX / canvas.width;
                    pointer.texcoordY = 1.0 - posY / canvas.height;
                    pointer.deltaX = correctDeltaX(pointer.texcoordX - pointer.prevTexcoordX);
                    pointer.deltaY = correctDeltaY(pointer.texcoordY - pointer.prevTexcoordY);
                    pointer.moved = Math.abs(pointer.deltaX) > 0 || Math.abs(pointer.deltaY) > 0;
                }

                function updatePointerUpData (pointer) {
                    pointer.down = false;
                }

                function correctDeltaX (delta) {
                    let aspectRatio = canvas.width / canvas.height;
                    if (aspectRatio < 1) delta *= aspectRatio;
                    return delta;
                }

                function correctDeltaY (delta) {
                    let aspectRatio = canvas.width / canvas.height;
                    if (aspectRatio > 1) delta /= aspectRatio;
                    return delta;
                }

                function generateColor () {
                    let c = HSVtoRGB(Math.random(), 1.0, 1.0);
                    c.r *= 0.15;
                    c.g *= 0.15;
                    c.b *= 0.15;
                    return c;
                }

                function HSVtoRGB (h, s, v) {
                    let r, g, b, i, f, p, q, t;
                    i = Math.floor(h * 6);
                    f = h * 6 - i;
                    p = v * (1 - s);
                    q = v * (1 - f * s);
                    t = v * (1 - (1 - f) * s);

                    switch (i % 6) {
                        case 0: r = v, g = t, b = p; break;
                        case 1: r = q, g = v, b = p; break;
                        case 2: r = p, g = v, b = t; break;
                        case 3: r = p, g = q, b = v; break;
                        case 4: r = t, g = p, b = v; break;
                        case 5: r = v, g = p, b = q; break;
                    }

                    return {
                        r,
                        g,
                        b
                    };
                }

                function normalizeColor (input) {
                    let output = {
                        r: input.r / 255,
                        g: input.g / 255,
                        b: input.b / 255
                    };
                    return output;
                }

                function wrap (value, min, max) {
                    let range = max - min;
                    if (range == 0) return min;
                    return (value - min) % range + min;
                }

                function getResolution (resolution) {
                    let aspectRatio = gl.drawingBufferWidth / gl.drawingBufferHeight;
                    if (aspectRatio < 1)
                        aspectRatio = 1.0 / aspectRatio;

                    let min = Math.round(resolution);
                    let max = Math.round(resolution * aspectRatio);

                    if (gl.drawingBufferWidth > gl.drawingBufferHeight)
                        return { width: max, height: min };
                    else
                        return { width: min, height: max };
                }

                function getTextureScale (texture, width, height) {
                    return {
                        x: width / texture.width,
                        y: height / texture.height
                    };
                }

                function scaleByPixelRatio (input) {
                    let pixelRatio = window.devicePixelRatio || 1;
                    return Math.floor(input * pixelRatio);
                }

                function hashCode (s) {
                    if (s.length == 0) return 0;
                    let hash = 0;
                    for (let i = 0; i < s.length; i++) {
                        hash = (hash << 5) - hash + s.charCodeAt(i);
                        hash |= 0; // Convert to 32bit integer
                    }
                    return hash;
                };

                // Clean up function
                return () => {
                    if (nyxamokeRef.current) {
                        // Clean up event listeners
                        canvas.removeEventListener('mousedown', () => {});
                        document.removeEventListener('mousemove', () => {});
                        window.removeEventListener('mouseup', () => {});
                        canvas.removeEventListener('touchstart', () => {});
                        canvas.removeEventListener('touchmove', () => {});
                        window.removeEventListener('touchend', () => {});
                        window.removeEventListener('keydown', () => {});
    
                        // remove the canvas
                        nyxamokeRef.current.removeChild(canvas);
                    }
                };
            }
        }
        //Aphorbit
        if (cursor.id === 16) {
            if (aphorbitRef.current) {
                const targetDiv = aphorbitRef.current;

                class ImageCursor {
                    constructor(config, container) {
                        this.config = config;
                        this.container = container; // Reference to targetDiv
                        this.cursorElement = null;
                        this.activePointX = (this.config.cursor.image.activePoint?.x || 0) / 100;
                        this.activePointY = (this.config.cursor.image.activePoint?.y || 0) / 100;

                        // Store event listeners for removal
                        this.mouseMoveHandler = this.handleMouseMove.bind(this);
                        this.mouseLeaveHandler = this.handleMouseLeave.bind(this);

                        this.init();
                    }

                    init() {
                        // Create the cursor wrapper and image element
                        this.cursorElement = document.createElement('div');
                        this.cursorImage = document.createElement('img');

                        // Apply wrapper styles
                        this.applyWrapperStyles();

                        // Apply initial image styles
                        this.applyImageStyles(this.config.cursor.image);

                        // Append the image to the wrapper
                        this.cursorElement.appendChild(this.cursorImage);

                        // Add the wrapper to the targetDiv
                        this.container.appendChild(this.cursorElement);

                        // Center cursor on initial load
                        this.centerCursor();

                        // Track mouse events inside the container
                        this.container.addEventListener('mousemove', this.mouseMoveHandler);
                        this.container.addEventListener('mouseleave', this.mouseLeaveHandler);
                    }

                    applyWrapperStyles() {
                        const wrapperStyles = {
                            position: 'absolute', // Relative to the container
                            top: '0',
                            left: '0',
                            pointerEvents: 'none',
                            zIndex: '99',
                            opacity: this.config.cursor.image.opacity || 1,
                            mixBlendMode: this.config.cursor.image.mixBlendMode || 'normal',
                            transform: 'translate(-50%, -50%)',
                            transition: `${this.config.cursor.image.transitionDuration || '0.2s'} ${this.config.cursor.image.transitionTimingFunction || 'ease-out'}`,
                        };

                        Object.assign(this.cursorElement.style, wrapperStyles);
                    }

                    applyImageStyles(imageConfig) {
                        const imageStyles = {
                            width: imageConfig.width || '100px',
                            backgroundColor: imageConfig.backgroundColor || 'transparent',
                            borderRadius: imageConfig.borderRadius || '0',
                            padding: imageConfig.padding || '0',
                        };
                        this.cursorImage.src = imageConfig.imageUrl;
                        Object.assign(this.cursorImage.style, imageStyles);
                    }

                    centerCursor() {
                        const rect = this.container.getBoundingClientRect();
                        const centerX = rect.width / 2;
                        const centerY = rect.height / 2;

                        this.cursorElement.style.transform = `translate(${centerX}px, ${centerY}px)`;
                    }

                    handleMouseMove(e) {
                        const rect = this.container.getBoundingClientRect();

                        // Calculate the cursor position relative to the container
                        const offsetX = e.clientX - rect.left - this.cursorElement.offsetWidth * this.activePointX;
                        const offsetY = e.clientY - rect.top - this.cursorElement.offsetHeight * this.activePointY;

                        this.cursorElement.style.transform = `translate(${offsetX}px, ${offsetY}px)`;
                    }

                    handleMouseLeave() {
                        // Reset cursor to the center of the container
                        this.centerCursor();
                    }

                    // Cleanup function to remove elements and event listeners
                    destroy() {
                        if (this.container && this.cursorElement) {
                            this.container.removeEventListener('mousemove', this.mouseMoveHandler);
                            this.container.removeEventListener('mouseleave', this.mouseLeaveHandler);

                            this.container.removeChild(this.cursorElement); // Remove cursor element
                        }

                        this.cursorElement = null; // Prevent memory leaks
                        this.cursorImage = null;
                    }
                }

                // Initialize the custom cursor
                const imageCursor = new ImageCursor(cursor, targetDiv);

                // Return the destroy method to be called when cleaning up (e.g., unmounting the component)
                return () => {
                    imageCursor.destroy();
                };
            }
        }
        //Persephont
        if (cursor.id === 17) {
            if (persephontRef.current) {
                const targetDiv = persephontRef.current;

                // Cleanup function to remove existing cursor
                const cleanupCursor = () => {
                    const existingCursor = targetDiv.querySelector('.cursor-wrapper');
                    if (existingCursor) {
                        targetDiv.removeChild(existingCursor);
                    }
                };

                // Remove any existing cursor before initializing a new one
                cleanupCursor();

                class CustomCursor {
                    constructor(cursorObject) {
                        this.cursorObject = cursorObject;
                        this.cursorWrapper = document.createElement('div');
                        this.cursorInner = document.createElement('div');
                        this.cursorOuter = document.createElement('div');
                        this.hoverTargets = [];
                        this.init();
                    }

                    init() {
                        this.cursorWrapper.className = 'cursor-wrapper';
                        this.cursorWrapper.style.position = 'absolute';  // Changed to absolute
                        this.cursorWrapper.style.top = '0';
                        this.cursorWrapper.style.left = '0';
                        this.cursorWrapper.style.width = '100%';  // Added width
                        this.cursorWrapper.style.height = '100%'; // Added height
                        this.cursorWrapper.style.pointerEvents = 'none';
                        this.cursorWrapper.style.willChange = 'transform';

                        this.cursorInner.className = 'cursor-inner';
                        this.cursorOuter.className = 'cursor-outer';

                        this.applyBaseStyles(this.cursorInner);
                        this.applyBaseStyles(this.cursorOuter);

                        this.cursorInner.style.display = 'flex';
                        this.cursorInner.style.alignItems = 'center';
                        this.cursorInner.style.justifyContent = 'center';

                        this.cursorWrapper.appendChild(this.cursorInner);
                        this.cursorWrapper.appendChild(this.cursorOuter);

                        targetDiv.appendChild(this.cursorWrapper);

                        this.applyStyles(this.cursorInner, this.cursorObject.cursor.inner);
                        this.applyStyles(this.cursorOuter, this.cursorObject.cursor.outer);

                        // Position cursor at center initially
                        this.centerCursor();

                        targetDiv.addEventListener('mousemove', this.updateCursorPosition.bind(this));
                        targetDiv.addEventListener('mouseleave', this.centerCursor.bind(this));
                        this.addHoverListeners();
                    }

                    applyBaseStyles(element) {
                        element.style.position = 'absolute';
                        element.style.top = '0';
                        element.style.left = '0';
                        element.style.pointerEvents = 'none';
                        element.style.willChange = 'transform';
                    }

                    applyStyles(element, styles) {
                        for (const [key, value] of Object.entries(styles)) {
                            if (key === 'backdropFilter') {
                                element.style.backdropFilter = value;
                                element.style.webkitBackdropFilter = value;
                            } else if (key !== 'text') {
                                element.style[key] = value;
                            }
                        }
                        if (styles.text) {
                            element.textContent = styles.text;
                        }
                    }

                    centerCursor() {
                        const wrapperRect = this.cursorWrapper.getBoundingClientRect();
                        const centerX = wrapperRect.width / 2;
                        const centerY = wrapperRect.height / 2;

                        const outerWidth = parseFloat(window.getComputedStyle(this.cursorOuter).width);
                        const outerHeight = parseFloat(window.getComputedStyle(this.cursorOuter).height);
                        const innerWidth = parseFloat(window.getComputedStyle(this.cursorInner).width);
                        const innerHeight = parseFloat(window.getComputedStyle(this.cursorInner).height);

                        this.cursorOuter.style.transform = `translate(${centerX - outerWidth / 2}px, ${centerY - outerHeight / 2}px)`;
                        this.cursorInner.style.transform = `translate(${centerX - innerWidth / 2}px, ${centerY - innerHeight / 2}px)`;
                    }

                    updateCursorPosition(event) {
                        const wrapperRect = this.cursorWrapper.getBoundingClientRect();
                        const x = event.clientX - wrapperRect.left;
                        const y = event.clientY - wrapperRect.top;

                        const outerWidth = parseFloat(window.getComputedStyle(this.cursorOuter).width);
                        const outerHeight = parseFloat(window.getComputedStyle(this.cursorOuter).height);
                        const innerWidth = parseFloat(window.getComputedStyle(this.cursorInner).width);
                        const innerHeight = parseFloat(window.getComputedStyle(this.cursorInner).height);

                        this.cursorOuter.style.transform = `translate(${x - outerWidth / 2}px, ${y - outerHeight / 2}px)`;
                        this.cursorInner.style.transform = `translate(${x - innerWidth / 2}px, ${y - innerHeight / 2}px)`;
                    }

                    addHoverListeners() {
                        const hoverTargets = document.querySelectorAll(this.cursorObject.hoverSelector);

                        hoverTargets.forEach((target) => {
                            target.addEventListener('mouseenter', () => {
                                this.applyStyles(this.cursorInner, this.cursorObject.hover.inner);
                                this.applyStyles(this.cursorOuter, this.cursorObject.hover.outer);
                            });

                            target.addEventListener('mouseleave', () => {
                                this.applyStyles(this.cursorInner, this.cursorObject.cursor.inner);
                                this.applyStyles(this.cursorOuter, this.cursorObject.cursor.outer);
                            });
                        });
                    }
                }

                new CustomCursor(cursor);
            }
        }
        //Iriadient
        if (cursor.id === 18) {
            if (iriadientRef.current) {
                const targetDiv = iriadientRef.current;
                // Cleanup function to remove existing cursor
                const cleanupCursor = () => {
                    const existingCursor = targetDiv.querySelector('.custom-cursor');
                    if (existingCursor) {
                        targetDiv.removeChild(existingCursor);
                    }
                };

                // Remove any existing cursor before initializing a new one
                cleanupCursor();

                const centerShapesArray = {
                    1: "M23.9996 12.0235C17.5625 12.4117 12.4114 17.563 12.0232 24H11.9762C11.588 17.563 6.4369 12.4117 0 12.0235V11.9765C6.4369 11.5883 11.588 6.43719 11.9762 0H12.0232C12.4114 6.43719 17.5625 11.5883 23.9996 11.9765V12.0235Z",
                    2: "M16.5962 1.03651L22.9428 7.38312C23.1381 7.57838 23.1381 7.89496 22.9428 8.09022C22.8679 8.16513 22.7712 8.21431 22.6665 8.23067L21.1924 8.46113L15.5356 2.80428L15.7477 1.31935C15.7868 1.04599 16.04 0.856036 16.3134 0.895088C16.4205 0.910388 16.5197 0.960011 16.5962 1.03651ZM4.59487 20.1478C8.31725 16.8163 12.5899 15.82 17.2379 14.6273L17.6843 10.6099L13.3869 6.31241L9.36936 6.7588C8.17674 11.4068 7.18038 15.6795 3.84886 19.4018L2.4541 18.0071C5.28253 14.7072 6.34319 11.0539 7.7574 4.9256L14.1214 4.21849L19.7783 9.87539L19.0711 16.2393C12.9429 17.6535 9.28947 18.7142 5.98964 21.5426L4.59487 20.1478ZM9.87872 14.118C9.09767 13.3369 9.09767 12.0706 9.87872 11.2896C10.6598 10.5085 11.9261 10.5085 12.7071 11.2896C13.4882 12.0706 13.4882 13.3369 12.7071 14.118C11.9261 14.899 10.6598 14.899 9.87872 14.118Z",
                    3: "M16.0037 9.41421L7.39712 18.0208L5.98291 16.6066L14.5895 8H7.00373V6H18.0037V17H16.0037V9.41421Z",
                    4: "M5.82843 6.99955L8.36396 9.53509L6.94975 10.9493L2 5.99955L6.94975 1.0498L8.36396 2.46402L5.82843 4.99955H13C17.4183 4.99955 21 8.58127 21 12.9996C21 17.4178 17.4183 20.9996 13 20.9996H4V18.9996H13C16.3137 18.9996 19 16.3133 19 12.9996C19 9.68584 16.3137 6.99955 13 6.99955H5.82843Z",
                    5: "M11 20L1 12L11 4V9C16.5228 9 21 13.4772 21 19C21 19.2727 20.9891 19.5428 20.9677 19.81C19.5055 17.0364 16.6381 15.119 13.313 15.0053L13 15H10.9999L11 20ZM8.99986 13H10.9999L13.0341 13.0003L13.3814 13.0065C14.6657 13.0504 15.9053 13.3165 17.0568 13.7734C15.5898 12.0749 13.4204 11 11 11H9V8.16125L4.20156 12L8.99992 15.8387L8.99986 13Z",
                    6: "M16.9337 8.96494C16.426 5.03562 13.0675 2 9 2 4.58172 2 1 5.58172 1 10 1 11.8924 1.65707 13.6313 2.7555 15.0011 3.56351 16.0087 4.00033 17.1252 4.00025 18.3061L4 22H13L13.001 19H15C16.1046 19 17 18.1046 17 17V14.071L18.9593 13.2317C19.3025 13.0847 19.3324 12.7367 19.1842 12.5037L16.9337 8.96494ZM3 10C3 6.68629 5.68629 4 9 4 12.0243 4 14.5665 6.25141 14.9501 9.22118L15.0072 9.66262 16.5497 12.0881 15 12.7519V17H11.0017L11.0007 20H6.00013L6.00025 18.3063C6.00036 16.6672 5.40965 15.114 4.31578 13.7499 3.46818 12.6929 3 11.3849 3 10ZM21.1535 18.1024 19.4893 16.9929C20.4436 15.5642 21 13.8471 21 12.0001 21 10.153 20.4436 8.4359 19.4893 7.00722L21.1535 5.89771C22.32 7.64386 23 9.74254 23 12.0001 23 14.2576 22.32 16.3562 21.1535 18.1024Z",
                    7: "M17.0003 13V14C17.0003 16.7696 16.3364 19.445 15.0853 21.8455L14.8585 22.2663L13.1116 21.2924C14.2716 19.2115 14.9211 16.8817 14.9935 14.4559L15.0003 14V13H17.0003ZM11.0003 10H13.0003V14L12.9948 14.3787C12.9153 17.1495 11.9645 19.7731 10.3038 21.928L10.073 22.2189L8.52406 20.9536C10.0408 19.0969 10.9145 16.8017 10.9943 14.3663L11.0003 14V10ZM12.0003 6C14.7617 6 17.0003 8.23858 17.0003 11H15.0003C15.0003 9.34315 13.6571 8 12.0003 8C10.3434 8 9.00025 9.34315 9.00025 11V14C9.00025 16.2354 8.1806 18.3444 6.72928 19.9768L6.51767 20.2067L5.06955 18.8273C6.23328 17.6056 6.92099 16.0118 6.99381 14.3027L7.00025 14V11C7.00025 8.23858 9.23883 6 12.0003 6ZM12.0003 2C16.9708 2 21.0003 6.02944 21.0003 11V14C21.0003 15.6979 20.7985 17.3699 20.4035 18.9903L20.2647 19.5285L18.3349 19.0032C18.726 17.5662 18.9475 16.0808 18.9919 14.5684L19.0003 14V11C19.0003 7.13401 15.8662 4 12.0003 4C10.4279 4 8.97663 4.51841 7.80805 5.39364L6.38308 3.96769C7.92267 2.73631 9.87547 2 12.0003 2ZM4.96794 5.38282L6.39389 6.8078C5.5635 7.91652 5.0543 9.27971 5.00431 10.7593L4.99961 10.999L5.00378 13C5.00378 14.1195 4.73991 15.2026 4.24263 16.1772L4.08648 16.4663L2.34961 15.4747C2.72889 14.8103 2.95077 14.0681 2.99539 13.2924L3.00378 13L3.00361 11C3.00025 8.87522 3.73656 6.92242 4.96794 5.38282Z",
                    8: "M12.5002 2C12.2241 2 12.0002 2.22386 12.0002 2.5V12H10.0002V4.5C10.0002 4.22386 9.77634 4 9.5002 4C9.22405 4 9.0002 4.22386 9.0002 4.5V14C8.64653 14 7.00024 14 7.00024 14C6.61911 12.3792 5.64236 11.4407 4.5954 11.3216C4.87926 12.0664 5.36117 13.2592 6.16634 15.0995C7.02511 17.0622 7.89128 18.5218 9.00374 19.4986C10.0783 20.442 11.4586 21 13.5002 21C16.5378 21 19.0002 18.5377 19.0002 15.5002V7C19.0002 6.72386 18.7763 6.5 18.5002 6.5C18.2241 6.5 18.0002 6.72386 18.0002 7V12H16.0002V4C16.0002 3.72386 15.7763 3.5 15.5002 3.5C15.2241 3.5 15.0002 3.72386 15.0002 4V12H13.0002V2.5C13.0002 2.22386 12.7763 2 12.5002 2ZM21.0002 15.5002C21.0002 19.6424 17.6423 23 13.5002 23C11.0417 23 9.17214 22.308 7.68416 21.0015C6.23411 19.7283 5.22528 17.9381 4.33405 15.9012C3.40393 13.7753 2.89004 12.4804 2.60991 11.7235C2.25318 10.7597 2.74616 9.41212 4.08583 9.31846C5.24076 9.23771 6.22061 9.61249 7.0002 10.2587V4.5C7.0002 3.11929 8.11949 2 9.5002 2C9.68522 2 9.86554 2.0201 10.0391 2.05823C10.2477 0.888227 11.2702 0 12.5002 0C13.5602 0 14.4661 0.659694 14.8298 1.59091C15.0431 1.53167 15.268 1.5 15.5002 1.5C16.8809 1.5 18.0002 2.61929 18.0002 4V4.55001C18.1618 4.51722 18.329 4.5 18.5002 4.5C19.8809 4.5 21.0002 5.61929 21.0002 7V15.5002Z",
                    9: "M16.5 3C19.5376 3 22 5.5 22 9C22 16 14.5 20 12 21.5C9.5 20 2 16 2 9C2 5.5 4.5 3 7.5 3C9.35997 3 11 4 12 5C13 4 14.64 3 16.5 3ZM12.9339 18.6038C13.8155 18.0485 14.61 17.4955 15.3549 16.9029C18.3337 14.533 20 11.9435 20 9C20 6.64076 18.463 5 16.5 5C15.4241 5 14.2593 5.56911 13.4142 6.41421L12 7.82843L10.5858 6.41421C9.74068 5.56911 8.5759 5 7.5 5C5.55906 5 4 6.6565 4 9C4 11.9435 5.66627 14.533 8.64514 16.9029C9.39 17.4955 10.1845 18.0485 11.0661 18.6038C11.3646 18.7919 11.6611 18.9729 12 19.1752C12.3389 18.9729 12.6354 18.7919 12.9339 18.6038Z",
                    10: "M10.6144 17.7956C10.277 18.5682 9.20776 18.5682 8.8704 17.7956L7.99275 15.7854C7.21171 13.9966 5.80589 12.5726 4.0523 11.7942L1.63658 10.7219C.868536 10.381.868537 9.26368 1.63658 8.92276L3.97685 7.88394C5.77553 7.08552 7.20657 5.60881 7.97427 3.75892L8.8633 1.61673C9.19319.821767 10.2916.821765 10.6215 1.61673L11.5105 3.75894C12.2782 5.60881 13.7092 7.08552 15.5079 7.88394L17.8482 8.92276C18.6162 9.26368 18.6162 10.381 17.8482 10.7219L15.4325 11.7942C13.6789 12.5726 12.2731 13.9966 11.492 15.7854L10.6144 17.7956ZM4.53956 9.82234C6.8254 10.837 8.68402 12.5048 9.74238 14.7996 10.8008 12.5048 12.6594 10.837 14.9452 9.82234 12.6321 8.79557 10.7676 7.04647 9.74239 4.71088 8.71719 7.04648 6.85267 8.79557 4.53956 9.82234ZM19.4014 22.6899 19.6482 22.1242C20.0882 21.1156 20.8807 20.3125 21.8695 19.8732L22.6299 19.5353C23.0412 19.3526 23.0412 18.7549 22.6299 18.5722L21.9121 18.2532C20.8978 17.8026 20.0911 16.9698 19.6586 15.9269L19.4052 15.3156C19.2285 14.8896 18.6395 14.8896 18.4628 15.3156L18.2094 15.9269C17.777 16.9698 16.9703 17.8026 15.956 18.2532L15.2381 18.5722C14.8269 18.7549 14.8269 19.3526 15.2381 19.5353L15.9985 19.8732C16.9874 20.3125 17.7798 21.1156 18.2198 22.1242L18.4667 22.6899C18.6473 23.104 19.2207 23.104 19.4014 22.6899ZM18.3745 19.0469 18.937 18.4883 19.4878 19.0469 18.937 19.5898 18.3745 19.0469Z",
                };
        
                class CustomCursor {
                    constructor(config) {
                        this.svgNS = "http://www.w3.org/2000/svg";
                        this.config = config;
                        this.currentX = 0;
                        this.currentY = 0;
                        this.isHovering = false;
                        this.gradientColors = this.generateRandomColors();
                        this.targetColors = this.generateRandomColors();
                        this.progress = 0;
                        this.rotation = 0;
        
                        this.init();
                        this.setupEventListeners();
                        this.startColorTransition();
                        this.startRotation();
                    }
        
                    init() {
                        const cursor = document.createElement('div');
                        cursor.className = 'custom-cursor';
                        cursor.style.position = 'absolute'; // Changed from fixed to absolute
                        cursor.style.pointerEvents = 'none';
                        cursor.style.zIndex = '99';
                        cursor.style.mixBlendMode = this.config.cursor.outerShape.mixBlendMode;
                        cursor.style.willChange = 'transform';
                        cursor.style.top = '0';
                        cursor.style.left = '0';
        
                        // Initialize SVG and circle
                        this.radius = this.config.cursor.outerShape.radius; // Default radius
                        this.cx = this.radius; // Center X
                        this.cy = this.radius; // Center Y
        
                        this.svg = document.createElementNS(this.svgNS, 'svg');
                        this.svg.style.transition = `transform 300ms ease-out`;
                        this.updateSVGDimensions(); // Set initial SVG dimensions
        
                        this.circle = document.createElementNS(this.svgNS, 'circle');
                        this.circle.setAttribute('cx', this.cx);
                        this.circle.setAttribute('cy', this.cy);
                        this.circle.setAttribute('r', this.radius);
                        this.circle.setAttribute('opacity', this.config.cursor.outerShape.opacity);
                        this.circle.style.transformOrigin = `${this.cx}px ${this.cy}px`;
        
                        // Apply transition properties
                        const transitionDuration = this.config.cursor.outerShape.transitionDuration || '0.1s';
                        const transitionTimingFunction = this.config.cursor.outerShape.transitionTimingFunction || 'ease-out';
                        cursor.style.transition = `transform ${transitionDuration} ${transitionTimingFunction}`;
        
                        // Gradient setup
                        const gradient = document.createElementNS(this.svgNS, 'linearGradient');
                        gradient.setAttribute('id', 'cursor-gradient');
                        gradient.setAttribute('x1', '0%');
                        gradient.setAttribute('y1', '0%');
                        gradient.setAttribute('x2', '100%');
                        gradient.setAttribute('y2', '100%');
        
                        const stop1 = document.createElementNS(this.svgNS, 'stop');
                        stop1.setAttribute('offset', '0%');
        
                        const stop2 = document.createElementNS(this.svgNS, 'stop');
                        stop2.setAttribute('offset', '100%');
        
                        gradient.appendChild(stop1);
                        gradient.appendChild(stop2);
        
                        this.circle.setAttribute('fill', 'url(#cursor-gradient)');
        
                        // Center shape setup
                        this.shapeGroup = document.createElementNS(this.svgNS, "g");
                        this.shapeGroup.setAttribute("class", "center-group");
                        this.shapeGroup.setAttribute("data-type", this.config.cursor.centerShape.type);
        
                        this.shapePath = document.createElementNS(this.svgNS, "path");
                        const pathData = centerShapesArray[this.config.cursor.centerShape.type] || centerShapesArray[3];
                        this.shapePath.setAttribute("d", pathData);
        
                        // Style the path
                        this.shapePath.setAttribute("fill", this.config.cursor.centerShape.fill || "black");
                        this.shapePath.setAttribute("opacity", this.config.cursor.centerShape.opacity || 1);
        
                        // Append the path to the group
                        this.shapeGroup.appendChild(this.shapePath);
        
                        // Append elements to SVG
                        this.svg.appendChild(gradient);
                        this.svg.appendChild(this.circle);
                        this.svg.appendChild(this.shapeGroup);
        
                        // Append SVG to cursor
                        cursor.appendChild(this.svg);
        
                        // Append cursor to document body
                        targetDiv.appendChild(cursor);
        
                        // Store references
                        this.cursor = cursor;
                        this.gradient = gradient;
                        this.stop1 = stop1;
                        this.stop2 = stop2;
        
                        // Center the shape initially
                        this.centerShape();
        
                        // Position the cursor at the center of the wrapper on page load
                        this.centerCursor();
                    }
        
                    centerCursor() {
                        const wrapperRect = targetDiv.getBoundingClientRect();
                        const centerX = wrapperRect.width / 2;
                        const centerY = wrapperRect.height / 2;
        
                        this.updateTransform(centerX, centerY);
                    }
        
                    updateSVGDimensions() {
                        const svgSize = this.radius * 2; // SVG size based on radius
                        this.svg.setAttribute('width', svgSize);
                        this.svg.setAttribute('height', svgSize);
                        this.svg.setAttribute('viewBox', `0 0 ${svgSize} ${svgSize}`);
                    }
        
                    centerShape() {
                        requestAnimationFrame(() => {
                            const bbox = this.shapeGroup.getBBox();
                            const centerX = bbox.x + bbox.width / 2;
                            const centerY = bbox.y + bbox.height / 2;
        
                            const offsetX = this.cx - centerX;
                            const offsetY = this.cy - centerY;
        
                            this.shapeGroup.setAttribute('transform', `translate(${offsetX}, ${offsetY})`);
                        });
                    }
        
                    startRotation() {
                        const animate = () => {
                            this.rotation = (this.rotation + Number(this.config.cursor.outerShape.rotationSpeed)) % 360; // Avoid large numbers
                            this.circle.style.transform = `rotate(${this.rotation}deg)`;
                            requestAnimationFrame(animate);
                        };
                        animate();
                    }
        
                    generateRandomColors() {
                        const color1 = {
                            h: Math.floor(Math.random() * 360),
                            s: 70,
                            l: 50
                        };
                        const color2 = {
                            h: Math.floor(Math.random() * 360),
                            s: 70,
                            l: 50
                        };
                        return [
                            `hsl(${color1.h}, ${color1.s}%, ${color1.l}%)`,
                            `hsl(${color2.h}, ${color2.s}%, ${color2.l}%)`
                        ];
                    }
        
                    updateCursorPosition(event) {
                        const wrapperRect = targetDiv.getBoundingClientRect();
                        const x = event.clientX - wrapperRect.left;
                        const y = event.clientY - wrapperRect.top;
        
                        this.updateTransform(x, y);
                    }
        
                    updateTransform(x, y) {
                        const cursorTransform = `translate(${x - this.radius}px, ${y - this.radius}px)`;
                        this.cursor.style.transform = cursorTransform;
        
                        if (this.circle) {
                            const circleTransform = `rotate(${this.rotation}deg)`;
                            this.circle.style.transform = circleTransform;
                        }
                    }
        
                    updateGradient() {
                        this.stop1.setAttribute('stop-color', this.interpolateColors(this.gradientColors[0], this.targetColors[0], this.progress));
                        this.stop2.setAttribute('stop-color', this.interpolateColors(this.gradientColors[1], this.targetColors[1], this.progress));
                    }
        
                    interpolateColors(color1, color2, progress) {
                        const hsl1 = this.parseHSL(color1);
                        const hsl2 = this.parseHSL(color2);
        
                        const h = this.interpolateHue(hsl1.h, hsl2.h, progress);
                        const s = hsl1.s + (hsl2.s - hsl1.s) * progress;
                        const l = hsl1.l + (hsl2.l - hsl1.l) * progress;
        
                        return `hsl(${Math.round(h)}, ${Math.round(s)}%, ${Math.round(l)}%)`;
                    }
        
                    parseHSL(hslString) {
                        const values = hslString.match(/hsl\(([^,]+),\s*([^,]+)%,\s*([^)]+)%\)/);
                        if (!values) return { h: 0, s: 0, l: 0 };
                        return {
                            h: parseFloat(values[1]),
                            s: parseFloat(values[2]),
                            l: parseFloat(values[3])
                        };
                    }
        
                    interpolateHue(h1, h2, progress) {
                        const diff = h2 - h1;
                        if (Math.abs(diff) <= 180) {
                            return h1 + diff * progress;
                        }
                        if (diff > 180) {
                            return h1 + (diff - 360) * progress;
                        }
                        return h1 + (diff + 360) * progress;
                    }
        
                    startColorTransition() {
                        const animate = () => {
                            this.progress += Number(this.config.cursor.outerShape.hueSpeed);
        
                            if (this.progress >= 1) {
                                this.progress = 0;
                                this.gradientColors = [...this.targetColors];
                                this.targetColors = this.generateRandomColors();
                            }
        
                            this.updateGradient();
                            requestAnimationFrame(animate);
                        };
        
                        animate();
                    }
        
                    setupEventListeners() {
                        targetDiv.addEventListener('mousemove', (e) => {
                            this.updateCursorPosition(e);
                        });
        
                        targetDiv.addEventListener('mouseleave', () => {
                            this.centerCursor();
                        });
                    }
                }
        
                // Initialize the custom cursor
                const customCursor = new CustomCursor(cursor);
            }
        }
        
    }, [cursor]);

    // Center all cursors
    const centerCursors = () => {
        if (wrapperRef.current) {
            const rect = wrapperRef.current.getBoundingClientRect();
            const centerX = rect.width / 2;
            const centerY = rect.height / 2;

            Object.keys(cursor.cursor).forEach((_, index) => {
                positions.current[index] = {
                    x: centerX,
                    y: centerY,
                };
            });
        }
    };

    // Initialize positions on mount
    useEffect(() => {
        centerCursors();
    }, []);

    // Handle position updates
    useEffect(() => {
        if (!isHovering) {
            return;
        }

        Object.keys(cursor.cursor).forEach((_, index) => {
            const updatePosition = () => {
                const target = position;
                positions.current[index] = {
                    x: target.x,
                    y: target.y,
                };
                animationIds.current[index] = requestAnimationFrame(updatePosition);
            };

            updatePosition();
        });

        return () => {
            animationIds.current.forEach((id) => cancelAnimationFrame(id));
            animationIds.current = [];
        };
    }, [position, cursor.cursor, isHovering]);

    const processStyles = (styles) => {
        // Get a list of valid CSS properties
        const cssPropertyList = new Set(Object.keys(document.createElement('div').style));
        
        // Separate CSS styles and non-CSS styles
        const cssStyles = {};
        const cssVariables = {};
        let textContent = null;
      
        for (const [key, value] of Object.entries(styles)) {
          if (cssPropertyList.has(key)) {
            // It's a valid CSS property
            cssStyles[key] = value;
          } else {
            // Handle special cases
            switch (key) {
              case 'text':
                textContent = value;
                cssVariables[`--${key}`] = `"${value}"`; // Add quotes for CSS
                break;
              case 'textSize':
                cssStyles.fontSize = value;
                break;
              case 'textWeight':
                cssStyles.fontWeight = value;
                break;
              case 'textColor':
                cssStyles.color = value;
                break;
              default:
                // Treat as a custom property, convert to CSS variable
                cssVariables[`--${key}`] = value;
            }
          }
        }
      
        return { cssStyles, cssVariables, textContent };
    };
    const handleMouseLeave = () => {
        // Cancel any ongoing animations
        animationIds.current.forEach((id) => cancelAnimationFrame(id));
        animationIds.current = [];
        
        // Center the cursors
        centerCursors();
    };

    const renderCursorMarkup = () => {
        switch (cursor.id) {
            case 7:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={dionytrisRef}></div>
                );
            case 8:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={aronekoRef}></div>
                );
            case 10:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={hephamageRef}></div>
                );
            case 12:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={hestellipseeRef}></div>
                );
            case 13:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={demetrobyteRef}></div>
                );
            case 14:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={nyxamokeRef}></div>
                );
            case 15:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={gaialoopRef}></div>
                );
            case 16:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={aphorbitRef}></div>
                );
            case 17:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={persephontRef}></div>
                );
            case 18:
                return (
                    <div className={`${cursor.name.toLowerCase()} w-full h-full`} ref={iriadientRef}></div>
                );
            default:
                return (
                    <>
                    {Object.values(cursor.cursor).map((item, index) => {
                        const { cssStyles, cssVariables, textContent } = processStyles(item);
                        
                        const defaultTop = cssStyles.top || '0px';
                        const defaultLeft = cssStyles.left || '0px';

                        return (
                            <div
                            key={index}
                            style={{
                                position: 'absolute',
                                boxSizing: 'border-box',
                                top: cssStyles.height === 'auto' ? defaultTop : `-${parseFloat(cssStyles.height) / 2}px`,
                                left: cssStyles.width === 'auto' ? defaultLeft : `-${parseFloat(cssStyles.width) / 2}px`,
                                pointerEvents: 'none',
                                transform: `translate(${positions.current[index].x}px, ${positions.current[index].y}px)`,
                                transitionDuration: isHovering ? cssStyles.transitionDuration : '0.3s',
                                ...cssStyles,
                                ...cssVariables,
                            }}
                            >
                            {textContent && (
                                <div className="cursor-text">
                                {textContent}
                                </div>
                            )}
                            </div>
                        );
                    })}
                    </>
                );
        }
    };

    return (
        <div className="top-0 left-0 w-full h-full" style={{ position: 'absolute' }} ref={wrapperRef} onMouseLeave={handleMouseLeave}>
            {renderCursorMarkup()}
        </div>
    );
};

export default CursorItem;